#!/usr/bin/env python3
# -*- coding: utf-8 -*-

######################################################################
### PYTHON. APPRENTISSAGE ACTIF (JP. ROY, EDITIONS ELLIPSES, 2020) ###
######################################################################

"""
exo_arbres2_3.py (11.104)
Les arbres 2-3 d'expressions algébriques, avec un "if" ternaire fonctionnel.
On autorise maintenant les opérateurs booléens = < <= > >= binaires !
Donc les feuilles seront des nombres, des variables ou des booléens.
"""

def arbre(r,ag,ad,add=None):
    '''Retourne un arbre de racine r, de fils gauche ag et de fils droit ad.
    Si add est un arbre 2-3, le noeud aura 3 fils au lieu de 2.'''
    if add == None:
        return [r,ag,ad]
    return [r,ag,ad,add]

def est_feuille(a):
    '''Retourne True ssi l'arbre a est réduit à une feuille.'''
    return type(a) in {int, float, str, bool}

def racine(a):
    '''Retourne la racine de l'arbre binaire a, et une erreur si
    a est une feuille.'''
    if est_feuille(a) :
        raise ValueError('Pas de racine pour la feuille {}'.format(a))
    return a[0]

def fg(a):
    '''Retourne le sous-arbre (fils) gauche de l'arbre binaire a,
    ou une erreur si a est une feuille.'''
    if est_feuille(a):
        raise ValueError('Pas de fg pour la feuille {}'.format(a))
    return a[1]

def fd(a):
    '''Retourne le sous-arbre (fils) gauche de l'arbre binaire a,
    ou une erreur si a est une feuille.'''
    if est_feuille(a):
        raise ValueError('Pas de fd pour la feuille {}'.format(a))
    return a[2]

def fdd(a):
    '''Retourne le frère droit du fils droit de l'arbre binaire a,
    ou une erreur si a est une feuille ou un noeud binaire.'''
    if est_feuille(a):
        raise ValueError('Pas de fdd pour la feuille {}'.format(a))
    if est_operateur(racine(a)):
        raise ValueError('Pas de fdd pour un noeud binaire {}'.format(racine(a)))
    return a[3]

def est_operateur(x):
    '''Retourne True ssi x est un opérateur binaire autorisé.'''
    return x in {'+','-','*','/','<','<=','>','>=','='}

if __name__ == '__main__':
    print("Exemple d'arbre 2-3 d'expression algébrique :")
    print()
    atest = ['+',['if',['=','x',2],['-','x',1],['+','x','y']],['/','x',2]]
    print('>>> atest\n-->',atest)
    print('>>> fdd(fg(atest))\n-->',fdd(fg(atest)))
    try:
        print('>>> fdd(atest)\n-->',fdd(atest))      # n'existe pas (erreur) !
    except:
        print("# le fdd de atest n'existe pas !")


