######################################################################
### PYTHON. APPRENTISSAGE ACTIF (JP. ROY, EDITIONS ELLIPSES, 2020) ###
######################################################################

# beads_l    - L'enregistreur RecordToSample produisant un Sample en memoire
# pouvant etre transforme en fichier-disque au format wav (ou mp3, aif, etc).
# Exercice 11.137 l)

# Ce sketch ne comporte ni setup ni draw...
#
# On prepare un son sur ac et un enregistreur sur ac.
# On ecoute le son et on l'enregistre dans un "sample" (morceau) en memoire.
# puis on écoute ce sample en memoire, et on l'enregistre dans un fichier .wav.
# Enfin on charge le fichier .wav en memoire sous la forme d'un sample,
# que l'on ecoute.

add_library('beads')
ac = AudioContext()

def wait(ms):          # ne rien faire pendant ms millisecondes
    t = millis()       # l'heure courante en Processing
    while millis() - t < ms : pass

############ Construction de l'enregistreur rts #########################

# pour fabriquer un morceau de 5 sec en mono à 44.1 kHz
sample = Sample(5000,1,44100)   # vide, son contenu va etre rempli par l'enregistreur rts
# L'enregistreur terminera tout seul (FINITE) après 5 sec de prise de son dans sample
rts = RecordToSample(ac,sample,RecordToSample.Mode.FINITE)
rts.addInput(ac.out)            # on branche la sortie audio aussi sur l'enregistreur
ac.out.addDependent(rts)        # on rajoute rts sur la liste des UGens dépendantes de ac.out

############ Préparation d'un son, voir la question g) ##################

env = Envelope(ac,200)
for i in range(5):
    env.addSegment(800,500)
    env.addSegment(200,500)
env.addSegment(0,300)
wp = WavePlayer(ac,env,Buffer.SINE)     # une sorte de vibrato
g = Gain(ac,1,0.5)
g.addInput(wp)
ac.out.addInput(g)    # le son est pret a etre joue mais ac n'a pas encore demarre !

########### On ecoute l'audio en enregistrant ##########################

println("L'enregistrement de la sortie audio est en cours !")
ac.start()    # ac.out previent ses dependances qu'il demarre
# on attend que l'enregistreur ait termine
wait(6000)

########## On joue le sample enregistre en memoire #####################

print('On joue le sample enregistre en memoire pour verification.')
player = SamplePlayer(ac,sample)
g = Gain(ac,1,0.5)
g.addInput(player)
ac.out.addInput(g)
wait(7000)

########## On sauvegarde le sample dans un fichier-disque ##############

# On peut enregistrer le son sample sur le disque au format WAV notamment (sans perte)
println('On enregistre le son sample dans le fichier-disque "audio_test.wav"')
sample.write('audio_test.wav',AudioFileType.WAV)   # ou MP3, AIF, OGG
# Sur Mac, le fichier est enregistré à la racine de mon compte utilisateur, adaptez...
wait(3000)

######### On ecoute le morceau sur disque ##############################

# Et pour tester, on charge ce fichier-disque en mémoire dans un sample que l'on lit.
print('On charge le fichier-disque "audio_test.wav" et on le joue pour verification.')
sample = Sample('audio_test.wav')    # un nouveau morceau
player = SamplePlayer(ac,sample)     # que l'on fait jouer
g.addInput(player)
wait(6000)

print('Stop')
ac.stop()
    
