#!/usr/bin/env python3
# -*- coding: utf-8 -*-

######################################################################
### PYTHON. APPRENTISSAGE ACTIF (JP. ROY, EDITIONS ELLIPSES, 2020) ###
######################################################################

"""
exo_3d.py (11.128)
Les matrices de rotation 3D autour des axes de coordonnées
Rappel : pour la classe Mat, un vecteur 3D est une liste [a,b,c]
Avec gestion d'une caméra fixe sur Ox et tous les objets sont
derrière l'écran !
On utilise tkinter avec ses classes Canvas et Button.
"""

from exo_mat2 import Mat    # classe des matrices (n,p) sans numpy, voir livre
from math import sin, cos, pi

# --------------------------- FACES-------------------------------

# une figure 3-filaire est une liste de faces triangulaires données
# par 3 points.

def une_pyramide(k):
    '''Retourne sous forme de figure 3-filaire une pyramide reliant
    l'origine O à un point sur chaque axe à distance k de O.'''
    return [[[k,0,0],[0,k,0],[0,0,k]],
            [[k,0,0],[0,0,0],[0,0,k]],
            [[0,0,0],[0,k,0],[0,0,k]]]

# une figure 4-filaire est une liste de faces quadrilatères données
# par 4 points.
    
def un_cube(k):                       
    '''Retourne sous forme de figure 4-filaire le cube dessiné
    dans la figure du livre.'''
    return [[[0,0,k],[k,0,k],[k,k,k],[0,k,k]],
            [[0,0,2*k],[k,0,2*k],[k,k,2*k],[0,k,2*k]], 
            [[0,0,k],[k,0,k],[k,0,2*k],[0,0,2*k]],
            [[0,k,k],[k,k,k],[k,k,2*k],[0,k,2*k]],
            [[0,0,k],[0,k,k],[0,k,2*k],[0,0,2*k]],
            [[k,0,k],[k,k,k],[k,k,2*k],[k,0,2*k]]] 

cube = un_cube(80)
pyramide = une_pyramide(100)
# une figure 4-filaire à 1 face : une plaque à côtés "parallèles"
plaque = [[[80,-80,100],[80,80,100],[-80,80,100],[-80,-80,100]]]

# --------------------------- ROTATIONS -------------------------
# Matrices de rotation autour des axes de coordonnées Ox, Oy, Oz

def mat_rot_z(a):
    '''Retourne la matrice de rotation d'angle a en radians autour de 0z'''
    return Mat([[cos(a), -sin(a),  0],
                [sin(a),  cos(a),  0],
                [  0,       0,     1]])

def mat_rot_y(a):
    '''Retourne la matrice de rotation d'angle a en radians autour de 0y'''
    return Mat([[cos(a),   0, -sin(a)],
                [  0,      1,     0  ],
                [sin(a),   0,  cos(a)]])
    
def mat_rot_x(a):
    '''Retourne la matrice de rotation d'angle a en radians autour de 0x'''
    return Mat([[1,     0,        0   ],
                [0,   cos(a),  -sin(a)],
                [0,   sin(a),  cos(a)]])

rx_pi_sur_16 = mat_rot_x(pi/16)
ry_pi_sur_16 = mat_rot_y(pi/16)
rz_pi_sur_16 = mat_rot_z(pi/16)

def appliquer(m,f):
    '''Prend une matrice m de format 3x3 et une figure n-filaire f,
    et retourne la figure n-filaire obtenue en appliquant m sur chaque
    sommet de f.'''
    def app_face(face):
        return list(map(lambda v: m.app(v),face))
    return list(map(app_face,f))

# --------------- La VUE dans une Interface Graphique -----------------

from tkinter import *

ROOT = Tk()
ROOT.title('Rotation 3D')
ROOT.geometry('+100+100')
TAILLE = 500
CANVAS = Canvas(ROOT,width=TAILLE,height=TAILLE)
CANVAS.pack()

# La caméra fixe est placée au point C et l'écran est le plan d'équation x=xE
# parallèle à yOz. Mieux vaut que les objets soient derrière l'écran car sinon
# ils seront trop déformés. Les objets sont 'devant' nos yeux :-)
(xC,yC,zC) = (400,0,0)  # caméra
E = 100                # et écran : x = E (l'origine O est derrière l'écran)

def camera(x,y,z):                   # transformation par caméra
    '''Retourne le point d'intersection (y',z') de l'écran x=xE et de la droite
    CM joignant la caméra C(xC,0,0) au point M(x,y,z).'''
    # M'(E,y',z') et on résoud M' = C + k*vec(CM) pour calculer k
    k = (E - xC)/(x - xC)
    return (yC + k*(y-yC), zC + k*(z-zC))   # dans le plan d'écran x = E

def dessiner():
    def dessiner_face(face) :
        '''La tortue lea dessine la projection d'une face sur
        le plan y0z (x=0)'''
        for i in range(len(face) - 1):
            (ypi,zpi) = camera(*face[i])
            (ypj,zpj) = camera(*face[i+1])
            CANVAS.create_line(ypi+TAILLE/2,TAILLE/2-zpi,ypj+TAILLE/2,TAILLE/2-zpj)
        (yp0,zp0) = camera(*face[0])
        CANVAS.create_line(ypj+TAILLE/2,TAILLE/2-zpj,yp0+TAILLE/2,TAILLE/2-zp0)
    CANVAS.delete(ALL)
    # dessin des axes Oy et Oz (l'axe Ox est perpendiculaire à l'écran)
    # dessin de la figure
    CANVAS.create_line(0,TAILLE/2,TAILLE,TAILLE/2)   # Oy
    CANVAS.create_line(TAILLE/2,0,TAILLE/2,TAILLE)   # Oz
    for face in figure: 
        dessiner_face(face)

def maj_canvas(transf):
    global figure
    figure = appliquer(transf,figure)
    dessiner()

def rotX(): maj_canvas(rx_pi_sur_16)
def rotY(): maj_canvas(ry_pi_sur_16)
def rotZ(): maj_canvas(rz_pi_sur_16)

def auto():
    global horloge
    if horloge: CANVAS.after_cancel(horloge)
    maj_canvas(rx_pi_sur_16 * ry_pi_sur_16 * rz_pi_sur_16)  # produit matriciel
    horloge = CANVAS.after(100,auto)                        # toutes les 100 msec

def manuel():
    global horloge
    if horloge: 
        CANVAS.after_cancel(horloge)
        horloge = None
    
Button(ROOT,text='RotX',command=rotX).pack(side=LEFT)
Button(ROOT,text='RotY',command=rotY).pack(side=LEFT)
Button(ROOT,text='RotZ',command=rotZ).pack(side=LEFT)
Button(ROOT,text='Suspend',command=manuel).pack(side=RIGHT)
Button(ROOT,text='Auto',command=auto).pack(side=RIGHT)

#figure = pyramide
figure = cube           # <--- choisissez votre figure...
#figure = plaque

horloge = None          # départ en mode manuel
dessiner()
print('Ok')
# Jouez avec, il reste beaucoup d'améliorations à faire dans le GUI...
# En ce qui concerne le modèle, il serait intéressant de jeter un oeil
# aux quaternions pour faire les rotations (et apprendre un joli secteur
# de la programmation des jeux au passage, très peu enseigné par les
# matheux...). On fait dans l'espace avec les quaternions de norme 1
# ce que l'on fait dans le plan avec les nombres complexes de module 1.

ROOT.focus_force()      # l'interface graphique prend la main
ROOT.mainloop()         # pour une exécution au Terminal (sinon "%gui tk"...)

from webbrowser import open as browse
#browse('https://fr.wikipedia.org/wiki/Matrice_de_rotation')
#browse('https://fr.wikipedia.org/wiki/Quaternions_et_rotation_dans_l%27espace')
#browse('http://www.mathoman.com/index.php/1689-une-belle-methode-pour-determiner-la-composee-de-deux-rotations')

