#!/usr/bin/env python3
# -*- coding: utf-8 -*-

######################################################################
### PYTHON. APPRENTISSAGE ACTIF (JP. ROY, EDITIONS ELLIPSES, 2020) ###
######################################################################

"""
exo_ajustement.py (11.109)
Ajustement d'une courbe en électronique ("curve fitting") avec la 
bibliothèque scientifique scipy.
"""

# Notre modèle de base que l'on veut ajuster sera la famille des fonctions
# osc suivantes, fonctions de t à 4 paramètres indépendants a,b,c,d. On
# noterait en maths osc_{a,b,c,d}(t)

def osc(t,a,b,c,d):     # t = temps, a,b,c,d = paramètres indépendants
    return a * np.exp(b*t) * np.cos(c*t + d)

# Préparation des données (normalement elles sortent d'un appareil de 
# mesure)

import numpy as np
t_data = np.linspace(0,8,30)
osc_data = osc(t_data,2.5,-0.4,8,0)  # a=2.5, b=-0.4, c=8, d=0 

print('Les données osc_data sur une courbe osc :')
print(repr(osc_data))
print('et après bruitage :')
osc_data = osc_data + 0.1*np.random.random_sample(size=t_data.size)
print(repr(osc_data))
print()

# Préparation du graphique des données avec matplotlib

import matplotlib.pyplot as plt
plt.plot(t_data,osc_data,'k.')      # le nuage de points

# Ajustement des paramètres a,b,c,d avec curve_fit

from scipy.optimize import curve_fit
(popt, pcov) = curve_fit(osc, t_data, osc_data)    # ajustement !
(a,b,c,d) = popt
print('Après ajustement avec curve_fit :')
print('a={}, b={},\nc={}, d={}'.format(a,b,c,d))

# Affichage lisse de la courbe ajustée, par-dessus les points

t = np.linspace(0,8,300)
plt.plot(t_data,osc_data,'k.')
plt.plot(t,osc(t,a,b,c,d),'r-')
plt.show(block=False)

# COMPLEMENTS EN LIGNE
from webbrowser import open as browse
#browse('https://docs.scipy.org/doc/scipy/reference/generated/scipy.optimize.curve_fit.html')
#browse('https://fr.wikipedia.org/wiki/Ajustement_de_courbe')


