#!/usr/bin/env python3
# -*- coding: utf-8 -*-

######################################################################
### PYTHON. APPRENTISSAGE ACTIF (JP. ROY, EDITIONS ELLIPSES, 2020) ###
######################################################################

"""
exo_analex.py (11.48)
Analyseur lexical d'expressions arithmétiques parenthésées sur les quatre
opérateurs + - * / avec nombres entiers et variables.
On ne gère pas le '-' unaire.
"""

__all__ = ('token')             # pour :         from exo_token import *

from string import digits, ascii_lowercase       # '0123456789' et 'abcdefghijklmnopqrstuvwxyz'

def token_v1():         # le tokenizer résultat
    c = avance()        # le caractère courant
    if not c: return None    # fin de la chaîne !
    if c in ('+', '-', '*', '/') : return ('op',c)
    if c == '(': return ('parg',c)
    if c == ')': return ('pard',c)
    if c in digits : return entier(c)
    if c in ascii_lowercase : return variable(c)
    raise ValueError('Caractère inattendu {}'.format(c))

def entier(c):
    '''Retourne le token entier qui débute au caractère courant c = s[i]'''
    n = 0
    while True:
        n = n * 10 + ord(c) - ord('0')
        c = avance()
        if not c: return ('int',n)
        if c not in digits: recule() ; return ('int',n)

def variable(c):
    '''Retourne le nom de variable qui débute au caractère courant c = s[i]'''
    res = ''
    while True:
        res = res + c
        c = avance()
        if not c: return ('var',res)
        if c not in ascii_lowercase : recule() ; return ('var',res)

def avance():
    '''Lit le caractère courant et avance le pointeur de lecture.'''
    global i            # beurk
    if i >= len(s): return None
    else : c = s[i] ; i = i + 1 ; return c

def recule():
    global i
    i = i - 1

# La présence d'une variable globale i est néfaste, elle m'a obligé à
# nommer j la variable de la boucle for qui suit. Mieux vaut englober
# s comme paramètre et i comme variable locale à token et renvoyer une
# fonction d'arité 0.

def token(s):               # on suppose s bien formée
    i = 0                   # indice du prochain caractère à analyser
    def tok():              # le tokenizer résultat, en tête...
        c = avance()        # le caractère courant
        if not c: return None    # fin de la chaîne !
        if c in ('+', '-', '*', '/') : return ('op',c)
        if c == '(': return ('parg',c)
        if c == ')': return ('pard',c)
        if c in digits : return entier(c)
        if c in ascii_lowercase : return variable(c)
        raise ValueError('Caractère inattendu {}'.format(c))
    def avance():
        '''Lit le caractère courant et avance le pointeur de lecture.'''
        nonlocal i          # et non global !
        if i >= len(s) : return None
        else : c = s[i] ; i = i + 1 ; return c
    def recule():
        nonlocal i          # et non global !
        i = i - 1
    def entier(c):
        '''Retourne le token entier qui débute au caractère courant c = s[i]'''
        n = 0
        while True:
            n = n * 10 + ord(c) - ord('0')
            c = avance()
            if not c: return ('int',n)
            if c not in digits: recule() ; return ('int',n)
    def variable(c):
        '''Retourne le nom de variable qui débute au caractère courant c = s[i]'''
        res = ''
        while True:
            res = res + c
            c = avance()
            if not c: return ('var',res)
            if c not in ascii_lowercase : recule() ; return ('var',res)
    return tok

# Malgré tout, on se doute qu'en augmentant le nombre de types de tokens,
# la tâche sera ingrate... La suite au §8.2 du livre.

if __name__ == '__main__':
    s = '-23+x*(somme-1234)-512' ; i = 0
    print('Analyse lexicale de s =',repr(s))
    print()
    print('--- Première version où s et i sont globales :')
    for j in range(13):
        print('token_v1() -->',token_v1())
    print()
    print('--- Seconde version où i et s sont localisées :')
    del s ; del i               # suppression de s et i en mémoire
    tok = token('-23+x*(somme-1234)-512')       # un "tokenizer"...
    for j in range(13):
        print('tok() -->',tok())                # toc-toc

    # COMPLEMENTS EN LIGNE
    from webbrowser import open as browse
    browse('https://fr.wikipedia.org/wiki/Analyse_lexicale')

