#!/usr/bin/env python3
# -*- coding: utf-8 -*-

######################################################################
### PYTHON. APPRENTISSAGE ACTIF (JP. ROY, EDITIONS ELLIPSES, 2020) ###
######################################################################

"""
exo_arbrebin.py (11.102)
Les arbres binaires d'expressions sur les opérateurs  +  -  *  /
"""

from arbres2 import *
# arbre, est_feuille, racine, fd, fg

# Question a)
def hauteur(a):
    '''Retourne la hauteur de l'arbre binaire a.'''
    if est_feuille(a): return 0
    return 1 + max(hauteur(fg(a)),hauteur(fd(a)))

# Question b)
def feuillage(a):
    '''Retourne la liste des feuille de l'arbre binaire a dans
    un parcours préfixe : racine -> fils gauche -> fils droit.'''
    if est_feuille(a): return [a]
    return feuillage(fg(a)) + feuillage(fd(a))

# Question c)
from operator import __add__,__sub__,__mul__,__truediv__
OPS = {'+':__add__, '-':__sub__, '*':__mul__, '/':__truediv__}

def valeur_arith(a):
    '''Retourne la valeur de l'arbre binaire arithmétique a.
    L'opérateur / sera la division flottante.'''
    if est_feuille(a) : return a
    op = OPS[racine(a)]
    return op(valeur_arith(fg(a)),valeur_arith(fd(a)))

# Question d)
def valeur(a,mem):
    '''Retourne la valeur de l'arbre binaire algébrique a dans une
    mémoire mem. L'opérateur / sera la division flottante.'''
    if est_feuille(a):
        if type(a) in (int,float): return a       # un nombre
        try:
            return mem[a]
        except KeyError:
            raise ValueError('Variable indéfinie {}'.format(a))
    op = OPS[racine(a)]
    return op(valeur(fg(a),mem),valeur(fd(a),mem))

# Question e)
def postfixe(a):
    '''Retourne la liste plate des éléments rencontrés dans
    un parcours postfixe de l'arbre a.'''
    if est_feuille(a): return [a]
    return postfixe(fg(a)) + postfixe(fd(a)) + [racine(a)]

# Question f)
def sous_arbres(a):
    '''Retourne la liste des sous-arbres de l'arbre binaire a.'''
    # Nos arbres sont des tuples non mutables, ils peuvent donc être
    # éléments d'un ensemble.
    if est_feuille(a): return [a]
    return [a] + sous_arbres(fg(a)) + sous_arbres(fd(a))

# Question g)
def subs(a1,a2,a):
    '''Retourne le nouvel arbre obtenu en remplaçant tous les sous-arbres
    a1 de a par l'arbre a2. C'est un "changement de variable"...'''
    if a == a1: return a2
    if est_feuille(a): return a
    return arbre(racine(a),subs(a1,a2,fg(a)),subs(a1,a2,fd(a)))

# Question h)
def tree_english(a):
    '''Returns the english translation of the binary tree a.'''
    if est_feuille(a): return str(a)
    hrg = tree_english(fg(a))
    hrd = tree_english(fd(a))
    r = racine(a)
    if r == '+': return 'the sum of ' + hrg + ' and ' + hrd
    if r == '-': return 'the difference between ' + hrg + ' and ' + hrd
    if r == '*': return 'the product of ' + hrg + ' with ' + hrd
    if r == '/': return 'the quotient of' + hrg + ' by ' + hrd
    raise ValueError('Unknow tree operator : {}'.format(r))    # hum...

# Question i)
# La fonction suivante est plus délicate. Une récurrence brutale ne
# fonctionne pas. Il faut élargir le problème : je pars d'une expression
# qui "débute" par une phrase correcte et qui se termine par... du bruit,
# des éléments qui ne servent à rien. Je retourne un couple (A,B) formé
# de l'arbre A solution du problème et du bruit B non structuré.
# Morale : quand on est en difficulté : GENERALISER LE PROBLEME !
# Attention, les appels récursifs vont dépoter :-)
def english_tree(s):
    OPS = {'sum':'+', 'difference':'-', 'product':'*', 'quotient':'/'}
    def trad(op_name):     # l'opérateur de nom anglais op_name
        try : return OPS[op_name]
        except : raise ValueError('Unknow tree operator {}'.format(op_name))
    def arboriser(L):      # L = [---A---,--B--], on retourne (A,B)
        if L == []:
            raise ValueError('Expression incomplète !')
        if L[0] == 'the':  # je suis en présence d'un noeud
            (Ag,bg) = arboriser(L[3:])      # je tiens le fils gauche !
            (Ad,bd) = arboriser(bg[1:])     # et maintenant le fils droit, fini !
            return (arbre(trad(L[1]),Ag,Ad), bd)
        return (L[0],L[1:])
    def vers_liste(s):     # presque s.split, mais les nombres restent des nombres
        def vers_nombre(s):    # retourne un nombre si s a la syntaxe d'un nombre
            try: s = int(s)
            except:            # ce n'est donc pas un entier
                try: s = float(s)
                except: pass   # ce n'est pas non plus un flottant
            return s
        return list(map(vers_nombre,s.split()))
    # on fait confiance au fait que s est bien formée !
    L = vers_liste(s)           # on préfère travailler sur une liste
    [A,b] = arboriser(L)    # analyse syntaxique par descente récursive
    if b != []:             # y avait-il vraiment du bruit ?
        raise ValueError('Expression trop longue ...{}'.format(' '.join(b)))
    return A


if __name__ == '__main__':
    print("Algorithmes sur les arbres binaires d'expression.")
    print()
    atest = ['+',['+','x',1],['*',2,['+','x',1]]]       # arbre algébrique
    print('>>> atest\n-->',atest)
    print('>>> hauteur(atest)\n-->',hauteur(atest))
    print()
    print('>>> feuillage(atest)\n-->',feuillage(atest))
    atest2 = ['+',['*',2,['-',3,5]],1]                 # arbre arithmétique
    print()
    print('>>> atest2\n-->',atest2)
    print('>>> valeur_arith(atest2)\n-->',valeur_arith(atest2))
    print()
    print(">>> valeur(atest,{'x':4,'y':3})\n-->",valeur(atest,{'x':4,'y':3}))
    print()
    print('>>> postfixe(atest)\n-->',postfixe(atest))
    print()
    print('>>> sous_arbres(atest)   # avec répétitions...\n-->',sous_arbres(atest))
    print()
    print("# Je pose u = x+1 dans atest :")
    print(">>> subs(['+','x',1],'u',atest)\n-->",subs(['+','x',1],'u',atest))
    print()
    print('# Translation of atest in english (no ambiguity with binary ops !) :')
    print('>>> atest\n-->',atest)
    s = tree_english(atest)
    print('>>> s = tree_english(atest)')
    print('>>> s\n-->',repr(s))
    print()
    print('# Traduction en sens inverse (retrouve-t-on atest) :')
    print('>>> english_tree(s)\n-->',english_tree(s))
    print('>>> english_tree(tree_english(atest)) == atest\n-->',
          english_tree(tree_english(atest)) == atest)

    # COMPLEMENTS EN LIGNE
    from webbrowser import open as browse
    #browse('https://fr.wikipedia.org/wiki/Arbre_binaire')



