#!/usr/bin/env python3
# -*- coding: utf-8 -*-

######################################################################
### PYTHON. APPRENTISSAGE ACTIF (JP. ROY, EDITIONS ELLIPSES, 2020) ###
######################################################################

"""
exo_aut2d.py (11.101)
Automates cellulaire 2-D : le Jeu de la Vie de John Conway
"""

# ----------- le modèle de calcul ------------------------------------

# Une population de cellules vivantes est un ensemble W = {(i,j),...}
# d'éléments (i,j) représentant les numéros de ligne et de colonne
# dans une matrice. Ci-dessous (2,3) représente la cellule
# en ligne 2 et colonne 3.

def contenu(cell,W):
    '''Retourne la valeur 0 ou 1 dans W de la cellule cell == (i,j)'''
    return int(cell in W)

def voisins(cell):           # ensemble des 8 voisins d'une cellule cell
    (i,j) = cell
    return {(i-1,j-1), (i-1,j), (i-1,j+1),\
            (i,j-1),            (i,j+1),\
            (i+1,j-1), (i+1,j), (i+1,j+1)}

def somme_voisins(cell,W):   # somme des contenus des voisins de la cellule cell
    return sum(map(lambda c: contenu(c,W),voisins(cell)))

from functools import reduce

def evolution(W):
    res = set()
    # la liste des seules cellules à examiner
    candidats = reduce(lambda e,f: e.union(f),list(map(voisins,W)),set())
    for cell in candidats:
        s = somme_voisins(cell,W)
        if contenu(cell,W) == 1:
            if s in (2,3): res.add(cell)
        elif s == 3: res.add(cell)
    return res

#--------------------- la vue texte -----------------------------------

def afficher_texte(W):
    '''Affiche la population dans une matrice auto-extensible
    vers le bas et vers la droite seulement.'''
    print(W)
    imax = max(map(lambda cell: cell[0],W))
    jmax = max(map(lambda cell: cell[1],W))
    for i in range(imax+2):
        for j in range(jmax+2):
            if contenu((i,j),W) == 1: print('1',end=' ')
            else : print('0',end=' ')
        print()
    print()

#--------------------- la vue graphique -------------------------------

N = 600 / 2         # pour une fenêtre graphique tortue de 600 x 600

from turtle_init import *
# def init(x=0,y=0,cap=0)
# afficher(msg,x,y)
# sleep(dt)            en secondes flottantes
# mode('standard')

def coords(i,j,s):
    '''Retourne la position tortue du centre de la case en ligne i
    et colonne j. Les cases du grillage sont de cote s.'''
    return (-N + s/2 + j*s, N - s/2 - i*s)

def afficher_graphique(W,nbgen=121,d=0,s=15):
    getcanvas().focus_force()    # getcanvas() --> la fenêtre graphique
    ht() ; up()
    for g in range(nbgen):
        tracer(False)
        clear()
        for (i,j) in W:
            goto(coords(i,j,s))
            dot(s-1)
        tracer(True)
        sleep(0.1)
        W = evolution(W)

#--------------------- les tests -------------------------------------

if __name__ == '__main__':
    # la configuration des cellules vivantes figurant dans le livre :
    W = {(1,2),(1,3),(2,2),(2,3),           # bloc de 4
         (1,6),(1,7),                       # couple isolé
#         (2,9),(2,10),(2,11),               # clignotant
         (4,5),(5,6),(6,4),(6,5),(6,6)}     # planeur

    # mode texte :
    for g in range(4):
        print('Génération',g)
        afficher_texte(W)
        W = evolution(W)

    # mode tortue :
    afficher_graphique(W,d=0.1)
    mainloop()

    # Cherchez sur le Web le "canon à planeurs" (glider gun)
    # qui expulse des planeurs (et permet au passage de faire couler de
    # l'information, faisant du Jeu de la Vie un "calculateur universel").

    # COMPLEMENTS EN LIGNE
    from webbrowser import open as browse
    #browse('http://golly.sourceforge.net')                 # un simulateur
    #browse('http://psoup.math.wisc.edu/mcell/index.html')  # un simulateur
    #browse('https://fr.wikipedia.org/wiki/Jeu_de_la_vie')
    #browse('http://deptinfo.unice.fr/%7ejulia/AL/')        # cours et TD 12








