#!/usr/bin/env python3
# -*- coding: utf-8 -*-

######################################################################
### PYTHON. APPRENTISSAGE ACTIF (JP. ROY, EDITIONS ELLIPSES, 2020) ###
######################################################################

"""
exo_bisolve.py (11.33)
Recherche d'une solution approchée par dichotomie de f(x)=0
où f : [a,b] --> R est continue et vérifie f(a) * f(b) < 0
"""

def bisolve(f,a,b,h):           # h gouverne la précision
    '''Retourne une solution approchée de f(x)=0 avec f continue sur [a,b]
    et f(a) * f(b) < 0. La solution m vérifie |f(m)| < h''' 
    if f(a) * f(b) < 0:
        while True:             # stop lorsque f(m) est assez proche de 0
            m = (a + b) / 2
            if abs(f(m)) < h : return m
            if f(a) * f(m) < 0 : b = m
            else : a = m

if __name__ == '__main__':
    from math import sin
    print('Test de bisolve avec h = 0.0001')
    print()
    print('Une solution approchée de x**2 = 2 dans [-5,0] :')
    print(bisolve(lambda x: x*x-2,-5,0,0.0001),'  (exact: -1.4142135623730951)')
    print()
    print('Une solution approchée de x**5-3*x+1 = 0 :')
    print('(on attend -1.38879198440725 ou 0.334734141943353 ou 1.21464804269846)')
    print(bisolve(lambda x: x**5-3*x+1,-2,2,0.0001))
    print()
    print('Une valeur approchée de pi :')
    print(bisolve(lambda x: -sin(x),0.5,4,0.0001),'  (exact: 3.141592653589793)')

    # COMPLEMENTS EN LIGNE
    from webbrowser import open as browse
    #browse('https://fr.wikipedia.org/wiki/Algorithme_de_recherche_d%27un_z%e9ro_d%27une_fonction')
    #browse('https://fr.wikipedia.org/wiki/M%e9thode_de_dichotomie')

    
    

                
    


    

