#!/usr/bin/env python3
# -*- coding: utf-8 -*-

######################################################################
### PYTHON. APPRENTISSAGE ACTIF (JP. ROY, EDITIONS ELLIPSES, 2020) ###
######################################################################

"""
exo_cesar.py (11.42)
La méthode de Jules César pour coder/décoder les messages secrets
"""

def map_string_v1(f,s):
    '''Retourne une nouvelle chaîne obtenue en appliquant la fonction
    f sur chaque caractère de la chaîne s.'''
    res = ''
    for i in range(len(s)):    # pour tous les indices des caractères de s
        res = res + f(s[i])
    return res

# Mais en fait range ici ne sert a rien, puisque la *position* des caracteres
# de s n'intervient pas. On boucle alors directement sur les caracteres :

def map_string(f,s):           # mieux
    '''Retourne une nouvelle chaîne obtenue en appliquant la fonction
    f sur chaque caractère de la chaîne s.'''
    res = ''
    for c in s:
        res = res + f(c)
    return res

# Pour coder une chaîne avec la méthode de Cesar, IL SUFFIT DE SAVOIR CODER UN CARACTERE (le mot
# 'caractère' se dit 'character' en anglais et s'abrévie en 'char'. En Python, il n'existe pas
# de type char ! Les caractères sont simplement des string de longueur 1.

def code_char(c,k):                 # codage du caractère c avec la clé k
    if not c.isupper(): return c    # on ne code que les majuscules !
    pos = ord(c) + k                # poussage brutal de k positions à droite...
    if pos > ord('Z'): pos = pos - 26     # ...et rectification au besoin
    return chr(pos)

# Maintenant j'attaque le codage d'une string, simple application de map_string, plutot
# que rediger la meme boucle que dans map_string. La programmation est un jeu de construction !

def code_césar(msg,k):  
    return map_string(lambda c : code_char(c,k), msg)

# Décodage d'un message secret de cle k. Stratégie identique.

def décode_char(c,k):
    if not c.isupper(): return c 
    pos = ord(c) - k
    if pos < ord('A'): pos = pos + 26
    return chr(pos)

# NB. Il faudrait, une fois la mise au point faite, placer décode_char
# à l'intérieur du texte de décode_césar (la "privatiser")...

def décode_césar(msg,k):
    return map_string(lambda c: décode_char(c,k), msg)

# Décodage en aveugle. Heureusement il n'y a que 25 clés possibles !

def décode_force_brute(msg):
    '''Affiche tous les résultats avec chaque clé. Trouvez la clé !'''
    print('Je dois décoder le message crypté par Jules :')
    print(repr(msg))
    print("J'attaque par force brute :")
    for k in range (1,26):
        print('cle =',k,':',décode_césar(msg,k))
    print('---> Vous voyez la clé ?...')
    # Pour trouver la clé automatiquement il faudrait jouer sur la fréquence
    # des lettres si le message est assez long.

if __name__ == '__main__':
    if map_string(lambda c: c.upper(),'abcd') != 'ABCD' :
        print('Bug dans map_string')
    print('La clé vaut 2. Codage de A :',code_char('A',2))
    print('La clé vaut 3. Codage de Z :',code_char('Z',3))
    print('La clé vaut 4. Codage de b :',code_char('b',4),'  (maj. seulement)')
    print()
    msg = 'ENVOYEZ 36 HOMMES !'
    print('Je code',repr(msg),'avec la clé 3')
    msg_codé = code_césar(msg,3)      # cle k = 3
    print("et j'obtiens :",repr(msg_codé))
    if msg_codé != 'HQYRBHC 36 KRPPHV !' :
        print('Bug dans code_cesar')
    print('Vérification :',repr(décode_césar(msg_codé,3)))
    print()
    décode_force_brute('SMUMF UZSJ SM VAWKWD')
    
    # COMPLEMENTS EN LIGNE
    from webbrowser import open as browse
    #browse('https://fr.wikipedia.org/wiki/Cryptographie')
    #browse('https://www.dcode.fr/analyse-frequences')
    #browse('http://www.dcode.fr/chiffre-cesar')