#!/usr/bin/env python3
# -*- coding: utf-8 -*-

######################################################################
### PYTHON. APPRENTISSAGE ACTIF (JP. ROY, EDITIONS ELLIPSES, 2020) ###
######################################################################

"""
exo_chif1.py (11.17)
Travail arithmétique sur les chiffres d'un entier en base b = 2, 8 ou 16
"""

def somchif_rec(n):     # par récurrence sur n entier >= 0
    '''Retourne la somme des chiffres décimaux de l'entier n.
    Algorithme récursif.'''
    if n < 10: return n
    return n % 10 + somchif_rec(n // 10)

def somchif(n):         # itératif
    '''Retourne la somme des chiffres décimaux de l'entier n'''
    res = 0
    while n > 0:        # épluchage des chifres tant qu'il en reste
        u = n % 10      # le chiffre des unités de n
        res = res + u
        n = n // 10     # pour supprimer le chiffre des unités
    return res

def somchif2_rec(n):    # par récurrence sur n entier >= 0
    '''Retourne la somme des chiffres binaires de l'entier n.
    Algorithme récursif.'''
    if n < 2: return n
    return n % 2 + somchif2_rec(n // 2)
    
def somchif2(n):        # itératif
    '''Retourne la somme des chiffres binaires de l'entier n.
    Algorithme itératif.'''
    res = 0
    while n > 0 :       # épluchage des chifres tant qu'il en reste
        u = n % 2       # le chiffre des unités de n
        res = res + u
        n = n // 2      # pour supprimer le chiffre des unités
    return res

# Mais ceci n'est pas fameux, on écrit deux fois le MEME algorithme.
# En factorisant ce qui est en commun, on voit qu'il suffit de FAIRE
# ABSTRACTION de la base de numération b que l'on passe en argument :

def somchif_base_rec(n, b=10):    # par récurrence sur n entier >= 0
    '''Retourne la somme des chiffres en base b de l'entier n.
    Algorithme récursif.'''
    if n < b: return n
    return n % b + somchif_base_rec(n // b)

def somchif_base(n, b=10):        # itératif
    '''Retourne la somme des chiffres en base b de l'entier n.
    Algorithme itératif.'''
    res = 0
    while n > 0:        # épluchage des chifres tant qu'il en reste
        u = n % b       # le chiffre des unités de n en base b
        res = res + u
        n = n // b      # pour supprimer le chiffre des unités en base b
    return res

# Renversement d'un entier en base b
    
def renverser(n, b=10):       # int --> int, n entier >= 0
    '''Retourne l'entier dont les chiffres décimaux sont ceux de n 
    mais à l'envers. Itératif.'''
    res = 0
    while n > 0 :
        u = n % b 
        res = b * res + u
        n = n // b
    return res


if __name__ == '__main__':
    print('Somme des chiffres de 2021 (récursif) :', somchif_rec(2021))
    print('Somme des chiffres de 2021 (itératif) :', somchif(2021))
    print()
    print('Somme des chiffres binaires de 2021 (récursif) :', somchif2_rec(2021))
    print('Somme des chiffres binaires de 2021 (itératif) :', somchif2(2021))
    print('Vérification : 2021 ==',bin(2021),'en binaire')
    print()
    print("Avec l'algorithme en base b :")
    print('Somme des chiffres de 2021 :',somchif(2021))
    print('Somme des chiffres de 2021 en binaire :',somchif_base(2021,2))
    print('Somme des chiffres de 2021 en hexa :',somchif_base_rec(2021,16))
    print('Vérification : 2021 ==',hex(2021),'en hexadécimal (base 16)')
    print()
    print('Renversement de 20021400 :',renverser(20021400))

    # COMPLEMENTS EN LIGNE
    from webbrowser import open as browse
    #browse('https://fr.wikipedia.org/wiki/Base_(arithmétique)')
    