#!/usr/bin/env python3
# -*- coding: utf-8 -*-

######################################################################
### PYTHON. APPRENTISSAGE ACTIF (JP. ROY, EDITIONS ELLIPSES, 2020) ###
######################################################################

"""
exo_chif2.py (11.40)
Travail textuel sur les chiffres d'un entier
"""

def somchif_v1(n):
    '''Retourne la somme des chiffres décimaux de l'entier n'''
    s = str(n)
    res = 0
    for c in s :            # idiome : pour chaque caractère c de s...
        res = res + int(c)
    return res

def somchif(n):             # le vrai style Python...
    '''Retourne la somme des chiffres décimaux de l'entier n'''
    return sum(int(c) for c in str(n))

# Codage décimal --> binaire

def int2strbin(n):
    '''Retourne l'écriture binaire de n dans une chaîne de caractères'''
    res = ''
    while n > 0:           # on fait une suite de divisions par 2
        res = str(n % 2) + res
        n = n // 2
    return res

def int2strbin_rec(n):     # autre version par récurrence sur n
    '''Retourne l'écriture binaire de n dans une chaîne de caractères'''
    if n < 2 : return str(n)
    return int2strbin_rec(n // 2) + str(n % 2)

def int2strbin_bin(n):     # TRICHEUR : en utilisant la fonction bin de Python :-)
    return bin(n)[2:]      # on enlève les deux premiers caractères '0b'

# Décodage binaire --> décimal

def strbin2int(s):         # decodage du binaire avec "schema de Hörner"
    res = 0
    ord0 = ord('0')        # pour ne pas le recalculer à chaque tour de boucle...
    for c in s:
        res = res * 2 + ord(c) - ord0   # AUCUN CALCUL DE PUISSANCES !!!
    return res

# Renversement d'un entier (et d'une chaîne au passage)

def renverser_chaine(s):
    '''Retourne une copie de la chaîne s, à l'envers'''
    return s[::-1]        # une tranche parcourue à l'envers !!!

def renverser_chaine_v2(s):   # plus classique, bof
    '''Retourne une copie de la chaîne s, à l'envers'''
    res = ''
    for c in s:
        res = c + res
    return res

def renverser(n):             # renversement d'un entier
    '''Retourne l'entier dont les chiffres décimaux sont ceux de n à l'envers'''
    return int(renverser_chaine(str(n)))

def verifier(expr,res):      # petit utilitaire sympa...
    resultat = eval(expr)    # dangeureux dans un programme réel ! Sécurité ?
    if resultat != res : print('Bug dans',repr(expr))
    
if __name__ == '__main__':
    # test de somchif
    verifier('somchif_v1(2021)',5)
    print('>>> somchif(2021)\n{}'.format(somchif(2021)))
    # test de int2strbin
    verifier('int2strbin(55)','110111')
    verifier('int2strbin(256)','100000000')
    print('>>> int2strbin(55)\n{}'.format(repr(int2strbin(55))))
    verifier('int2strbin_rec(55)','110111')
    verifier('int2strbin_rec(256)','100000000')
    verifier('int2strbin_bin(55)','110111')
    verifier('int2strbin_bin(256)','100000000')
    # test de strbin2int
    verifier("strbin2int('100000000')",256)
    verifier("strbin2int('110111')",55)
    print(">>> strbin2int('110111')\n{}".format(strbin2int('110111')))
    # test de renverser_chaine
    verifier("renverser_chaine_v2('abcd efg')",'gfe dcba')
    print(">>> renverser_chaine_v2('abcd efg')\n{}".format(repr(renverser_chaine_v2('abcd efg'))))
    # test de renverser
    verifier('renverser(40210)',1204)
    print('>>> renverser(40210)\n{}'.format(renverser(40210)))
    
    
    
    
    

