#!/usr/bin/env python3
# -*- coding: utf-8 -*-

######################################################################
### PYTHON. APPRENTISSAGE ACTIF (JP. ROY, EDITIONS ELLIPSES, 2020) ###
######################################################################

"""
exo_choice.py (11.60)
Tirage d'un élément au hasard dans une séquence
"""

from random import randint

def choix(s):
    '''Retourne un élément aléatoire de la séquence s non vide'''
    # return random.choice(s)          # en utilisant la primitive choice
    return s[randint(0,len(s)-1)]      # en temps d'accès O(1) à la liste

def cumuler(L):
    '''Retourne la liste des sommes cumulées de la liste de nombres L non vide'''
    # return itertools.accumulate(L)   # en utilisant la primitive accumulate
    res = [L[0]]
    for i in range(1,len(L)):
        res.append(L[i] + res[-1])
    return res

from random import random
from bisect import bisect   # bisect == bisect_left (voir doc en ligne)

def tirage(L,Lp):
    '''Retourne un élément aléatoire de la liste L mais : chaque élément x == L[i]
    a une probabilité p == Lp[i] d'être choisi.'''
    assert sum(Lp) == 1     # une "assertion" comme en Java
    Lc = cumuler(Lp)        # liste des probabilités cumulées
    x = random()            # dans [0,1[
    i = bisect(Lc,x)        # ou une p'tite boucle :-)
    return L[i]

if __name__ == '__main__':
    L = range(100)
    print('Tirage de 20 nombres au hasard entre 0 et 100 :')
    for i in range(20):
        print(choix(L),end=' ')
    print('\n')
    print('10000 fois "tirage([2,3,5],[1/6,1/3,1/2])"')
    p2 = p3 = p5 = 0
    for i in range(10000):
        x = tirage([2,3,5],[1/6,1/3,1/2])
        if x == 2: p2 += 1
        elif x == 3: p3 += 1
        else: p5 += 1
    print('Probabilités obtenues :')
    print('p(2) =',p2/10000,', p(3) =',p3/10000,', p(5) =',p5/10000)
    print('Be happy !')
    
    # COMPLEMENTS EN LIGNE
    from webbrowser import open as browse
    #browse('https://docs.python.org/3/library/itertools.html?highlight=itertools')
    #browse('https://docs.python.org/3/library/random.html')
    #browse('https://docs.python.org/3/library/bisect.html')
    
    
    
        
 
    
    


    





      
