#!/usr/bin/env python3
# -*- coding: utf-8 -*-

######################################################################
### PYTHON. APPRENTISSAGE ACTIF (JP. ROY, EDITIONS ELLIPSES, 2020) ###
######################################################################

"""
exo_combf.py (11.98)
Le type frozenset. Application aux combinaisons dans un ensemble E.
"""

def parties(E):                         # par récurrence sur E
    '''Retourne l'ensemble des parties (frozensets) de l'ensemble E.'''
    # algorithme récursif vu dans exo_ulistes.py
    # On construit un ensemble (set) de frozensets !
    if E == set(): return {frozenset()} # 2**0 == 1
    x = E.pop()                         # mutation de E :-(
    hr = parties(E)                     # hyp. de rec. (E a perdu un élément)
    E.add(x)                            # on le remet en place...
    return hr | {P|{x}  for P in hr}    # P|{x} est un frozenset

def combinaisons(E,p):                  # récurrence sur E et p
    '''Retourne l'ensemble des parties (frozensets) à p éléments de l'ensemble E.'''
    if p == 0: return {frozenset()}     # réduit à la partie vide
    if E == set(): return set()
    # Une combinaison à p éléments de E est soit une combinaison à p éléments 
    # de E - {x}, soit la réunion avec {x} d'une combinaison à p-1 éléments 
    # de E - {x}. Ok ?
    x = E.pop()                 # extraction d'un élément de E, donc ici F == E
    hr1 = combinaisons(E,p)     # hyp. de rec. : E conserve les mêmes éléments !
    hr2 = combinaisons(E,p-1)   #       "            "             "
    E.add(x)                    # pour respecter l'hyp. de rec. !!!
    return hr1 | {P|{x} for P in hr2}

# Le module itertools fournit des outils pour la combinatoire, y-compris
# la fonction combinations(obj,p) qui retourne un itérateur sur les combinaisons
# à p éléments de l'objet itérable obj passé en argument. Voir à la fin des tests.

if __name__ == '__main__':
    E1 = {1,2,3}
    PE1 = parties(E1)
    print('# Il y a',len(PE1),"parties de l'ensemble E1 =",E1,':')
    print('>>> PE1\n-->',PE1)
    print('>>> E1\n-->',E1)
    print()
    print("# On peut convertir PE1 en liste d'ensembles :")
    print('>>> list(map(set,PE1))\n-->',list(map(set,PE1)))
    # morale : une liste de set ou un set de frozensets...
    print()
    E2 = {1,2,3,4}
    print("# Combinaisons à 3 éléments de l'ensemble E2 =",E2)
    print('# (il y en a binomial(4,3) == 4)')
    print('>>> combinaisons(E2,3)\n-->',combinaisons(E2,3))
    print('>>> E2\n-->',E2)
    print()
    from itertools import combinations
    print('# Idem avec la fonction "combinations" du module itertools :')
    print('>>> set(combinations(E2,3))\n-->',set(combinations(E2,3)))
    
    # COMPLEMENTS EN LIGNE POUR LES AMATEURS D'ARITH...
    from webbrowser import open as browse
#    browse('https://docs.python.org/fr/3/library/itertools.html')
#    browse('https://fr.wikiversity.org/wiki/Combinatoire/Exercices/Combinaisons')
#    browse('https://fr.wikipedia.org/wiki/Combinatoire')
#    browse('http://villemin.gerard.free.fr/Wwwgvmm/Compter/Factcomb.htm')
    
    
    
    
