#!/usr/bin/env python3
# -*- coding: utf-8 -*-

######################################################################
### PYTHON. APPRENTISSAGE ACTIF (JP. ROY, EDITIONS ELLIPSES, 2020) ###
######################################################################

"""
exo_cpbb.py  (11.111)
Les chaînes de parenthèses bien balancées. Je ne traite que les
parenthèses '(' et ')', mais on pourrait ajouter les accolades
et les crochets : '(){[]([]){()}}' est bien balancée.
"""

# a) Utilisation d'une pile (c'est essentiellement cela qui empêche une
# solution par expression régulière : il y a de la mémoire quelque part...).
from exo_pile import *

def cpbb_pile(s):
    p = pile_vide()
    for c in s:
        if c == '(':
            empiler(c,p)
        elif c == ')':
            if est_pile_vide(p): return False
            depiler(p)
    return est_pile_vide(p)

# Solution par récurrence (pas d'illusion, la mémoire est dans la pile
# de récursion...).

def cpbb_aux(s,i):       # récurrence sur i
    # In fine, cette fonction auxilliaire devrait être locale à cpbb_rec
    '''Prend une chaîne s telle que s[0:i] soit une cpbb, et recherche la
    plus longue cpbb S débutant à l'indice i de la chaîne s.
    Elle retourne l'indice suivant cette cpbb S.'''
    # s[0:i] est une cpbb ! J'examine ce qui se passe à l'indice i
    if (i == len(s)) or (s[i] == ')'):
        return i
    if s[i] == '(':
        # j'essaye de prolonger la cpbb s[0:i] à droite
        j = cpbb_aux(s,i+1)
        if j ==len(s):    # perdu
            return i-1
        if s[j] == ')':   # j'en ai trouvé une, je continue
            return cpbb_aux(s,j+1)
        return i

def cpbb_rec(s):
    '''Retourne True ssi la chaîne s est une cpbb. Sinon retourne le premier
    indice i fautif (tel que s[0:i] soit la plus grande cpbb débutant s).'''
    k = cpbb_aux(s,0)    # s[0:0] == '' est une cpbb !
    return (k == len(s)) or k

def cpbb_gen(k):
    '''Génère toutes les cpbb c non vides de longueur k > 0 en utilisant :
                  c == '(' + c1 + ')' + c2               '''
    def aux(k):                     # k est pair
        if k == 0:
            yield ''
            return
        for i in range(0,k-1,2):    # i est la longueur (paire) possible de c1
            for c1 in aux(i):
                for c2 in aux(k - i - 2):
                    yield '(' + c1 + ')' + c2
    return aux(k)

if __name__ == '__main__':
    print('Les chaînes de parenthèses "bien balancées".')
    print()
    for i, cpbb in enumerate((cpbb_pile, cpbb_rec)):
        if i == 0 : print('------- cpbb_pile -------')
        else : print('-------- cpbb_rec --------')
        print('()(())()() -->',cpbb('()(())()()'))
        print('()(()()) -->',cpbb('()(()())'))
        print('(((())())()) -->',cpbb('(((())())())'))
        print(')((()))()) -->',cpbb(')((()))())'))
        print('()())() -->',cpbb('()())()'))
        print('()(())))() -->',cpbb('()(())))()'))
        print('(((())())())((( -->',cpbb('(((())())())((('))
        print('((( -->',cpbb('((('))
        print('))) -->',cpbb(')))'))
        print()
    print('-------- cpbb_gen --------')
    from math import factorial
    binomial = lambda n,p: factorial(n) // (factorial(p) * factorial(n-p))
    catalan = lambda n: binomial(2*n,n) // (n + 1)  # les "nombres de Catalan"
    k = 6
    print('Génération des {} cpbb de longueur {} :'.format(catalan(k//2),k))
    gen = cpbb_gen(k)
    while True:
        try: print(next(gen),end='  ')
        except StopIteration: print() ; break
    print('et il y a {} cpbb de longueur 10'.format(sum(1 for c in cpbb_gen(10))))
    print('(le nombre de Catalan C(5) vaut bien {}).'.format(catalan(5)))

    # COMPLEMENTS EN LIGNE
    from webbrowser import open as browse
    #browse('https://fr.wikipedia.org/wiki/Nombre_de_Catalan')
    #browse('https://www.techno-science.net/definition/6068.html')
    #browse('http://www-math.mit.edu/~djk/18.310/18.310F04/parentheses.pdf')










