#!/usr/bin/env python3
# -*- coding: utf-8 -*-

######################################################################
### PYTHON. APPRENTISSAGE ACTIF (JP. ROY, EDITIONS ELLIPSES, 2020) ###
######################################################################

"""
exo_cplfaq.py (11.29)
Etude du temps de calcul de n!
"""

# Les détails mathématiques sont dans le fichier "complexite_fac.pdf"

import gc    # pour un nettoyage manuel de la mémoire (garbage collector)...

# Je n'utilise pas math.factorial car je ne sais pas quel est l'algorithme
# employé, sans doute moins naïf que le suivant (voir la 3ème url donnée en bas).
def fac(n): 
    '''Retourne n! pour n entier >= 0 avec un calcul itératif.'''
    res = 1
    for i in range(2,n+1):
        res = res * i
    return res

for i in range(10) : print(fac(i),end=' ')
print('...')
print()

#from math import factorial as fac      # plus rapide ???

from time import time      # le chronomètre de Python...
gc.collect()               # nettoyage des résidus en mémoire
start = time() ; f1 = fac(50000) ; t1 = time() - start
print('50000! comporte',len(str(f1)),'chiffres.')
print(' Temps t1 de 50000!  : {:.3f} sec'.format(t1))
gc.collect()               # nettoyage des résidus en mémoire
start = time() ; f2 = fac(100000) ; t2 = time() - start
print('100000! comporte',len(str(f2)),'chiffres.')
print(' Temps t2 de 100000! : {:.3f} sec'.format(t2))   # essayez plusieurs fois...
print('t2/t1 = {:.3f}'.format(t2/t1))
print()

# Le temps de calcul est multiplié environ par 4 (ou un poil plus).
# On peut conjecturer qu'il est quasi quadratique -- en O(n**2) -- puisque multiplier
# n par 2 conduit à multiplier le temps de calcul par à peu près 4.

from math import log

def temps(n) :
    return (n*log(n))**2    # avec peut-être une constante cachée quelque part ?...

print('Avec la complexité "théorique" O((n*log(n))**2),')
print('on attendait t2/t1 = {:.3f}'.format(temps(100000)/temps(50000)))

# COMPLEMENTS EN LIGNE
from webbrowser import open as browse
#browse('https://fr.wikipedia.org/wiki/Comparaison_série-intégrale')
#browse('https://mitpress.mit.edu/books/street-fighting-mathematics')
#browse('http://www.luschny.de/math/factorial/FastFactorialFunctions.htm') # oh my god !
