#!/usr/bin/env python3
# -*- coding: utf-8 -*-

######################################################################
### PYTHON. APPRENTISSAGE ACTIF (JP. ROY, EDITIONS ELLIPSES, 2020) ###
######################################################################

"""
exo_cplxgeo.py (11.15)
Nombres complexes et géométrie plane
"""

def parallelogramme(a,b,c) :     # complex x complex x complex --> complex
    '''Retourne d tel que abcd soit un parallelogramme'''
    # on doit avoir vec(ad) = vec(ab) + vec(ac) soit : d - a = (b - a) + (c - a)
    return b + c - a

#-------------

from cmath import exp as cexp     # j'importe l'exponentielle complexe
from math import pi

def rotation(z,a,alpha) :         # complex x complex x float --> complex
    '''Retourne l'image z' de z par la rotation de centre a et d'angle alpha 
    en radians.'''
    # on exprime que vec(az') est l'image de vec(az) par la rotation d'angle 
    # alpha, soit : z' = a + cexp(i*alpha)*z
    return a + cexp(1j * alpha) * (z - a)

#-------------

def prodscal(z1,z2) :             # le produit scalaire en complexe
    return (z1 * z2.conjugate()).real    # conjugate est une méthode de complex

def projection(u,v) :             # complex x complex --> complex
    '''Retourne le complexe w projete orthogonal de u sur v (v != 0)'''
    # on exprime que u-w est orthogonal à v, et que w est colineaire à u :
    # (u-w) . u = 0 et w = k u. Or le produit scalaire z1 . z2 de z1 et z2
    # n'est autre que la partie réelle du produit de z1 par le conjugué de z2.
    # Il est alors immédiat de calculer la constante k.
    ps = prodscal(u,v)
    return ps / (abs(v)**2) * v

# N.B. le test __name__ == '__main__' permet d'importer ce fichier sans
# effectuer les tests.

if __name__ == '__main__' :
    print('Soit a=-1-2j, b = 0 et c = 2-j. Alors d =',end=' ')
    print(parallelogramme(-1-2j,0,2-1j))
    print("Je fais tourner z=3 d'un angle pi/2 autour de 1+j :",end=' ')
    print(rotation(3,1+1j,pi/2))
    print('Produit scalaire de 4+2j et 1+j :',end=' ')
    print(((4+2j) * (1+1j).conjugate()).real)
    print('Projection orthogonale de 4+2j sur 1+j :',projection(4+2j,1+1j))

    # COMPLEMENTS EN LIGNE
    from webbrowser import open as browse
    #browse('https://fr.wikiversity.org/wiki/Approche_géométrique_des_nombres_complexes/Apports_à_la_géométrie')
    
    
