#!/usr/bin/env python3
# -*- coding: utf-8 -*-

######################################################################
### PYTHON. APPRENTISSAGE ACTIF (JP. ROY, EDITIONS ELLIPSES, 2020) ###
######################################################################

"""
exo_cps.py (11.88)
La pile de récursion, le style CPS, la trampoline.
"""

from exo_pile import *
# pile_vide(), est_pile_vide(p), sommet(p), empiler(x,p), depiler(p).

# Question a)

def fac_pile(n):
    '''Retourne n! par dérécursivation de fac avec une pile. Bien
    entendu, on sait le faire ICI sans pile. Ce n'est pas toujours
    le cas. Rappel :   fac(n) == 1 if n == 0 else n * fac(n-1) '''
    # phase 1, on empile (phase de "contrôle" : mises en attente)
    p = pile_vide()
    while n > 0:
        # on empile l'opération en attente 
        empiler(lambda x,y: x*y,p)      # operator.__mul__
        # mais aussi son contexte de calcul
        empiler(n,p)
        # et on passe à l'étape suivante
        n = n - 1
    # case de base : n == 0
    res = 1
    # phase 2, on dépile (calculs) en sens inverse
    while not est_pile_vide(p):
        arg = depiler(p)
        op = depiler(p)
        res = op(res,arg)
    # la pile est vide, plus rien en attente, ciao bella !
    return res

print('Test de la factorielle à pile :')
print('>>> fac_pile(5)\n-->',fac_pile(5))
print()

# Question b) Mise sous forme CPS

def cfac(n,f):              # version CPS de fac
    if n == 0 : return f(1)
    return cfac(n-1,lambda r: f(n*r))

print('Test de la factorielle cfac en style CPS :')
print('>>> cfac(5,lambda r: r)      # f = identité\n-->',cfac(5,lambda r: r))
print('>>> cfac(5,lambda r: 2*r)\n-->',cfac(5,lambda r: 2*r))

# Question c)   Gel de l'appel récursif pour éviter d'empiler.

def cfac_gel(n,f):                  # version gelée de cfac
    if n == 0 : return f(1)
    # on gèle aussi la continuation f
    return lambda : cfac_gel(n-1,lambda r: lambda : f(n*r))

print('>>> cfac_gel(5,lambda r: r)  # oups, tout est gelé !\n-->',cfac_gel(5,lambda r: r))
print()

# Question d) On dégèle avec une trampoline. Un sport qui réchauffe...
# D'après le blog d'Eli Bendersky, April 2017 (eli.thegreenplace.net)

def trampoline(f,*args):           # la trampoline
    v = f(*args)                   # valeur de la fonction
    while callable(v):             # est-ce bien un glaçon ?
        v = v()                    # oui, on dégèle en profondeur...
    return v                       # jusqu'à obtenir une valeur comestible

print('Exécution ITERATIVE de la version cfac_gel par trampoline :')
print('>>> trampoline(cfac,5,lambda r: 2*r)\n-->',
      trampoline(cfac_gel,5,lambda r: 2*r))
print('>>> len(str(trampoline(cfac,20000,lambda r: r)\n-->',
      trampoline(cfac_gel,20000,lambda r: r))     # plus de pile !

# N'en déduisez pas que ce sont des techniques "populaires" de programmation.
# L'exemple n'est là que pour mettre le doigt sur la programmation paresseuse
# qui recèle de grandes richesses théoriques et pratiques... Exemple :
# https://www.apmep.fr/IMG/pdf/16_Roy_Hamming.pdf

# COMPLEMENTS EN LIGNE
from webbrowser import open as browse
#browse('https://zestedesavoir.com/tutoriels/954/notions-de-python-avancees/2-functions/1-callables/')
#browse('http://matt.might.net/articles/by-example-continuation-passing-style/')
#browse('http://2ality.com/2012/06/continuation-passing-style.html')
#browse('https://eli.thegreenplace.net/2017/on-recursion-continuations-and-trampolines/')
#browse('https://en.wikipedia.org/wiki/Continuation-passing_style')
#browse("https://fr.wikipedia.org/wiki/Pile_d'exécution")



