#!/usr/bin/env python3
# -*- coding: utf-8 -*-

######################################################################
### PYTHON. APPRENTISSAGE ACTIF (JP. ROY, EDITIONS ELLIPSES, 2020) ###
######################################################################

"""
exo_deriv.py (11.26)
Dérivation numérique approchée
"""

# Question a)
def deriv_v1(f,a,h=0.001):      # function x float x float --> float
    return (f(a+h) - f(a)) / h # on suppose f dérivable en x=a

# Question b)
def deriv_v2(f,h=0.001):        # function x float --> function
    return lambda x: (f(x+h) - f(x)) / h

# Question c)
# En isolant f'(x) dans le développement d'ordre 2 de f en x, on observe
# que l'erreur est en O(h).
    
# Question d)
def deriv(f,h=0.001):           # version finale symétrique, à exporter
    return lambda x: (f(x+h/2) - f(x-h/2)) / h

# Question d)
# La formule demandée s'obtient en additonnant le développement d'ordre 2
# de f(x+h) et celui de f(x-h). On trouve dans les traités d'Analyse
# Numérique des majorations sur l'erreur commise en O(h**2).
    
def deriv_seconde(f,h=0.001):        # extrapolation en 3 points pour f"
    return lambda x: (f(x+h) - 2*f(x) + f(x-h)) / (h*h)
    
if __name__ == '__main__' :
    from math import log, sin, cos
    f = log         # la fonction pour tester !
    print('Je teste sur la fonction f = log.')
    print('Avec deriv_v1(f,a,h) pour h = 0.001 :')
    print("  f'(2) vaut",deriv_v1(f,2),'   (exact = 0.5)')
    print()
    print("Avec deriv_v2(f,h) pour h = 0.001 :")
    print("  f'(2) vaut",deriv_v2(f)(2),'   (exact = 0.5)')
    print("  f''(2) vaut",deriv_v2(deriv_v2(f))(2),'  (exact = -0.25)')
    print()
    print("Avec deriv(f,h) pour h = 0.001 :")
    print("  f'(2) vaut",deriv(f)(2),'   (exact = 0.5)')
    print("  f''(2) vaut",deriv(deriv(f))(2),'  (exact = -0.25)')
    print()
    print("Avec deriv_seconde, f''(2) vaut",deriv_seconde(f)(2))
    
    # COMPLEMENTS EN LIGNE
    from webbrowser import open as browse
    #browse('https://fr.wikipedia.org/wiki/D%E9riv%E9e')
    #browse('https://www.math.u-bordeaux.fr/~pfischer/Teaching_files/cours.pdf')
    
    
    
    
    

                
    


    

