#!/usr/bin/env python3
# -*- coding: utf-8 -*-

######################################################################
### PYTHON. APPRENTISSAGE ACTIF (JP. ROY, EDITIONS ELLIPSES, 2020) ###
######################################################################

"""
exo_dfreq.py (11.99)
Le dictionnaire des fréquences des éléments d'une collection.
"""

def frequences_v1(s):
    '''Retourne le dictionnaire dont les clés sont les "mots" d'une
    distincts d'une chaîne s, et les valeurs sont les fréquences
    d'apparition de chaque mot dans s.'''
    E = set(s.split())                   # ensemble des mots de s
    return {m:s.count(m) for m in E}

def frequences_v2(s):
    '''Idem frequences_v1, mais on retourne une liste de tuples
    triée par fréquences croissantes.'''
    E = set(s.split())
    return sorted([(m,s.count(m)) for m in E],key=lambda t:t[1])

def plus_frequents(s):
    '''Retourne l'ensemble des mots ayant une fréquence maximale
    d'apparition dans la chaîne s.'''
    freqs = frequences_v2(s)
    print('freqs =',freqs)
    max_freq = freqs[-1][1]
    return set(m for (m,f) in freqs if f == max_freq)


if __name__ == '__main__':
    s = 'do re do mi do si la la mi la'
    print('>>> s\n-->',repr(s))
    print()
    print('>>> frequences_v1(s)\n-->',frequences_v1(s))
    print('>>> frequences_v2(s)\n-->',frequences_v2(s))
    print('>>> plus_frequents(s)\n-->',plus_frequents(s))
    
    
    
