#!/usr/bin/env python3
# -*- coding: utf-8 -*-

######################################################################
### PYTHON. APPRENTISSAGE ACTIF (JP. ROY, EDITIONS ELLIPSES, 2020) ###
######################################################################

"""
exo_div.py (11.16)
La division euclidienne dans les entiers naturels.
"""

def div_rec(a,b):               # par récurrence sur a
    '''Prend deux entiers naturels a >= 0 et b > 0, et retourne un
    couple (q,r) ou q est le quotient et r le reste de a par b qui
    vérifie : a == b*q + r et 0 <= r < b.'''
    if a < b :
        return (0, a)
    (q,r) = div_rec(a - b, b)   # hypothèse de récurrence sur a-b
    return (q + 1, r)


def div(a,b):                   # version itérative
    q = 0
    while a >= b :
        a = a - b
        q = q + 1
    return (q, a)
    
if __name__ == '__main__':
    a = 16 ; b = 5
    (q,r) = div_rec(a,b)
    print('Récurrence : div_rec(16,5) =',(q,r))
    print('En',a,'il y va',q,'fois',b,'et il reste',r)
    print(a,'=',b,'*',q,'+',r,'et 0 <=',r,'<',b)
    print()
    print('Mais en Python, la récurrence est pénalisée par défaut,')
    print('comme le montre le calcul de div_rec(7000,2) :')
    try:      # j'essaye seulement en attrapant une "exception" au vol...
        print('div_rec(7000,2) =',div_rec(7000,2))
    except:   # je le savais... mais j'évite ainsi une erreur fatale
        print('<<< le calcul (en fait la "pile") a explosé ! >>>')
    print()
    
    a = 16 ; b = 5
    (q,r) = div(a,b)
    print('Itération : div(16,5) =',(q,r))
    print('Mais cette fois : div(7000,2) -->',div(7000,2))
    print("car l'itération ne consomme pas d'espace de pile.")
    print()
    
    import sys
    print('Rustine hasardeuse pour un calcul trop récursif :')
    print('Actuellement, sys.getrecursionlimit() ==',sys.getrecursionlimit())
    print('Avec un sys.setrecursionlimit(3200), il vient quand même :')
    sys.setrecursionlimit(3600)         # et jusqu'où peut-on baisser ?...
    print('div_rec(7000,2) -->',div_rec(7000,2))      # ouf.
    
    # COMPLEMENTS EN LIGNE
    from webbrowser import open as browse
    #browse('https://fr.wikipedia.org/wiki/Division_euclidienne')
    #browse('https://docs.python.org/3/library/sys.html#sys.setrecursionlimit')
    
    


