#!/usr/bin/env python3
# -*- coding: utf-8 -*-

######################################################################
### PYTHON. APPRENTISSAGE ACTIF (JP. ROY, EDITIONS ELLIPSES, 2020) ###
######################################################################

"""
exo_erato.py (11.80)
Le crible d'Eratosthene pour construire la liste des
nombres premioers de 2 jusqu'à n.
"""

def crible(n): 
    '''Retourne la liste des entiers premiers de [2,n]'''
    L = list(range(2,n+1))
    for i in range(len(L)):         # pour tous les éléments de L
        if L[i] != 0 :              # si non deja rayé
            j = i + L[i]            # position du premier multiple à droite de i
            while j < len(L):       # je les raye
                L[j] = 0            # ici la "suppression" ne coûte rien !...
                j = j + L[i]
    return [x for x in L if x != 0] # un 'L.pop' aurait coûté plus cher

from math import sqrt

def crible(n):                      # optimisé : on stoppe à sqrt(n)
    '''Retourne la liste des entiers premiers de [2,n]'''
    L = list(range(2,n+1))
    for i in range(int(sqrt(len(L)))):   # pour tous les "petits" éléments de L
        if L[i] != 0:               # si non deja rayé
            j = i + L[i]            # position du premier multiple à droite de i
            while j < len(L):       # je les raye
                L[j] = 0            # ici la "suppression" ne coûte rien !...
                j = j + L[i]
    return [x for x in L if x != 0]

if __name__ == '__main__':
    from time import time
    print('>>> crible(30)\n-->',crible(30))
    print()
    
    n = 100000
    print('>>> n = 100_000')
    start = time() ; LP = crible(100_000) ; end = time()
    print('LP = crible({}) prend {:.2f} sec.'.format(n,end-start))
    NBP = len(LP)
    print('La liste LP contient',NBP,'nombres premiers')
    print('(connu de Thomas Brancker en 1668)')
    print('et son dernier élément est {}.'.format(LP[-1]))
    print()
    
    from random import randint
    print('# Un nombre premier aléatoire < 100_000 :',LP[randint(0,len(LP)-1)])
    print()
    
    print('''Le "Théorème des Nombres Premiers" (Hadamard et 
La Vallée Poussin, 1896) affirme que le nombre NBP d'entiers 
premiers jusqu'à n est "équivalent" au sens de l'analyse 
mathématique à n/ln(n) au voisinage de l'infini.''')
    from math import log
    print('Vérification avec n = 100_000 : NBP/(n/log(n)) = {:.3f} (proche de 1)'.format(NBP/(n/log(n))))
    print()
    print("NB. Le prolongement *difficile* en est l'Hypothèse de Riemann,")
    print("toujours pas démontrée en 2019. Peut-être pour votre")
    print("génération, qui sait ? Croisez les doigts :-)")
    
    # COMPLEMENTS EN LIGNE POUR LES AMATEURS D'ARITH...
    from webbrowser import open as browse
#    browse('http://iml.univ-mrs.fr/editions/publi2005/LaRechercheHS20_Lachaud.pdf')
#    browse('https://fr.wikipedia.org/wiki/Th%e9or%e8me_des_nombres_premiers')
#    browse('https://fr.wikipedia.org/wiki/Crible_d\'Ératosthène')

