#!/usr/bin/env python3
# -*- coding: utf-8 -*-

######################################################################
### PYTHON. APPRENTISSAGE ACTIF (JP. ROY, EDITIONS ELLIPSES, 2020) ###
######################################################################

"""
exo_fibonacci.py (11.24)
Calcul des nombres de Fibonacci
"""

# version par récurrence "enveloppée" (la fonction + est en attente)
def fibrec(n):
    if n < 2: return n
    return fibrec(n-1) + fibrec(n-2)

# la même en plus court
def fibrec(n):
    return n if n < 2 else fibrec(n-1) + fibrec(n-2)

# une autre version récursive mais cette fois "quasi-itérative" : traduisible
# facilement en boucle mais non itérative telle quelle en Python.
def fibquasi(n, a=0, b=1):         
    if n == 0: return a
    return fibquasi(n-1, b, a+b)   # appel récursif terminal

# version itérative obtenue en faisant bouger un couple (a,b) de nombres
# de Fibonacci consécutifs. Une autre vision de fibquasi, mais qui ne
# consomme pas de pile.
def fibiter(n) :
    (a,b) = (0,1)
    i = 0
    while i < n :
        #assert (a == fibquasi(i)) and (b == fibquasi(i+1))    # invariant !
        (a,b) = (b,a + b)
        i = i + 1
        #assert (a == fibquasi(i)) and (b == fibquasi(i+1))    # invariant !
    return a


if __name__ == '__main__' :
    from time import time
    start = time() ; res = fibrec(35) ; end = time()
    print('fibrec(35) = {} en {} sec (très lent)'.format(res,end-start))
    print('fibrec   :',end=' ')
    for i in range(10) : print(fibrec(i),end=' ')
    print('...')
    print()
    start = time() ; res = fibquasi(100) ; end = time()
    print('fibquasi(100) = {} en {:.6f} sec (rapide)'.format(res,end-start))
    print('fibquasi :',end=' ')
    for i in range(10) : print(fibquasi(i),end=' ')
    print('...')
    print('(mais il faut régler sys.setrecursionlimit si n est grand...)')
    print()
    if False :   # True pour seulement comparer les traces, False pour continuer
        from code_trace import trace
        fibrec = trace(fibrec)
        print('\nTrace de fibrec(5) :')
        fibrec(6)
        print()
        fibquasi = trace(fibquasi)
        print('\nTrace de fibquasi(5) :')
        fibquasi(5)
        print()
    else :
        start = time() ; res = fibiter(100) ; end = time()
        print('fibiter(100)  = {} en {:.6f} sec (rapide)'.format(res,end-start))
        print('fibiter  :',end=' ')
        for i in range(10) : print(fibiter(i),end=' ')
        print('...')
        print()
        # La solution exacte de la suite f_n définie ainsi peut se calculer.
        # f_0=0, f_1=1, f_n=f_{n-1} + f_{n-2}   # un peu d'algèbre linéaire...
        print('Suite de Fibonacci avec la formule de Binet flottante :')
        from math import sqrt
        sqrt5 = sqrt(5)
        phi = (1 + sqrt5) / 2      # le nombre d'or
        psi = (1 - sqrt5) / 2      # et son conjugué
        fib_binet = lambda n: (phi**n - psi**n) / sqrt5     # puissance en 0(log n)
        print([fib_binet(k) for k in range(10)])
    
    # COMPLEMENTS EN LIGNE
    from webbrowser import open as browse
    #browse('https://fr.wikipedia.org/wiki/Leonardo_Fibonacci')
    #browse('https://fr.wikipedia.org/wiki/Suite_de_Fibonacci#Expression_fonctionnelle')
    
    