#!/usr/bin/env python3
# -*- coding: utf-8 -*-

######################################################################
### PYTHON. APPRENTISSAGE ACTIF (JP. ROY, EDITIONS ELLIPSES, 2020) ###
######################################################################

"""
exo_filter.py (11.65)
Extraction d'éléments dans une séquence.
"""

def masque(s,L):
    '''Retourne une chaîne de caractères ne contenant que les caractères dont
    les positions dans la chaîne s figurent dans la liste L.'''
    res = ''
    for i in L :
        res += s[i]
    return res

# Inconvénient de la ligne "res += s[i]" : elle coûte len(res), donc l'algorithme
# est en principe quadratique : 1+2+3+4+... Je dis "en principe" car on connaît
# parfois mal les optimisations internes. La version ci-dessous est garantie de
# coût linéaire.

def masque(s,L):
    '''Retourne une chaîne de caractères ne contenant que les caractères dont
    les positions dans la chaîne s figurent dans la liste L.'''
    res = []                # on bascule dans les listes mutables !
    for i in L:
        res.append(s[i])    # coût en O(1)
    return ''.join(res)     # et on revient à la fin dans les chaînes

def filtrer(pred,L):
    return [x for x in L if pred(x)]     # clairement le bon style en Python

if __name__ == '__main__':
    print(">>> masque('azerty',[4,0,3,4,2])\n-->",
          repr(masque('azerty',[4,0,3,4,2])))
    print('>>> filtrer(lambda x : x >= 10,[12,3,9,10,5,17])\n-->',
          filtrer(lambda x : x >= 10,[12,3,9,10,5,17]))
    print()
    print("Exemple d'utilisation de filter :")
    print('>>> f = filter(lambda x : x >= 10,[12,3,9,10,5,17])')
    f = filter(lambda x : x >= 10,[12,3,9,10,5,17])
    print('>>> f          # un itérateur !\n-->',f)
    print('>>> list(f)    # transformation en liste\n-->',list(f))
    print('>>>list(f)     # oups, épuisé !\n-->',list(f))

    # Compléments en ligne
    from webbrowser import open as browse
    #browse('https://thepythonguru.com/python-builtin-functions/filter/')
    #browse('https://docs.python.org/fr/3/library/functions.html#filter')












