#!/usr/bin/env python3
# -*- coding: utf-8 -*-

######################################################################
### PYTHON. APPRENTISSAGE ACTIF (JP. ROY, EDITIONS ELLIPSES, 2020) ###
######################################################################

"""
exo_floatbin.py (11.3)
Ecriture binaire (à la main) des nombres flottants
"""

# float binaire --> float décimal

# Soit à convertir le binaire 1101.011 en décimal.
# On traite d'abord la partie entière 1101 qui produit 8+4+1 = 13
# Donc le résultat sera de la forme 13.xxx...
# On traite ensuite la partie fractionnaire 0.011 en développant
# les inverses des puissances de 2, soit 1/4 + 1/8 = 3/8 = 0.375
# Le résultat final est donc 13.375 décimal.

# Nous avons eu de la chance, car il n'y avait qu'un nombre fini
# de chiffres après la virgule en binaire. Ce n'est pas toujours
# le cas comme nous le verrons avec 0.1 décimal.

# Soit à convertir le décimal 3.625 en binaire.
# La partie entière 3 s'écrit 110 binaire. Le résultat sera donc 110.xxx...
# Traitons 0.625. Puisque nous travaillons non pas avec les puissances
# de 2 mais avec leurs inverses, nous effectuons des mutiplications
# par 2 en lieu et place des divisions par 2 utilisées dans le cas entier.
# Nous récupérons en séquence les chiffres binaires dans les parties
# entières des résultats des multiplications. Et nous recommençons sur la
# partie fractionnaire. 
# 0.625 * 2 = 1.25      récupération du chiffre binaire 1
# 0.25  * 2 = 0.5       récupération du chiffre binaire 0
# 0.5   * 2 = 1.0       récupération du chiffre binaire 1
# 0.0   * 2 ? STOP, l'algorithme s'arrête.
# La partie fractionnaire s'écrit 0.101, le résultat final est 110.101

# Nous avons eu de la chance que le calcul précédent se termine. Les nombres
# flottants qui n'ont qu'un nombre fini de chiffres en binaire sont
# de la forme n/2**k après réduction au même dénominateur... Ce n'est
# pas le cas de 3.525 par exemple. Ni du célèbre 0.1 :

# Ce n'est pas le cas de 0.1 décimal car sinon 0.1 = n/2**k monterait
# qu'il existe k tel que 2**k soit un multiple de 10, ce qui est absurde.
# 0.1 étant rationnel, son développement est périodique, on peut chercher
# sa période, et par chance elle arrive vite.
# 0.1 * 2 = 0.2         récupération du premier chiffre binaire 0
# 0.2 * 2 = 0.4         récupération du chiffre binaire 0  <--
# 0.4 * 2 = 0.8         récupération du chiffre binaire 0     \
# 0.8 * 2 = 1.6         récupération du chiffre binaire 1      \ 
# 0.6 * 2 = 1.2         récupération du chiffre binaire 1       \
# 0.2 * 2 = 0.4         STOP, je suis sur la période ! ----------
# Le nombre 0.1 décimal s'écrit donc 0.0_0011_0011_0011_0011_0011_0011....

print('0.1 ==',0.1)     # 0.1 hum...

# Python nous bluffe car il représente 0.1 en binaire dans la
# machine. Ne pouvant conserver une infinité de chiffres, il va tronquer
# sa représentation binaire. Python 3 tronque au 53ème bit, donc va
# représenter 0.1 sous la forme 0.1 = n/2**k où n est sur 53 bits, donc
# 2**52 < n < 2**53. En manipulant ces deux formules, il vient k = 56.
# et donc n = 0.1 * 2**56 = 7205759403792794. Au final, 0.1 se trouve
# représenté en machine par 7205759403792794/2**56. Le voici :

from decimal import Decimal, getcontext
getcontext().prec = 100    # au plus 100 chiffres significatifs svp !
print('Mais 0.1 est en réalité',Decimal(7205759403792794) / Decimal(2**56))    # Decimal(0.1)
print('et du coup :',end=' ')
print('0.1 + 0.1 + 0.1 == 0.3   --> ',0.1 + 0.1 + 0.1 == 0.3)
print('mais :')
print('0.1 + 0.1 + 0.1 + 0.1 == 0.4   --> ',0.1 + 0.1 + 0.1 + 0.1 == 0.4)
print('Morale : méfiez-vous comme de la PESTE du signe == sur les nombres flottants !!!')



