#!/usr/bin/env python3
# -*- coding: utf-8 -*-

######################################################################
### PYTHON. APPRENTISSAGE ACTIF (JP. ROY, EDITIONS ELLIPSES, 2020) ###
######################################################################

"""
exo_iban.py (11.45)
Vérification de la légalité d'un numéro bancaire IBAN (en janvier 2019)
"""


# Format IBAN en France : FRKK BBBB BGGG GGCC CCCC CCCC CKK
# Les caractères sont tous des lettres ASCII majuscules ou des chiffres romains de 0 à 9
# b = code banque, g = code guichet, c = numéro de compte, K = clef
# http://fr.wikipedia.org/wiki/ISO_13616

from string import ascii_uppercase, digits        # consultez les docs...

def valider_v1(iban):       # str --> bool
    # Uniquement des chiffres et des majuscules ascii, j'enlève les espaces:
    iban = iban.replace(' ','').upper()
    for c in iban:
        if (c not in ascii_uppercase) and (c not in digits) :
            raise ValueError('Caractère IBAN invalide {}'.format(c))
    # Supprimons les 4 premiers caractères et replaçons-les à la fin du compte
    iban = iban[4:] + iban[0:4]
    # Remplacement de chaque lettre par deux chiffres : A --> 10, B --> 11, etc
    # On commence par récupérer la chaîne des lettres distinctes
    lettres = ''         
    for c in iban:
        if c.isalpha() and (c not in lettres) : lettres += c
    # puis on opère les substitutions :
    for c in lettres:
        iban = iban.replace(c,str(ord(c)-ord('A')+10))
    # On transforme iban en nombre entier
    iban = int(iban)
    print('iban final numérique =',iban)
    # et on teste le reste de la division par 97
    contrôle = iban % 97
    if contrôle == 1: print('IBAN valide')
    else : print('contrôle =',contrôle,': IBAN invalide')

# Le code ci-dessus pourrait s'alléger si l'on connaît les listes et les
# dictionnaires :

def valider_v2(iban):       # str --> bool
    # Uniquement des chiffres et des majuscules ascii, j'enlève les espaces:
    iban = iban.replace(' ','').upper()
    for c in iban:
        if (c not in ascii_uppercase) and (c not in digits) :
            raise ValueError('Caractère IBAN invalide "{}"'.format(c))
    # Supprimons les 4 premiers caractères et replaçons-les à la fin du compte
    iban = iban[4:] + iban[0:4]
    # Remplacement de chaque lettre par deux chiffres : A --> 10, B --> 11, etc
    # On commence par construire le dictionnaire de substitution :
    subst = {chr(ord('A')+i):str(10+i) for i in range(0,26)}
    # puis on opère les substitutions :
    for c in subst : iban = iban.replace(c,subst[c])
    # On transforme iban en nombre entier
    iban = int(iban)
    print('iban final numérique =',iban)
    # et on teste le reste de la division par 97
    contrôle = iban % 97
    if contrôle == 1: print('IBAN valide')
    else : print('contrôle =',contrôle,': IBAN invalide')
    
if __name__ == '__main__':
    print('Votre IBAN ou <return> :')
    iban_ch = input('>>> ')
    if iban_ch == '': 
        iban_ch = 'CH10002300A1023502601'
        print(iban_ch)
    print("Vérification de l'IBAN :")
    print()
    print('Technique basique :')
    valider_v1(iban_ch)
    print()
    print('Technique avec dictionnaires :')
    valider_v2(iban_ch)
    
    # COMPLEMENTS EN LIGNE
    from webbrowser import open as browse
    #browse('https://www.iban.fr/verification.html')
    #browse('https://fr.wikipedia.org/wiki/International_Bank_Account_Number')



