#!/usr/bin/env python3
# -*- coding: utf-8 -*-

######################################################################
### PYTHON. APPRENTISSAGE ACTIF (JP. ROY, EDITIONS ELLIPSES, 2020) ###
######################################################################

"""
exo_integ1.py (11.28)
Intégration numérique approchée (Riemann)
"""

def riemann(f,a,b,n):            # a=x_0 < x_1 < ... < x_n=b
    h = (b - a) / n              # n intervalles de largeur h
    return h * sum(f(a + i*h) for i in range(n))

def simpson(f,a,b,n):            # a=x_0 < x_1 < ... < x_n=b, avec n pair
    h = (b - a) / n              # un nombre pair n d'intervalles de largeur h
    somme4 = sum(f(a+i*h) for i in range(1,n,2))
    somme2 = sum(f(a+i*h) for i in range(2,n-1,2))
    return h/3 * (f(a) + 4*somme4 + 2*somme2 + f(b))

# On ramène dans les deux cas le calcul numérique d'une intégrale à une
# somme pondérée de valeurs de la fonction sur un nombre fini de points. Le
# but est de minimiser le nombre de points en conservant une bonne précision.

if __name__ == '__main__':
    from math import sin, cos, sqrt, pi
    print('--- METHODE DE RIEMANN avec 500 points')
    print('Aire du sinus entre 0 et pi :')
    print(riemann(sin,0,pi,500),'    (exact: 2)')
    print('Valeur moyenne de cos(x) sur [0,pi/2] :')
    # rappel : moyenne(f) sur [a,b] = integrale(f,a,b,n) / (b-a)
    print(riemann(cos,0,pi/2,500) * 2 / pi,'   (exact: 0.6366197723675814)')  
    print()
    print('--- METHODE DE SIMPSON avec 32 intervalles (donc 33 points)')
    print('Aire du sinus entre 0 et pi :')
    print(simpson(sin,0,pi,32),'   (exact: 2)')
    print('Valeur moyenne de cos(x) sur [0,pi/2] :')
    # rappel : moyenne(f) sur [a,b] = integrale(f,a,b,n) / (b-a)
    print(simpson(cos,0,pi/2,32) * 2 / pi,'   (exact: 0.6366197723675814)')  
    
    # COMPLEMENT : AVEC SCIPY
    print('Le sinus avec scipy/simpson :')
    from scipy.integrate import simps as simpson
    import numpy as np
    N = 32; a = 0; b = pi;       # avec 32 intervalles
    x = np.linspace(a,b,N+1)     # soit 33 points
    y = np.sin(x)
    print(simpson(y,x))

    # COMPLEMENTS EN LIGNE
    from webbrowser import open as browse
#    browse('https://fr.wikipedia.org/wiki/Int%e9grale_de_Riemann')
#    browse('https://fr.wikipedia.org/wiki/M%e9thode_de_Simpson')
#    browse('https://www.courspython.com/integration-simpson.html')
#    browse('https://fr.wikipedia.org/wiki/Calcul_num%e9rique_d%27une_int%e9grale')   # Simpson !
#    browse('https://fr.wikipedia.org/wiki/Algorithme_de_Risch')
    
    
    

                
    


    

