#!/usr/bin/env python3
# -*- coding: utf-8 -*-

######################################################################
### PYTHON. APPRENTISSAGE ACTIF (JP. ROY, EDITIONS ELLIPSES, 2020) ###
######################################################################

"""
exo_integ2.py (11.31)
Intégrale double avec Fubini ou Riemann
"""

def integrale2_riemann(f,a,b,c,d,nx,ny):
    '''Retourne l'intégrale double de la fonction f(x,y) pour
    x dans [a,b] et y dans [c,d], l'intervalle [a,b] étant découpé
    en nx sous-intervalles, et [c,d] en ny sous-intervalles.'''
    hx = (b - a) / nx
    hy = (d - c) / ny
    return hx*hy*sum(f(a+i*hx,c+j*hy) for i in range(nx) for j in range(ny))


#from exo_integ1 import riemann as integrale    # integrale(f,a,b,n)
from exo_integ1 import simpson as integrale    # integrale(f,a,b,n), n pair

def integrale2_fubini(f,a,b,c,d,nx,ny):
    '''Retourne l'intégrale double de la fonction f(x,y) pour
    x dans [a,b] et y dans [c,d], l'intervalle [a,b] étant découpé
    en nx sous-intervalles, et [c,d] en ny sous-intervalles.'''
    def fx(x):
        return integrale(lambda y: f(x,y),c,d,ny)
    return integrale(fx,a,b,nx)

# On généralise à un domaine plan non rectangulaire mais compris entre deux
# courbes. Donc c et d deviennent des fonctions c(x) et d(x).
# Ex: integrale2_fubini(lambda x,y: x*y,0,1,lambda x:x,lambda x:x*x,32,32)
    
def integrale2_fubini(f,a,b,c,d,nx,ny):       # région non rectangulaire
    '''Retourne l'intégrale double de la fonction f(x,y) pour
    x dans [a,b] et pour x donné, y dans [c(x),d(x)], l'intervalle [a,b] 
    étant découpé en nx sous-intervalles, et [c(x),d(x)] en ny sous-intervalles.'''
    def to_func(obj):
        if type(obj) in {int,float}: 
            return lambda x: obj
        else:
            return obj     # déjà une fonction
    c = to_func(c) ; d = to_func(d)
    def fx(x):
        return integrale(lambda y: f(x,y),c(x),d(x),ny)
    return integrale(fx,a,b,nx)
    
if __name__ == '__main__':
    print("Calculs de l'intégrale double approchée")
    print('de f(x,y) = x**2-3*x*y sur [0,1] x [-1,1]')
    print('On attend 2/3 = 0.666666...')
    print()
    print('- directement avec la formule de Riemann :')
    print(integrale2_riemann(lambda x,y: x*x-3*x*y,0,1,-1,1,500,1000),'  (500_000 points)')
    print('- avec la formule de Fubini, et Simpson en avant-garde :')
    print(integrale2_fubini(lambda x,y: x*x-3*x*y,0,1,-1,1,50,50),'  (2500 points)')
    # La page web ci-dessous montre comment obtenir directement une formule
    # de quadrature de Simpson en dimension 2, en intégrant la formule de
    # Simpson en dimension 1.
    
    # COMPLEMENTS EN LIGNE
    from webbrowser import open as browse
    #browse('http://banach.millersville.edu/~bob/math375/DoubleInt/main.pdf')
    

