#!/usr/bin/env python3
# -*- coding: utf-8 -*-

######################################################################
### PYTHON. APPRENTISSAGE ACTIF (JP. ROY, EDITIONS ELLIPSES, 2020) ###
######################################################################

"""
exo_lagrange.py (11.91)
Le polynôme d'interpolation de Lagrange.
"""

from exo_polycreux import *       # poly0() et poly_xxxxxx()

def lagrange(Lx,i):
    '''Retourne le polynôme fondamental de Lagrange d'indice i pour
    la liste d'abscisses toutes distinctes Lx.'''
    xi = Lx[i]
    res = [[1,0]]
    for j in range(len(Lx)):
        if j == i: continue
        xj = Lx[j]
        res = poly_mul(res,poly_ext(1/(xi-xj),[[1,1],[-xj,0]]))
    return res

from functools import reduce

def interpol(Lpts):
    '''Retourne le polynome d'interpolation de Lagrange pour la liste de points
    du plan Lpts=[(x,y),...] dont toutes les abscisses sont distinctes.'''
    Lx = list(map(lambda pt : pt[0],Lpts))        # liste des abscisses distinctes
    Ly = list(map(lambda pt : pt[1],Lpts))        # liste des ordonnées quelconques
    LLi = list(map(lambda i : lagrange(Lx,i),range(len(Lx))))
    #print('Lx =',Lx,'et\nLLi=',LLi)
    res = poly0()
    for i in range(len(Lpts)):
        res = poly_sum(res,poly_ext(Ly[i],lagrange(Lx,i)))
    return res

from exo_legendre import poly_integ

def newton_cotes_integrale(f,a,b,n):
    '''Retourne l'integrale approchee de f sur [a,b] en interpolant f aux extremites d'un
    partage de [a,b] en n segments egaux, par un polynome d'interpolation de Lagrange.'''
    h = (b-a) / n
    p = interpol([(a+i*h,f(a+i*h)) for i in range(n+1)])      # points extrêmes a et b
    ip = poly_integ(p)                                      # une primitive de p
    fip = poly_fonc(ip)
    return fip(b) - fip(a)
    
if __name__ == '__main__':
    print("# Polynôme d'interpolation de Lagrange")
    print("# On s'intéresse aux abscisses 2, 3, 5.")
    print('>>> Lx = [2,3,5]')
    Lx = [2,3,5]
    print()
    
    print('# Test du polynôme fondamental L_0 pour Lx :')
    print('>>> L_0 = lagrange(Lx,0)')
    L_0 = lagrange([2,3,5],0)           # polynôme fondamental d'indice i=0 pour Lx=[2,3,5]
    print('>>> L_0\n-->',L_0)
    print('# et sa fonction polynôme associée :')
    f0 = poly_fonc(L_0)
    print('>>> f0 = poly_fonc(L_0)')
    print('# dont la valeur en 2, 3, 5 est :')
    print('>>> (f0(2), f0(3), f0(5))\n-->',(f0(2),f0(3),f0(5)))
    print()
    
    print("Test du polynôme d'interpolation passant par 3 points :")
    print('>>> q = interpol([(3,-2),(5,0),(0,4)])')
    q = interpol([(3,-2),(5,0),(0,4)])
    print('>>> q\n-->',q)
    print('>>> f = poly_fonc(q)')
    f = poly_fonc(q)
    print('>>> (f(3), f(5), f(0))\n',(f(3),f(5),f(0)))    # OK ?
    print()

    from math import sin, sqrt, pi, log
    print('Calcul par la méthode de Newton-Cotes avec 9 POINTS')
    print("de l'intégrale de f(x) = sin(x) sur [0,pi]")
    print('>>> newton_cotes_integrale(sin,0,pi,8)                # 2\n-->',
        newton_cotes_integrale(sin,0,pi,8))
    print('>>> newton_cotes_integrale(lambda x : 1/(1+x),0,1,8)  # log(2)\n-->',
          newton_cotes_integrale(lambda x : 1/(1+x),0,1,8))
    print('>>> log(2)\n-->',log(2))
    
    # illustration graphique de la courbe de q avec matplotlib
    poly_plot(q,-2,7,Lpts=[(3,-2),(5,0),(0,4)])

    # COMPLEMENTS EN LIGNE
    from webbrowser import open as browse
#    browse('https://www.maths-france.fr/MathSpe/GrandsClassiquesDeConcours/Polynomes/PolynomesLagrange.pdf')
#    browse('https://fr.wikipedia.org/wiki/Interpolation_lagrangienne')
    
    
