#!/usr/bin/env python3
# -*- coding: utf-8 -*-

######################################################################
### PYTHON. APPRENTISSAGE ACTIF (JP. ROY, EDITIONS ELLIPSES, 2020) ###
######################################################################

"""
exo_legendre.py (11.90)
Définition récursive des polynômes de Legendre
et test de leur orthogonalité deux à deux.
"""

from exo_polycreux import *
# poly_ext(k,p), poly_mul(p,q), poly_integ(p), poly_val(p,x)

def legendre(n):
    '''Retourne le polynôme de Legendre L_n, avec coefficients flottants.'''
    if n == 0: return [[1,0]]
    X = [[1,1]]
    if n == 1: return X
    return poly_ext(1/n,poly_sub(poly_ext(2*n-1,poly_mul(X,legendre(n-1))),
                                 poly_ext(n-1,legendre(n-2))))

def poly_integ(p):
    '''Retourne un polynôme creux dont le polynôme dérivé est p.'''
    return list(map(lambda m: [m[0]/(m[1]+1),m[1]+1],p))

if __name__ == '__main__':
    print('Les polynômes sont exprimés de manière CREUSE !')
    for i in range(2,6):
        print('legendre({}) = {}'.format(i,legendre(i)))
    print()

    # En fait on peut montrer que les coeffs des polynômes de Legendre
    # sont des rationnels.
    print('Calcul avec coeffs rationnels exacts.')
    from fractions import Fraction
    for i in range(2,6):
        Li = list(map(lambda m: [Fraction(m[0]),m[1]],legendre(i)))
        print('legendre_exact({}) = {}'.format(i,Li))
    print()

    print("Vérification de l'orthogonalité de L_n et L_m si n != m.")
    from random import randint
    (n,m) = (randint(0,6),randint(0,6))
    print("Test d'orthogonalité avec (n,m) =",(n,m))
    produit = poly_mul(legendre(n),legendre(m))
    print('Le produit L_{} * L_{} est le polynôme :\n'.format(n,m),produit)
    integrale = poly_integ(produit)
    print('dont une primitive est le polynôme :\n',integrale)
    test = poly_val(integrale,1) - poly_val(integrale,-1)
    print("dont l'intégrale sur [-1,1] vaut :",test)
    if test == 0: print('donc L_{} et L_{} sont orthogonaux'.format(n,m))
    else: print('ainsi L_{} et L_{} ne sont pas orthogonaux !'.format(n,m))

    # COMPLEMENTS EN LIGNE
    from webbrowser import open as browse
    #browse('http://www.uvt.rnu.tn/resources-uvt/cours/analyse-hilbertienne/bases-hilbertienne/chap2/sec4/node1.html')
    #browse('https://fr.wikipedia.org/wiki/Polynôme_de_Legendre')
