#!/usr/bin/env python3
# -*- coding: utf-8 -*-

######################################################################
### PYTHON. APPRENTISSAGE ACTIF (JP. ROY, EDITIONS ELLIPSES, 2020) ###
######################################################################

"""
exo_lex1.py (11.112)
Illustration de SLY, logiciel Lex/Yacc pour Python de D. Beazley
Analyseur lexical pour des expressions arithmétiques sur les
Opérateurs + - * / ** avec des nombres entiers et flottants et
des variables.
Ceci est la SOLUTION de l'exercice "Fin de l'analyseur lexical avec
lex" du §11.112, le fichier à compléter est 'exo_lex1_student.py'.
"""

from sly import Lexer

class LexerError(Exception):    # juste pour la lisibilité des exceptions
    pass

class ArithLexer(Lexer):
    # j'ajoute VAR, FLOAT et PUISS
    tokens = {VAR, FLOAT, INT, ADD, SUB, PUIS, MUL, DIV, PARG, PARD}
    # les différents types de tokens avec leur expression régulière
    VAR   = '[a-zA-Z][a-zA-Z0-9]*'
    FLOAT = '[0-9]+[.][0-9]+'
    INT   = '[0-9]+'
    ADD   = '\+'          # ou '[+]'
    SUB   = '-'
    PUIS  = '\*\*'        # ou '[*][*]'   et AVANT MUL (essayez de le placer après)
    MUL   = '\*'          # ou '[*]'
    DIV   = '/'
    PARG  = '\('         # ou '[(]'
    PARD  = '\)'         # ou '[)]'
    # la chaîne contenant les caractères à ignorer : espace et tabulation
    ignore = ' \t'

    def __init__(self):
        self.vars = set()         # l'ensemble des variables reconnues

    def VAR(self,t):              # une action à la lecture d'une variable
        self.vars.add(t.value)    # exemple d'action : stockage dans un ensemble
        return t

    def FLOAT(self, t):           # une action à la lecture d'un token t de type FLOAT
        t.value = float(t.value)
        return t

    def INT(self, t):             # une action à la lecture d'un token t de type INT
        t.value = int(t.value)
        return t

    def error(self,t):           # je suis sur un caractère provoquant une erreur
        msg = 'ERREUR sur "{}" en ligne {} colonne {}'.format(t.value[0],t.lineno,t.index)
        raise LexerError(msg)

if __name__ == '__main__':
    lexer = ArithLexer()
    def test_lexer(s):           # une petite fonction de test
        lexer.vars = set()
        print('--- chaîne =',repr(s))
        tok = lexer.tokenize(s)       # tok est un itérateur sur les tokens
        for t in tok: print((t.type,t.value))
        print('vars =',lexer.vars)
        print()

    test_lexer('-205 + 3*(8-255)-   412')
    test_lexer('12.567*()**2')   # lexicalement correct...
    test_lexer('3.5h12L24-n8')
    try:
        test_lexer('18**3.5+n.2')    # oups ! Le point n'est légal que dans un FLOAT
    except LexerError as e:
        print(e)
    print()

    def test_interactif():
        print('Entrez des expressions, .fini pour quitter !')
        while True:
            lexer.vars = set()
            s = input('? ')
            if s.lower() == '.fini': break
            tok = lexer.tokenize(s)
            for t in tok: print((t.type,t.value))
            print('vars =',lexer.vars)
            print()

    test_interactif()

    from webbrowser import open as browse
    # Les regexp...
    #browse('https://docs.python.org/fr/3/library/re.html')
    
    # Attention, le lien suivant pour PLY et non SLY, mais on apprend des choses...
    #browse('http://www.matthieuamiguet.ch/pages/compilateurs')
    
    # La vidéo YouTube de Dave à la conférence PYCON 2018
    #browse('https://www.youtube.com/watch?v=zJ9z6Ge-vXs')
    
    # La doc de SLY
    #browse('https://sly.readthedocs.io/en/latest/index.html')

