#!/usr/bin/env python3
# -*- coding: utf-8 -*-

######################################################################
### PYTHON. APPRENTISSAGE ACTIF (JP. ROY, EDITIONS ELLIPSES, 2020) ###
######################################################################

"""
exo_longueur.py (11.30)
Rectification d'un arc paramétré x=f(t), y=g(t), t dans [a,b]
On suppose f et g dérivables.
"""

# L'élément d'arc ds est assimilé à l'hypoténuse d'un triangle rectangle
# de côtés dx et dy, soit ds**2 = dx**2 + dy**2.
# Mais x=f(t) ==> dx = f'(t) dt. De même dy = g'(t) dt
# D'où ds**2 = (f'(t)**2 + g'(t)**2) dt et donc :
#                 ds = sqrt(f'(t)**2 + g'(t)**2) * dt
# Pour calculer la longueur L de l'arc, on intègre (on "somme") tous ces
# petits éléments d'arcs le long de la courbe.

from exo_integ1 import riemann as integrale      # integrale(f,a,b,n) avec n points
from exo_deriv import deriv             # deriv(f,h=0.001) calcule f'

from math import sqrt, cos, sin, pi

def longueur(f,g,a,b,n):               # découpage du temps en n morceaux
    h = (b - a) / n
    df = deriv(f,h)
    dg = deriv(g,h)
    return integrale(lambda t: sqrt(df(t)**2 + dg(t)**2),a,b,n)

# Le cercle de rayon 1 est un arc paramétré x=cos(t), y=sin(t) sur [0,2*pi]
longueur_cercle = longueur(cos,sin,0,2*pi,1000)    # avec 1000 points
print("Longueur d'un cercle de rayon 1 : {:.6f}".format(longueur_cercle))
print('(on attendait 2*pi = {:.6f})'.format(2*pi))
print()

print("Longueur d'une ellipse de demi-diamètres 3 et 2 :",end=' ')
# arc paramétré x=3*cos(t), y=2*sin(t) pour t dans [0,2*pi]
longueur_ellipse = longueur(lambda t: 3*cos(t),lambda t: 2*sin(t),0,2*pi,1000)
print('{:.4f}'.format(longueur_ellipse))
# En Mathematica : ArcLength[Circle[{x, y}, {2, 3}]] // N --> 15.8654
print('(on attendait 15.8654 avec Mathematica)')

from webbrowser import open as browse
#browse('https://fr.wikipedia.org/wiki/Longueur_d%27un_arc')
#browse('https://fr.wikipedia.org/wiki/Abscisse_curviligne')
#browse('https://www.khanacademy.org/math/ap-calculus-bc/bc-applications-of-integration-new/bc-8-13/v/arc-length-formula')
