#!/usr/bin/env python3
# -*- coding: utf-8 -*-

######################################################################
### PYTHON. APPRENTISSAGE ACTIF (JP. ROY, EDITIONS ELLIPSES, 2020) ###
######################################################################

"""
exo_map_reduce.py (11.69)
Les fonctionnelles préférées de Google ;-)
"""

def appliquer_v1(f,L):                 # one-liner, le bon style...
    return [f(x) for x in L]           # une "compréhension de liste"

def appliquer_v2(f,L):                 # itération classique, bof
    res = []
    for x in L:
        res.append(f(x))
    return res

def reduire(f,L,e=None):               # L non vide
    '''Retourne f(f(f(a,b),c),d) si L = [a,b,c,d] par exemple.'''
    if e == None : acc = L[0] ; start = 1
    else : acc = e ; start = 0
    for i in range(start,len(L)):
        acc = f(acc,L[i])
    return acc

def appliquer_v3(f,L):    # appliquer (map) vue comme une réduction (reduce)...
    return reduire(lambda acc,x: acc+[f(x)],L,[])

if __name__ == '__main__':
    if appliquer_v1(lambda x : 2*x,[3,7,2,4,1]) != [6,14,4,8,2]:
        raise ValueError('Bug dans appliquer_v1')
    if appliquer_v2(lambda x : 2*x,[3,7,2,4,1]) != [6,14,4,8,2]:
        raise ValueError('Bug dans appliquer_v2')
    if reduire(lambda acc,x : 2*acc + x,[1,2,3,4]) != 26:          # vérifiez à la main !
        raise ValueError('Bug dans reduire')
    if reduire(lambda acc,x : [x]+acc,[1,2,3,4],[]) != [4,3,2,1]:  # vérifiez à la main !
        raise ValueError('Bug dans reduire')
    if appliquer_v3(lambda x : 2*x,[3,7,2,4,1]) != [6,14,4,8,2]:
        raise ValueError('Bug dans appliquer_v3')

    # appliquer --> map    en Python
    # reduire   --> reduce en Python (dans le module functools)
    
    print('Exemples avec le vrai map de Python :')
    print('>>> list(map(lambda x : 2*x,[3,7,2,4,1]))\n-->',
          list(map(lambda x : 2*x,[3,7,2,4,1])))
    from math import log
    print('>>> tuple(map(log,[1,2,3]))\n-->',
          tuple(map(log,[1,2,3])))
    print('# map peut prendre plusieurs listes balayées en parallèle :')
    print('>>> list(map(pow,[1,2,3],[2,3,4]))\n-->',
          list(map(pow,[1,2,3],[2,3,4])))   # pow(x,y) == x**y
    print()
    
    from functools import reduce
    print('Exemples avec le vrai reduce de Python :')
    print(">>> reduce(lambda acc,x: acc+x,['t','a','r','t','e'])\n-->",
          repr(reduce(lambda acc,x: acc+x,['t','a','r','t','e'])))
    print('>>> reduce(lambda acc,y: [y] + acc, [1,2,3,4],[])\n-->',
          reduce(lambda acc,y: [y] + acc, [1,2,3,4],[]))   # mais coût quadratique !
    # Les opérateurs + - * etc. ont des contreparties fonctions (pour les passer
    # en argument à une fonctionnelle) :
    from operator import __mul__ as mul     # la fonction (x,y) --> x*y
    print('>>> reduce(mul,range(1,11))     # 10! en one-liner\n-->', 
          reduce(mul,range(1,11)))
    
    # COMPLEMENTS EN LIGNE
    from webbrowser import open as browse
#    browse('https://docs.python.org/fr/3.7/library/functools.html#functools.reduce')
#    browse('http://book.pythontips.com/en/latest/map_filter.html')
#    browse('https://www.geeksforgeeks.org/reduce-in-python/')
#    browse('https://www.python-course.eu/python3_lambda.php')
    
          




      
