#!/usr/bin/env python3
# -*- coding: utf-8 -*-

######################################################################
### PYTHON. APPRENTISSAGE ACTIF (JP. ROY, EDITIONS ELLIPSES, 2020) ###
######################################################################

"""
exo_mat1.py (11.83)
Calculs matriciels (sans numpy, sans classe, à la main)
"""

def build_matrix(f,n,p):
    '''Retourne une matrice ayant n lignes et p colonnes, et dont la
    composante en ligne i et colonne j est obtenu par f(i,j).'''
    return [[f(i,j) for j in range(p)] for i in range(n)]

def mat_dim(m):
    '''Retourne le format (n,p) de la matrice m (une liste de lignes).
    Elle sert aussi à vérifier que la matrice est correctement formée.'''
    n = len(m) ; p = len(m[0])
    assert all(map(lambda x : len(x) == p,m))        # ...de même longueur
    return (n,p)

from operator import __add__            # l'opérateur + vu comme une fonction

def mat_sum(m1,m2):
    '''Retourne la somme des matrices m1 et m2 de même format.'''
    if mat_dim(m1) != mat_dim(m2) : raise ValueError('Mauvais format de matrice !')
    return build_matrix(lambda i,j : m1[i][j] + m2[i][j],*mat_dim(m1))

def mat_ext(k,m):
    '''Retourne le produit externe du nombre k par la matrice m.'''
    return [[k*x for x in lig] for lig in m]

def mat_lig(m,i):
    '''Retourne la ligne i >= 0 de la matrice m.'''
    return m[i]

def mat_col(m,j):
    '''Retourne la colonne j >= 0 de la matrice m.'''
    (n,_) = mat_dim(m)
    return [m[i][j] for i in range(n)]

def mat_mul(m1,m2):
    (n1,p1) = mat_dim(m1)
    (n2,p2) = mat_dim(m2)
    if p1 != n2 : raise ValueError('Produit de matrices impossible !')
    def element(i,j):              # l'élément en ligne i et colonne j du résultat
        return sum(m1[i][k]*m2[k][j] for k in range(p1))
    # le produit aura pour format (n1,p2)
    return build_matrix(element,n1,p2)

def mat_repr(m,prec=1,sep='\t'):
    '''Retourne une chaîne formatée avec colonnes alignées représentant
    la matrice m, avec prec chiffres après le point décimal.'''
    def texte(x):
        return '{1:.{0}f}'.format(prec,x)
    (n,p) = mat_dim(m)
    L = [sep.join(list(map(texte,m[i]))) for i in range(n)]
    return '\n'.join(L)

if __name__ == '__main__':
    I3 = build_matrix(lambda i,j : 1 if i==j else 0, 3, 3)    # une "matrice identité"
    print('>>> I3\n-->',I3)
    A = [[2,-1,4], [5,3,0]]
    B = [[1,2,1], [3,-1,0]]
    C = [[3,1,2], [-1,5,4], [4,1,-2]]
    print('>>> A\n--> {}   de format {}'.format(A,mat_dim(A)))
    print('>>> B\n--> {}   de format {}'.format(B,mat_dim(B)))
    print('>>> C\n--> {}   de format {}'.format(C,mat_dim(C)))
    print()
    print('>>> mat_lig(C,1)\n-->',mat_lig(C,1))
    print('>>> mat_col(C,1)\n-->',mat_col(C,1))
    print()
    print('>>> mat_sum(A,B)\n-->',mat_sum(A,B))
    if mat_sum(A,B) != [[3, 1, 5], [8, 2, 0]]:
        raise ValueError('Bug dans mat_sum')
    print('>>> mat_ext(2,A)\n-->',mat_ext(2,A))
    if mat_ext(2,A) != [[4, -2, 8], [10, 6, 0]]:
        raise ValueError('Bug dans mat_ext')
    print('>>> mat_mul(A,C)\n-->',mat_mul(A,C))
    if mat_mul(A,C) != [[23, 1, -8], [12, 20, 22]]:
        raise ValueError('Bug dans mat_mul')
    print()
    print('>>> mat_repr(C)\n-->',repr(mat_repr(C,prec=0)))
    print('>>> print(mat_repr(C))')
    print(mat_repr(C))

    # L'idée de cet exercice, c'est simplement de coder une fois dans sa vie
    # les opérations matricielles, pas d'en faire un outil de production.
    # En particulier, la fonction mat_repr doit être améliorée si les
    # éléments de la matrice sont de longueur très différente (tabulations).
    
    # Encore une fois, pour du travail sérieux : numpy (ou les bibliothèques
    # de science des données comme PANDA.
    from webbrowser import open as browse
    #browse('http://pandas.pydata.org')
    #browse('http://www.python-simple.com/python-numpy-scipy/manipulation-array.php')















