#!/usr/bin/env python3
# -*- coding: utf-8 -*-

######################################################################
### PYTHON. APPRENTISSAGE ACTIF (JP. ROY, EDITIONS ELLIPSES, 2020) ###
######################################################################

"""
exo_memobin.py (11.100)
Application des dictionnaires aux fonctions à mémoire privée
"""

def memo_bin():
    '''Retourne une fonction à mémoire privée calculant les coefficients
    du binôme de Newton'''
    mem = {}                     # la mémoire privée de la fonction calcul
    def calcul(n,p):             # on suppose 0 <= p <= n
        nonlocal mem             # seulement à cause du cas n==p==-2
        if n == p == -1:         # porte d'entrée secrète...
            return mem           # dump (voir la mémoire privée)
        if n == p == -2:         # porte d'entrée secrète...
            mem = {}             # reset (ré-initialiser la mémoire)
            return
        if (p == 0) or (n == p) :   # 1 sur le bord infini du triangle de Pascal
            return 1
        if (n,p) in mem:       # A l'intérieur ? Ok, déjà calculé ?
            return mem[(n,p)]
        # sinon on calcule, on stocke, on retourne :
        res = calcul(n-1,p) + calcul(n-1,p-1)   # récursion non exponentielle !
        mem[(n,p)] = res
        return res
    return calcul                # on rend une fermeture à mémoire privée

if __name__ == '__main__' :
    from time import time
    binomial = memo_bin()       # un calculateur binomial(n,p) à mémoire
    print('Calcul de binomial(52,13) avec une mémo-fonction :')
    start = time() ; res = binomial(52,13) ; end = time()
    print('{} en {:.6f} secondes'.format(res,end-start))
    print()
    print('Je le refais une seconde fois, il est déjà calculé :')
    start = time() ; res = binomial(52,13) ; end = time()
    print('{} en {:.6f} secondes'.format(res,end-start))
    print()
    print('binomial(32,13) est aussi disponible dans mem :')
    start = time() ; res = binomial(32,13) ; end = time()
    print('{} en {:.6f} secondes'.format(res,end-start))
    print()
    print('En fait, la mémoire contient',     # porte d'entrée secrète...
          len(binomial(-1,-1)),'neurones.')   # p*(n-p)
    print('\n-----------------------------------------------------\n')
    # Python 3.5 a introduit la fonction 'lru_cache' dans le
    # module 'functools'. Elle permet d'automatiser ce mécanisme
    # de mémorisation. Voici un exemple avec memo_bin :

    def binomial(n,p):      # la version récursive exponentielle en temps
        if (p == 0) or (n == p) : return 1    # les bords du triangle de Pascal
        return binomial(n-1,p) + binomial(n-1,p-1)    # relation bien connue

    from functools import lru_cache
    cache = lru_cache(maxsize=None)
    binomial = cache(binomial)
    # cache est un "décorateur". Au lieu d'écrire la ligne précédente, il
    # suffisait d'écrire contre la marge gauche la ligne :
    #                      @lru_cache(maxsize=None)
    # juste au-dessus de la définition de la fonction binomial. Et du coup,
    # binomial calculera en nombre linéaire d'opérations, avec bien entendu
    # une consommation linéaire d'espace de stockage...
    print('Test du décorateur functools.lru_cache :')
    print('Calcul de binomial(52,13) avec lru_cache(maxsize=None) :')
    start = time() ; res = binomial(52,13) ; end = time()
    print('{} en {:.6f} secondes'.format(res,end-start))

    # COMPLEMENTS EN LIGNE
    from webbrowser import open as browse
    #browse('https://www.python-course.eu/python3_memoization.php')
    #browse('https://docs.python.org/fr/3/library/functools.html')   # lru_cache !
    #browse('https://avikdas.com/2019/04/15/a-graphical-introduction-to-dynamic-programming.html?utm_medium=email&utm_source=topic+optin&utm_campaign=awareness&utm_content=20190420+prog+nl&mkt_tok=eyJpIjoiTm1ReVl6VXlNVFZrWldZMyIsInQiOiJreUVQd2VCbkNlRG5hVFA1OStGZTYwTUFQZytLZVRrZk5FVUswbWxiNTFTaTNBS3hFcDFHRHdOTmlSeVUzVnU5ZkR6ZGV2dCtRcTl3YWRUdGFSaUt0NWpyOEFvTUwxUVk3QnM1VkJ2V3FiTk9Lb3ZxT0wrTzA1cjg2QjVZaUVYNiJ9')




