#!/usr/bin/env python3
# -*- coding: utf-8 -*-

######################################################################
### PYTHON. APPRENTISSAGE ACTIF (JP. ROY, EDITIONS ELLIPSES, 2020) ###
######################################################################

"""
exo_newton1.py (11.34)
Recherche d'une solution approchée de f(x)=0 pour f dérivable, et a une 
approximation initiale d'une solution. On suppose qu'il existe au moins
une solution !
"""

from exo_deriv import deriv

def une_racine(f,a,h) :     # a est une approximation initiale, h gouverne la précision
    '''Retourne une solution approchée r de f(x)=0 avec f dérivable, vérifiant
    |f(r)| < h. On suppose qu'il existe au moins une solution et que la dérivée
    de f ne s'annulle pas.'''
    # codage avec des fonctions internes, squelette d'algorithme intéressant
    def assez_bonne(a) :
        return abs(f(a)) < h
    def ameliore(a) :
        if assez_bonne(a) : return a
        return a - f(a)/deriv(f,h)(a)   # on espère que f'(a) n'est pas nul...
    cpt = 0
    while not assez_bonne(a) :
        a = ameliore(a)
        cpt += 1
    # print('coût =',cpt)                # pour observer le faible nombre de tours de boucle
    return a

if __name__ == '__main__' :
    from math import sin,sqrt,pi
    print('On prend h = 0.0001.')
    print()
    print('Une solution approchée de x**2 = 2 à partir de a=1 :')
    print(une_racine(lambda x : x*x-2,1,0.0001),'  (exact={})'.format(sqrt(2)))
    print()
    print('Une solution approchée de x**5-3*x+1 = 0 :')
    print('(on attend -1.38879198440725 ou 0.334734141943353 ou 1.21464804269846)')
    print(une_racine(lambda x : x**5-3*x+1,1,0.0001),' (à partir de a=1)')
    print(une_racine(lambda x : x**5-3*x+1,0,0.0001),' (à partir de a=0)')
    print(une_racine(lambda x : x**5-3*x+1,-2,0.0001),' (à partir de a=-2)')
    print()
    print('Une valeur approchée de pi à partir de a=3 :')
    print(une_racine(sin,3,0.0001),'  (exact={})'.format(pi))
    print()
    
    # COMPLEMENT AVEC SCIPY
    from scipy.optimize import newton as solve    # IL est déjà là :-)
    print('La solution de x**2 = 2 avec scipy.optimize.newton :')
    print(solve(lambda x: x**2-2,1,lambda x: 2*x))
    
    # COMPLEMENTS EN LIGNE
    from webbrowser import open as browse
    #browse('https://fr.wikipedia.org/wiki/M%e9thode_de_Newton')

    
    
    
    
    
    
    

                
    


    

