#!/usr/bin/env python3
# -*- coding: utf-8 -*-

######################################################################
### PYTHON. APPRENTISSAGE ACTIF (JP. ROY, EDITIONS ELLIPSES, 2020) ###
######################################################################

"""
exo_newton2.py (11.93)
Reprise de l'exercice exo_newton1.py pour la recherche de toutes les
racines d'un polynôme creux p dont on suppose toutes les racines distinctes.
"""

from exo_newton1 import une_racine
from exo_polycreux import *
from exo_racspol import derive as deriv # surcharge deriv de exo_newton1
from random import random

# Quelques fonctions utiles pour programmer par récurrence 
# sur les polynômes creux.
def tete(p): return p[0]        # le monôme de plus haut degré
def queue(p): return p[1:]      # le reste
def coeff(p): return p[0][0]    # le coeff. dominant de p

def poly_div(p1,p2):            # algorithme de division euclidienne
    '''Prend deux polynômes creux p1 et p2 et retourne un couple de
    polynômes creux (q,r) tels que : p1 = poly_sum(poly_mul(p2,q),r)
    avec degre(r) < degre(p2). On suppose degre(p2) > 0. Les coeffs.
    sont flottants mais on pourrait travailler avec des rationnels
    si p1 et p2 sont à coefficients rationnels...'''
    if degre(p1) < degre(p2): return [poly0(), p1]
    c = coeff(p1) / coeff(p2)   # division flottante !
    diff = degre(p1) - degre(p2)
    hr = poly_div(poly_sub(p1,poly_mul(p2,[[c,diff]])),p2)  # hyp. de récurrence
    (Q,R) = (poly_sum([[c,diff]],hr[0]),hr[1])
    return (Q,R)

def est_multiple(p1,p2):
    '''Retourne True ssi le polynome creux p1 est un multiple du
    polynome creux p2.'''
    (_,R) = poly_div(p1,p2)
    return is_poly0(R)

def les_racines(p,h):
    '''Prend un polynome creux p dont toutes les racines sont simples
    et reelles, et retourne la liste de toutes les racines.
    Avec un peu de chance...'''
    d = degre(p)    # il y a donc d racines réelles à chercher
    sol = []
    for i in range(d):
        xi = une_racine(poly_fonc(p),random(),h)
        sol.append(xi)
        (p,r) = poly_div(p,[[1,1],[-xi,0]])
    return sorted(sol)
        
if __name__ == '__main__':
    p1 = [[1,5], [2,2], [2,1], [-1,0]]
    p2 = [[1,2],[-1,0]]
    print(poly_div(p1,p2))
    # ([[1.0, 3], [1.0, 1], [2.0, 0]], [[3.0, 1], [1.0, 0]])
    p3 = [[2/3,5],[-1,3],[1/2,0]]
    p4 = [[1,1],[-1,0]]
    print(poly_div(p3,p4))
    # ([[0.66...,4], [0.66...,3], [-0.33..., 2], [-0.33..., 1], [-0.33..., 0]],
    #  [[0.166..., 0]])
    p5 = [[1,5],[-1,0]]
    p6 = [[1,1],[-1,0]]
    print(poly_div(p5,p6))
    # ([[1.0,4], [1.0,3], [1.0,2], [1.0,1], [1.0,0]], [])
    print('est_multiple(p5,p6) :',est_multiple(p5,p6))   # True
    p = poly_mul([[1,2],[-1,0]],     # polynôme factorisé à 4 racines réelles
                     poly_mul([[1,1],[-2,0]],[[1,1],[-3.5,0]]))
    print('>>> p\n-->',p)
    L = les_racines(p,0.001)
    print('>>> les_racines(p)\n-->',L)
    # [-1.0002..., 0.9999..., 2.0002..., 3.4999...]
    fp = poly_fonc(p)
    poly_plot(p,-2,4,Lpts=list(map(lambda r: (r,fp(r)),L)))
    
    # COMPLEMENTS EN LIGNE
    from webbrowser import open as browse
    #browse('https://fr.wikipedia.org/wiki/Division_d%27un_polyn%f4me')
    
    
    

                
    


    

