#!/usr/bin/env python3
# -*- coding: utf-8 -*-

######################################################################
### PYTHON. APPRENTISSAGE ACTIF (JP. ROY, EDITIONS ELLIPSES, 2020) ###
######################################################################

"""
exo_numpy.py (11.108)
Premiers essais de calculs numériques avec le module numpy.
"""

import numpy as np
from math import sqrt

# Question a)
# numpy manipule beaucoup les "tableaux" (arrays)
print('# Le module de calcul numérique "numpy" et ses tableaux.')
print(">>> x = np.linspace(1,10,6)    # j'en veux 6 de 1 à 10")
x = np.linspace(1,10,6)
print('>>> x\n-->',x)
print('>>> type(x)\n-->',type(x),'   # un "tableau", pas une liste !')
print('>>> np.arange(1,11,1.8)        # analogue de range\n-->',
    np.arange(1,11,1.8))
# Les fonctions mathématiques usuelles sont préfixées par np. pour
# fonctionner aussi sur tous les éléments d'un tableau :
print('>>> np.sqrt(x)\n-->',np.sqrt(x))

def integrale(f,a,b,n):
    # somme des aires de petits rectangles
    return sum(f(np.linspace(a,b,n))) * (b-a)/n

print('# Intégrale approchée de x-->sqrt(x+1) sur [0,1] :')
print('>>> integrale(lambda x: np.sqrt(x+1), 0,1,1000)\n-->',
      integrale(lambda x: np.sqrt(x+1), 0,1,1000))
# Bien entendu, la solution "industrielle" passerait par le
# module scipy.integrate...
print()

# Question b)

import matplotlib.pyplot as plt

x = np.linspace(0,np.pi,1000)
y = x * np.sin(2*x)

plt.plot(x,y,'r-')                    # red + lignes
plt.xlabel('x')
plt.ylabel('y')
plt.legend(['Courbe y = x*sin(2*x)'])
plt.show()                            # bloquant par défaut

# Question c)

def f(x,y) :
    return np.sin(-x**2 - y**2)       # <--- changer !!!

# liste des abscisses et des ordonnées des points du maillage
x = np.linspace(-3,3,51)
y = np.linspace(-2,2,41)
# création d'un maillage (x,y) en 2D
(xg,yg) = np.meshgrid(x, y)
# calcul de l'altitude z = f(x,y) de chaque point du maillage
zg = f(xg,yg)
# affichage de l'altitude en couleur
plt.pcolor(xg,yg,zg)   # essayez plt.contour(xg,yg,zg) pour voir les lignes de niveau !
plt.colorbar()         # pour voir la correspondance couleur <-> nombre
plt.show(block=False)
print()

# Question d)

from numpy import linalg as AL  # la bibliothèque d'Algèbre Linéaire

# système linéaire ayant une solution unique
#      6x - 5y + 2z = 0
#      -x +  y - 3z = 1                     matriciellement :    A X = B
#       x + 2y +  z = -2

print('# Matrice du système linéaire (avec ndarray) :')
A = np.array([[6,-5,2],[-1,1,-3],[1,2,1]],dtype=float)   # calculs flottants
print('>>> print(A)')
print(A)
B = np.array([0,1,-2],dtype=float)
print('>>> print(B)     # et son second membre')
print(B)
print("# La notation [a b c] indique l'ambiguité entre ligne et colonne !")
print('# B peut être vu comme vecteur ligne ou colonne suivant sa position,')
print('# la multiplication des matrices se note A@B ou A.dot(B) :')
print('>>> A@B     # B vecteur colonne\n-->',A@B)
print('>>> B@A     # B vecteur ligne\n-->',B@A)
print()
print('# - Solution du système linéaire avec solve du module numpy.linalg')
(x,y,z) = AL.solve(A,B)
print('(x,y,z) =',(x,y,z))
print()
print('# - Solution en passant par la matrice inverse de A')
dA = AL.det(A)
print('>>> AL.det(A)    # déterminant du système\n-->',dA)
if dA != 0:
    print('# donc A est inversible et son inverse est :')
    A_inv = AL.inv(A)      # et non A**(-1)
    print(A_inv)
    # vérification en utilisant np.eye(3) : la matrice unité d'ordre 3
    # et nb.allclose qui vérifie que les éléments de deux tableaux sont
    # "quasiment" les mêmes.
    print("Vérification de l'inverse :",
          'OK' if np.allclose(A @ A_inv,np.eye(3)) else 'NON !')
    (x,y,z) = A_inv @ B
    print('(x,y,z) =',(x,y,z))

# COMPLEMENTS EN LIGNE
from webbrowser import open as browse
#browse('http://www.scipy-lectures.org/intro/numpy/index.html')                      # ndarray
#browse('https://docs.scipy.org/doc/numpy/reference/ufuncs.html#available-ufuncs')   # fonctions universelles
#browse('https://www.datacamp.com/community/tutorials/python-numpy-tutorial')


