#!/usr/bin/env python3
# -*- coding: utf-8 -*-

######################################################################
### PYTHON. APPRENTISSAGE ACTIF (JP. ROY, EDITIONS ELLIPSES, 2020) ###
######################################################################

"""
exo_perceptron1.py (11.133)
Le "perceptron" de base, avec sa fonction d'activation Y de Heaviside.
On choisit à la main les poids dont celui du biais. Le biais est constant (1)
et forcé en première entrée du perceptron.
"""

# --------------- question a) ---------------------------------------

def Y(x):       # la fonction d'Heaviside
    return 1 if x >= 0 else 0

class Perceptron:
    '''Construit un perceptron dont on donne la liste des poids Lpoids,
    le premier poids étant celui du biais.'''
    
    def __init__(self,Lpoids):
        self.Lpoids = Lpoids

    def reponse(self,Lentrees):
        '''La liste Lentrees représente les données concrètes, sans le biais.'''
        if len(Lentrees) != len(self.Lpoids) - 1:
            raise ValueError("Mauvais nombre d'entrées !")
        Lentrees = [1] + Lentrees         # le biais est constant en tête et vaut 1
        somme_ponderee = sum(Lentrees[i] * self.Lpoids[i] for i in range(len(Lentrees)))
        return Y(somme_ponderee)

if __name__ == '__main__':
    print('Perceptron p1 : x1 OU x2')
    p1 = Perceptron([-1,1,1])
    for x in (0,1):
        for y in (0,1):
            print('{} ou {} --> {}'.format(x,y,p1.reponse([x,y])))
    print()

    print('Perceptron p2 : x1 ET x2')
    p2 = Perceptron([-3,1,2])       # le biais vaut -3, les deux poids sont 1 et 2
    for x in (0,1):
        for y in (0,1):
            print('{} et {} --> {}'.format(x,y,p2.reponse([x,y])))
    print()

    print('Perceptron p3 : NON x1')
    p3 = Perceptron([0,-1])         # le biais vaut 0, le poids de l'unique entrée est -1
    for x in (0,1):
        print('non {} --> {}'.format(x,p3.reponse([x])))
    print()

    print('Perceptron p4 : x1 ET (x2 OU (NON x3))')
    p4 = Perceptron([-2,2,1,-1])    # le biais vaut -2, les poids sont 2, 1 et -1
    for x in (0,1):
        for y in (0,1):
            for z in (0,1):
                print('{} et ({} ou (non {})) --> {}'.format(x,y,z,p4.reponse([x,y,z])))
    print()

# --------------- question b) ---------------------------------------

# Par l'absurde, supposons qu'il existe un perceptron réalisant le XOR (ou exclusif).
# p5 = Perceptron([w0,w1,w2]) où w0 est le biais. La réponse sur les entrées [x1,x2]
# est égale à Y(w0 + w1*x1 + w2*x2).
# On doit donc avoir d'après la table de vérité du XOR :
#   x1=0, x2=1 --> Y(w0 + w2) = 1 ==> w0 + w2 >= 0
#   x1=1, x2=0 --> Y(w0 + w1) = 1 ==> w0 + w1 >= 0
#   x1=0, x2=0 --> Y(w0) = 0 ==> w0 < 0
#   x1=1, x2=1 --> Y(w0 + w1 + w2) = 0 ==> w0 + w1 + w2 < 0
#
# Je vous laisse vérifier que ces 4 inéquations sont incompatibles. 
#                                                       CQVRV
    
# CQVRV ??? Ce Qu'il Vous Reste à Vérifier :-)










