#!/usr/bin/env python3
# -*- coding: utf-8 -*-

######################################################################
### PYTHON. APPRENTISSAGE ACTIF (JP. ROY, EDITIONS ELLIPSES, 2020) ###
######################################################################

"""
exo_perceptron2.py (11.133)
L'algorithme d'apprentissage des poids par le "perceptron".
Le premier poids est celui de l'entrée biaisée mise à 1.
"""

from random import random
from exo_perceptron1 import Perceptron
from time import time

class PerceptronApprentissage(Perceptron):
    '''Construit un perceptron capable de calculer les poids permettant de
    satisfaire un "training set".'''
    def __init__(self,Lexemples,timeout=120):
        '''La liste 'Lexemples' représente le training set : ce que l'on souhaire obtenir en réponse.
        Les poids sont initialisés aleatoirement. Abandonne le calcul au bout de 'timeout' secondes.'''
        super().__init__(Lpoids=[0 for i in range(len(Lexemples[0][0]) + 1)])
        self.Lexemples = [(L,v) for (L,v) in Lexemples]         # 1 pour le biais constant en premier
        self.timeout = timeout
        print('Training set :',self.Lexemples)
        print('Initialisation : bias = {} et poids = {}'.format(self.Lpoids[0],self.Lpoids[1:]))
        self.apprentissage()

    def apprentissage(self):
        print("J'apprends ({} sec max)...".format(self.timeout))
        start = time()
        ip = 0                                                  # numéro du premier exemple
        while not self.apprentissage_fini():                    # tant que TOUT n'est pas appris :
            (entrees,sortie_attendue) = self.Lexemples[ip]      # un nouvel exemple à apprendre
            sortie_obtenue = self.reponse(entrees)              # sortie obtenue sur l'exemple courant
            if sortie_obtenue != sortie_attendue :              # rectification de l'erreur
                self.modifier_poids(entrees,sortie_obtenue,sortie_attendue)   # learn !            
            ip = (ip + 1) % len(self.Lexemples)                  # circulaire
            if time() - start > self.timeout:
                print("Trop long, j'abandonne l'apprentissage !")
                return
        print('Apprentissage OK. Bias = {} et poids = {}'.format(self.Lpoids[0],self.Lpoids[1:]))
        
    def modifier_poids(self,entrees,sortie_obtenue,sortie_attendue) :     # mise à jour des poids
        entrees = [1] + entrees
        for i in range(len(self.Lpoids)) :
            self.Lpoids[i] += (sortie_attendue - sortie_obtenue) * entrees[i]
        
    def apprentissage_fini(self):                               # la leçon est-elle apprise ?
        for i in range(len(self.Lexemples)):
            (entrees,sortie_attendue) = self.Lexemples[i]
            if self.reponse(entrees) != sortie_attendue :
                return False
        return True

if __name__ == '__main__':
    # un perceptron qui apprend le OU
    p1 = PerceptronApprentissage(Lexemples=[([1,1],1),([0,0],0),([1,0],1),([0,1],1)])   # ce que l'on veut
    print('Vérifions si le perceptron p1 a appris la formule (x1 ou x2) :')
    for x in (0,1) :
        for y in (0,1) :
            print('{} ou {} --> {}'.format(x,y,p1.reponse([x,y])))
    print()

    # un perceptron qui apprend le ET
    p2 = PerceptronApprentissage(Lexemples=[([1,1],1),([0,0],0),([1,0],0),([0,1],0)])   # ce que l'on veut
    print('Vérifions si le perceptron p1 a appris la formule (x1 et x2) :')
    for x in (0,1) :
        for y in (0,1) :
            print('{} et {} --> {}'.format(x,y,p2.reponse([x,y])))
    print()

    # un perceptron qui apprend le NON
    p3 = PerceptronApprentissage(Lexemples=[([1],0),([0],1)])   # ce que l'on veut
    print('Vérifions si le perceptron p3 a appris la formule (non x1) :')
    for x in (0,1) :
            print('non {} --> {}'.format(x,p3.reponse([x])))
    print()

    # un perceptron qui apprend la formule logique (x1 et (x2 ou (non x3)))
    p4 = PerceptronApprentissage(Lexemples=[([1,0,0],1),([1,0,1],0),([1,1,0],1),([1,1,1],1),
                                            ([0,0,0],0),([0,0,1],0),([0,1,0],0),([0,1,1],0)])
    print('Vérifions si le perceptron p4 a appris la formule (x1 et (x2 ou (non x3))) :')
    for x in (0,1) :
        for y in (0,1) :
            for z in (0,1) :
                print('{} et ({} ou (non {})) -> {}'.format(x,y,z,p4.reponse([x,y,z])))
    print()

    # un perceptron qui essaye apprendre le "ou exclusif" XOR
    print('Une tentative avec le "ou exclusif", Ctrl-C pour stopper !')
    p5 = PerceptronApprentissage(Lexemples=[([1,1],0),([0,1],1),([1,0],1),([0,0],0)], timeout=30)
    print('Vérifions si le perceptron p5 a appris la formule (x1 XOR x2) :')
    for x in (0,1) :
        for y in (0,1) :
            print('{} oux {} --> {}'.format(x,y,p5.reponse([x,y])))
    print()

    # COMPLEMENTS EN LIGNE (qui débordent parfois du simple perceptron)
    from webbrowser import open as browse
    #browse('https://en.wikipedia.org/wiki/Perceptron')      # meilleur que la version française
    #browse('https://www.youtube.com/watch?v=5w-jolCle6g')   # 5 minutes :-)
    
    #browse('http://www.cs.toronto.edu/~hinton/') # cherchez le mot COURSERA et regardez les vidéos !
    # Le MOOC de Geoffrey Hinton (un pionnier) de 2012 n'est plus en activité. Pour quelque chose
    # de plus récent, il y a (parmi d'autres) celui d'Andrew Ng sur Coursera :
    # https://www.coursera.org/learn/neural-networks-deep-learning
    
    #browse('https://www.irif.fr/~kesner/enseignement/iup/cours81.pdf')
    #browse('https://en.wikipedia.org/wiki/History_of_artificial_intelligence#Perceptrons_and_the_dark_age_of_connectionism')
    
    # Les réseaux de neurones sont en quelque sorte à l'opposé de la
    # programmation "par règles" classique de l'I.A. Ils apprennent leur
    # propre représentation interne des données pour le problème en cours.
    # Le calcul symbolique style sympy ou Mathematica est un aboutissement
    # de l'approche classique. Jusqu'ici les neurones n'étaient pas adaptés
    # à ces techniques. Mais des chercheurs français de la division IA de
    # Facebook viennent d'utiliser des arbres d'expressions symboliques
    # (livre exos 105-106-107) dans le cadre des réseaux de neurones (pages 
    # 295-303) pour leur faire effectuer des calculs symboliques, ex: calcul 
    # d'intégrales indéfinies.
    # Cela pourrait être une belle avancée dans le deep learning :
    #browse('https://www.quantamagazine.org/symbolic-mathematics-finally-yields-to-neural-networks-20200520/')
    #browse('https://arxiv.org/pdf/1912.01412.pdf')

    
    
