#!/usr/bin/env python3
# -*- coding: utf-8 -*-

######################################################################
### PYTHON. APPRENTISSAGE ACTIF (JP. ROY, EDITIONS ELLIPSES, 2020) ###
######################################################################

"""
exo_pickle.py (11.123)
Le module 'pickle' pour sérialiser des données sur disque.
N'utilisez 'pickle' que dans le cas où JSON ne s'applique pas...
"""

class Voiture:
    def __init__(self,modele,annee):
        self.modele = modele
        self.annee = annee
    def __repr__(self):
        return 'Voiture[{},{}]'.format(self.modele,self.annee)

print('###################### avec JSON ##########################')
import json

# Attention, avec JSON, les clés des dictionnaires doivent être des
# chaînes de caractères Unicode (UTF-8) :
L = [{'cent':100, 'zéro':0}, {'3':[2.56], '1':True, '2':None}]
print('L version Python contient deux dictionnaires :')
print(L)
print()
print('---- sérialisation vers Ls en mémoire :')
Ls = json.dumps(L)
print('Ls version JSON (au format Javascript)  :')
print(Ls)

print('Je sors L de la mémoire avec "del" !')
del L

print('Et je désérialise Ls pour obtenir L :')
L = json.loads(Ls)
print(L)
print()

print('---- sérialisation dans fichier-disque "serial1.json" :')
print('L version Python en mémoire :')
print(L)
with open('serial1.json','w',encoding='utf-8') as f_out:
    json.dump(L,f_out)
print('version JSON sur disque dans "serial1.json" :')
with open('serial1.json','r',encoding='utf-8') as f_in:
    print(f_in.read())

print('Je sors L de la mémoire avec "del" !')
del L

print('Et je désérialise serial1.json dans L :')
with open('serial1.json','r',encoding='utf-8') as f_in:
    L = json.load(f_in)
print(L)
print()

print("ATTENTION. J'ajoute un objet de classe Voiture dans L !")
L.append(Voiture('Aston-Martin DB5',1964))   # celle de Bond, what else ?
print('L =',L)
print("Et j'essaye de sérialiser L :")
try :
    Ls = json.dumps(L)
except TypeError as e :
    print('ERREUR : ' + str(e))
    print('Oups. On passe donc à pickle...')
print()

print('###################### avec PICKLE #######################')
import pickle       # et encore dump et load, mais...

# Avec pickle, les clés d'un dictionnaire ne sont pas forcément des chaînes.
L = [{'cent':100, 'zéro':0},
     {3:[2.56], 1:True, 2:None},
     Voiture('Aston-Martin DB5',1964)]   # <- un objet de classe Voiture

print('---- sérialisation en mémoire :')
print("Les clés d'un dictionnaire ne sont plus forcément des chaînes !")
print('L en mémoire :',L)
print('Ls version pickle Ls en mémoire :')
Ls = pickle.dumps(L)
print(Ls)

print('Je sors L de la mémoire avec "del" !')
del L

print('Et je désérialise Ls en mémoire dans L :')
L = pickle.loads(Ls)
print(L)
print('L[2] est de type :',type(L[2]))
print()

print('---- sérialisation sur disque :')
print('L version Python en mémoire :')
print(L)
# Attention, il faut ouvrir un fichier BINAIRE en mode 'wb'.
# Pas d'encodage pour les fichiers binaires...
with open('serial2.pkl','wb') as f_out:
    pickle.dump(L,f_out)    # avec le protocole par défaut
print('La version PICKLE de L est sur disque dans "serial2.pkl".')
with open('serial2.pkl','rb') as f_in:
    print(f_in.read())

print('Je sors L de la mémoire !')
del L

print('Et je désérialise "serial2.pkl" dans L :')
with open('serial2.pkl','rb') as f_in:
    L = pickle.load(f_in)
print(L)
print('L[2] est de type :',type(L[2]))
print()

from webbrowser import open as browse
#browse('https://www.reddit.com/r/learnpython/comments/3db16f/json_vs_pickle_breakdown/')
#browse('https://pymotw.com/3/pickle/')
#browse('https://docs.python.org/3/library/pickle.html')
#browse('https://pymotw.com/3/json/')

