#!/usr/bin/env python3
# -*- coding: utf-8 -*-

######################################################################
### PYTHON. APPRENTISSAGE ACTIF (JP. ROY, EDITIONS ELLIPSES, 2020) ###
######################################################################

"""
exo_polycreux.py (11.89)
Polynômes creux [[c,e],...] come liste de monômes c*X**e en
puissances décroissantes.
"""

# question a) - Algorithme d'insertion dans une liste triée en croissant LT

def insertion(x,LT):
    '''Prend une liste LT triée en ordre croissant et retourne une copie de
    LT, triée en ordre croissant et contenant le nouvel élément x.'''
    # En fait, on recherche le premier élément de LT qui soit > x
    for i in range(len(LT)):
        if LT[i] > x: 
            return LT[:i] + [x] + LT[i:]    # une solution avec des tranches de liste
    return LT + [x]

# questions suivantes...
    
def poly0():                  # constructeur du polynome nul
    return []

def is_poly0(p):
    return p == []

def degre(p):
    if is_poly0(p): return float('-inf')    # les maths...
    return p[0][1]           # degré du monôme dominant (p est trié)

def mono_plus_poly(c,e,p):   # le monôme cx^e ajouté à sa place au polynôme p. Itératif !
    # algo d'insertion : on cherche le premier monôme de degré d <= e
    for i in range(len(p)):
        [ci,ei] = p[i]       # le monôme numéro i de p
        if ei == e:
            essai = c + ci
            if essai == 0 : return p[:i] + p[i+1:]
            return p[:i] + [[essai,e]] + p[i+1:]
        if ei < e:           # insertion d'un nouvel element !
            return p[:i] + [[c,e]] + p[i:]
    return p + [[c,e]]

def poly_sum(p,q):
    for [c,e] in p:
        q = mono_plus_poly(c,e,q)      # aucune mutation sur q...
    return q

def poly_ext(k,p):
    if k == 0: return poly0()
    return [[c*k, e] for [c,e] in p]

def poly_sub(p,q):
    '''Retourne la soustraction p-q de deux polynomes creux.'''
    return poly_sum(p,poly_ext(-1,q))

def mono_mul_poly(c,e,p):
    res = poly0()
    if c == 0 : return res
    for [ci,ei] in p:
        res.append([c * ci, e + ei])
    return res

def poly_mul(p,q):
    res = poly0()
    for [ci,ei] in p:
        res = poly_sum(res,mono_mul_poly(ci,ei,q))
    return res

def poly_coef(p,k):
    '''Retourne le coefficient de X**k dans p.'''
    for [c,e] in p:
        if e == k: return c
    return 0

def poly_val(p,x):
    '''Retourne la valeur du polynome p au point x.'''
    res = 0
    for [c,e] in p:
        res = res + c*(x**e)
    return res

def poly_fonc(p):
    '''Retourne la fonction polynôme associée au polynôme p.'''
    return lambda x: poly_val(p,x)

def poly_fonc(p):     # une autre manière plus générale de coder sans lambda
    '''Retourne la fonction polynôme associée au polynôme p.'''
    def res(x):
        return poly_val(p,x)
    return res

def poly_plot(p,a,b,Lpts=[],n=100):     # bonus
    '''Affiche avec matplotlib le polynôme creux p sur [a,b].
    Lpts est une liste de points (x,y) qui seront affichés aussi.
    n est le nombre de points (100 par défaut).'''
    import matplotlib.pyplot as plt
    import numpy as np
    f = poly_fonc(p)
    x = np.linspace(a,b,n)
    plt.plot(x,f(x),'r-')
    for (x,y) in Lpts:
        plt.plot(x,y,'bo')
    plt.show(block=False)

if __name__ == '__main__':
    # la question
    p = [[2,5],[-1,4],[-2,1]]
    q = [[1,4],[7,2],[-1,0]]
    print('>>> (p, q)\n-->',(p,q))
    print('>>> mono_plus_poly(1,4,p)      # X**4 + p\n-->',
        mono_plus_poly(1,4,p))
    print('>>> mono_plus_poly(5,3,p)      # 5X**3 + p\n-->',
        mono_plus_poly(5,3,p))
    print('>>> mono_plus_poly(1,0,p)      # 1 + p\n-->',
        mono_plus_poly(1,0,p))
    print('>>> poly_sum(p,q)              # p + q\n-->',
         poly_sum(p,q))
    print('>>> poly_ext(2,p)              # 2p\n-->',
        poly_ext(2,p))
    print('>>> poly_ext(0,p)              # 0p\n-->',
        poly_ext(0,p))
    print('>>> poly_sub(p,q)              # p - q\n-->',
        poly_sub(p,q))
    print('>>> mono_mul_poly(2,3,p)       # 2X**3 * p\n-->',
        mono_mul_poly(2,3,p))
    print('>>> poly_mul(p,q)              # p x q\n-->',
          poly_mul(p,q))
    print('>>> poly_mul([[1,100],[-1,0]],[[1,100],[1,0]])\n-->',
         poly_mul([[1,100],[-1,0]],[[1,100],[1,0]]))
    print('>>> poly_coef(p,2)             # coeff de X**2\n-->',
        poly_coef(p,2))
    print('>>> poly_coef(p,4)             # coeff de X**4\n-->>',
        poly_coef(p,4))
    print('>>> poly_val(p,1)              # p(1)\n-->',
        poly_val(p,1))
    print('>>> fp = poly_fonc(p)          # polynôme -> fonction')
    fp = poly_fonc(p)
    print('>>> (type(fp), fp(1))\n-->',
          (type(fp), fp(1)))
    poly_plot([[1,3],[-8,1]],-5,5,Lpts=[(0,0)])

    # COMPLEMENTS EN LIGNE
    from webbrowser import open as browse
#    browse('https://fr.wikipedia.org/wiki/M%E9thodes_de_quadrature_de_Gauss#M.C3.A9thode_de_Gauss-Legendre')
#    browse('https://fr.wikipedia.org/wiki/Polyn%F4me_de_Legendre#Formule_de_r.C3.A9currence_de_Bonnet')
#    browse('https://fr.wikipedia.org/wiki/Polynôme_formel')

