#!/usr/bin/env python3
# -*- coding: utf-8 -*-

######################################################################
### PYTHON. APPRENTISSAGE ACTIF (JP. ROY, EDITIONS ELLIPSES, 2020) ###
######################################################################

"""
exo_prime1.py (11.20)
Test naïf de primalité d'un entier n
"""

# n est un nombre premier 
#      <==> n >= 2 ET s'il n'admet aucun diviseur > 1 autre que lui-même

def est_premier(n):                        # int --> bool
    return (n >= 2) and (ppdiv(n) == n)    # d'où un sous-problème ppdiv

# Question a)
def ppdiv_v1(n):                           # int --> int, n >= 2
    # je cherche le plus petit diviseur dans [2,n-1]
    d = 2                                  # le premier candidt
    while n % d != 0:                      # tant que d ne divise pas n                     
        d = d + 1                          #     on monte !
    return d                               # au pire d == n

# Le nombre de divisions pour ppdiv_v1(n) est clairement d'ordre n dans le 
# pire des cas. On dit que son coût est d'ordre O(n).

# Question b)
def ppdiv_v2(n):                           # int --> int, n >= 2
    # je cherche le plus petit diviseur dans [2,n-1]
    if n % 2 == 0: return 2                # je règle le cas n pair en premier
    # donc je peux maintenant monter de 3 vers n, sur les impairs uniquement !
    d = 3
    while n % d != 0:
        d = d + 2             # deux fois moins cher que d = d + 1 mais bof...
    return d

# Le nombre de divisions est toujours d'ordre n dans le pire des cas. En fait
# n/2 mais O(n/2) = O(n), on dit que l'algorithme est LINEAIRE en le nombre 
# de divisions. Attention : pas en temps de calcul, car le coût des divisions
# dépend du nombre de chiffres...

# Question c)
# Si d < sqrt(n) est un diviseur de n, alors d' = n/d en est un autre. De plus :
# d < sqrt(n) ==> n/d' < sqrt(n) ==> d' > n/sqrt(n) ==> d' > sqrt(n).   CQFD
# NB. Si d = sqrt(n) est diviseur de n, alors d' = d

def ppdiv_v3(n):                 # avec au plus sqrt(n) opérations
    if n % 2 == 0: return 2
    d = 3
    while n % d != 0:
      if d * d > n: return n    # <==> d > sqrt(n) mais plus rapide et entier !
      d = d + 2
    return d

ppdiv = ppdiv_v3                # on va stopper à cette version définitive.

def premiers(n):                # affichage des nombres premiers de [2,n]
    for i in range(2,n+1):
        if est_premier(i):
            print(i,end=' ')
    print()

def nb_premiers(n):             # nombre de premiers dans [2,n]
    res = 0
    for i in range(2,n+1):
        if est_premier(i):
            res = res + 1
    return res

def nb_premiers(n):             # version plus pythonesque :-)
    return sum(1 for i in range(2,n+1) if est_premier(i))

# Le nombre de divisions effectuées par nb_premiers(n) est en O(n*sqrt(n))
# car il y a n tours de boucle, et chaque tour de boucle coûte O(sqrt(i)),
# donc sum(sqrt(i) for i in range(n)), qui est (par comparaison violente
# avec une intégrale) en O(n*sqrt(n))... Hum, un peu lapidaire.

if __name__ == '__main__' :
    print('ppdiv_v1(77) =',ppdiv_v1(77))
    print('ppdiv_v1(29) =',ppdiv_v1(29),'donc il est premier !')
    print('ppdiv_v2(77) =',ppdiv_v2(77))
    print('ppdiv_v2(29) =',ppdiv_v2(29),'donc il est premier !')
    print('accélération de ppdiv avec sqrt :')
    print('ppdiv(77) =',ppdiv(77))
    print('ppdiv(29) =',ppdiv(29),'donc il est premier !')
    print()
    premiers(100)
    print()
    print('Attendez, je calcule un peu...')
    nbp = nb_premiers(1_000_000)
    print("Jusqu'à 1_000_000 il y a nbp =",nbp,'nombres premiers.')
    print('Vérification :')
    print('  nbp * log(1_000_000) / 1_000_000 =',end = ' ')
    from math import log
    print(nbp * log(1_000_000) / 1_000_000,'proche de 1')
    print('Il va falloir autre chose pour les très grands nombres premiers !!')
    
    # COMPLEMENTS EN LIGNE
    from webbrowser import open as browse
    #browse('https://fr.wikipedia.org/wiki/Nombre_premier')
    #browse('http://images.math.cnrs.fr/Jacques-Hadamard-et-le-theoreme.html')
    
    
# NB. On peut remarquer que le reste r de la division d'un nombre premier p
# par 6 ne peut être égal qu'à 1 ou 5 (car p = 6q + r et si r vaut 0, 2, 3 ou
# 4, alors p serait multiple de 2 ou 3). Cela permettrait d'aller un peu plus
# vite que seulement de 2 en 2 dans ppdiv_v3, mais bof...


    
    

    


    
