#!/usr/bin/env python3
# -*- coding: utf-8 -*-

######################################################################
### PYTHON. APPRENTISSAGE ACTIF (JP. ROY, EDITIONS ELLIPSES, 2020) ###
######################################################################

"""
exo_prime2.py (11.23)
Technique de Fermat pour détecter si un grand nombre est premier.
Enfin, "très probablement" premier...
"""

from random import randint
from math import gcd
from exo_puismod import puismod

# Petit théorème de Fermat :
#        n premier ==> a**(n-1) = 1 (modulo n) pour tout a premier avec n
# ou:    n premier ==> n est pseudo-premier dans toute base a première avec n

# Question a)
def essai_Fermat() :
    n = 23
    for a in (2,3,5,7,11,13,17,19):     # les a premiers avec n=23
        if puismod(a,n-1,n) != 1:
            print('Bug sur Fermat avec a={}, n={}'.format(a,n))

# Question b)
# La réciproque est fausse mais "presque" vraie statistiquement. Si l'on effectue
# le test a**(p-1) = 1 (modulo p) plusieurs fois, très peu de nombres passent
# à-travers, ce sont les nombres de Carmichael (il y en a 255 jusqu'à 100_000_000).

# "Numbers that fool the Fermat test are called 'Carmichael numbers', and little
#  is known about them other than that they are extremely rare [...] In testing 
#  primality of very large numbers chosen at random, the chance of stumbling upon
#  a value that fools the Fermat test is less than the chance that a cosmic 
#  radiation will cause the computer to make an error in carrying out a 'correct' 
#  algorithm. Considering an algorithm to be inadequate for the first reason but
#  not for the second illustrates the difference between mathematics and 
#  engineering." (https://web.mit.edu/alexmv/6.037/sicp.pdf page 69, note en bas)

def pseudo_premier(n,a):
    '''Teste si n passe le "test de Fermat en base a".
    On suppose a premier avec n et a dans [2,n-1].'''
    return puismod(a,n-1,n) == 1 

def fermat_premier(n):
    '''Teste si n est "très probablbment premier au sens de Fermat". Cette
    fonction suppose que n est très grand.'''
    if n == 2: return True          # car le test de Fermat ne s'applique pas
    for i in range(20):             # on procède à 20 tests de Fermat
        while True:                 # on tire a dans [2,n-1] premier avec n
            a = randint(2,n)
            if gcd(a,n) == 1: break
        if not pseudo_premier(n,a):
            return False            # certainement non premier !
    return True                     # très probablement premier...

# 
def mersenne(k):        # le k-ème nombre de Mersenne (certains sont premiers)
    return 2**k - 1

if __name__ == '__main__':
    # Question a)
    essai_Fermat()      # doit fonctionner en silence :-)
    # Question b)
    print('Nombres Fermat-premiers < 60 :') 
    print([i for i in range(2,60) if fermat_premier(i)])
    print()
    print('Test sur les nombres de Mersenne de la forme : mersenne(k) = 2**k-1')
    m = mersenne(521)
    print('mersenne(521) vaut :')
    print(m,'({} chiffres)'.format(len(str(m))))
    print()
    if fermat_premier(m): 
        print('mersenne(521) est premier avec Fermat !')
    if not fermat_premier(mersenne(523)): 
        print("mersenne(523) n'est pas premier")
    print()
    print('Le plus petit nombre premier ayant 400 chiffres est (patience) :')
    d = 10**399 + 1
    while not fermat_premier(d): 
        d += 2
    print(d,'\n')

    # Nombres de Carmichael
    print('Je vérifie que 561 est un nombre de Carmichael en testant')
    print('tous les a dans [2,n-1] premiers avec n.')
    print('fermat_premier(561) -->',fermat_premier(561),'   (oups ! 561 % 17 == 0)')
    print()
    
    from exo_prime1 import est_premier     # algo lent, pour les "petits" entiers seulement...

    # un "nombre de Carmichael" est un nombre non premier qui est pseudo-premier
    # dans toute base a < n première avec n.
    def est_Carmichael(n):                # pour les "petits" entiers !
        if est_premier(n): return False
        for a in range(2,n):
            if gcd(a,n) == 1:
                if not pseudo_premier(n,a): 
                    return False
        return True

    if est_Carmichael(561): 
        print('Oui, 561 est bien un nombre de Carmichael')
    print('Les nombres de Carmichael < 100_000 :')
    for n in range(2,100_000):
        if est_Carmichael(n): 
            print(n,end=' ')
    print()
    # Nous écrirons cette boucle - plus tard, page 82 - bien plus simplement :
    # print(list(filter(est_Carmichael,range(2,100_000))))

    # Encore une fois, cette technique de Fermat n'est intéressante que si
    # l'entier n est TRES GRAND !
    
    # COMPLEMENTS EN LIGNE
    from webbrowser import open as browse
    #browse('https://fr.wikipedia.org/wiki/Petit_théorème_de_Fermat')
    #browse('https://fr.wikipedia.org/wiki/Nombre_de_Carmichael')
    #browse('http://villemin.gerard.free.fr/ThNbDemo/TextCarm.htm#Approche')
    #browse('https://tel.archives-ouvertes.fr/tel-00112121/document')
    
    
    

                
    


    

