#!/usr/bin/env python3
# -*- coding: utf-8 -*-

######################################################################
### PYTHON. APPRENTISSAGE ACTIF (JP. ROY, EDITIONS ELLIPSES, 2020) ###
######################################################################

"""
exo_pymunk_balle.py (11.130)
La balle verticale en pymunk. On ne gère plus les collisions !
----> Erreur en ligne 2673 dans le livre, qui parle d'un fichier
exo_balle_pymunk.py au lieu de exo_pymunk_balle.py. Sorry.

L'intégration de Pyglet avec la console de Spyder n'est pas complète
à ce jour (Spyder 3.3.6). En cas de problèmes à l'exécution, utilisez
IDLE pour programmer avec pymunk (commande 'idle3' au Terminal),
ou bien exécutez directement au Terminal, ou bien tuez la console courante
de Spyder pour exécuter à nouveau...
Toujours pas de '%gui pyglet' à la console IPython. Ceci peut évoluer...
"""

import pymunk

space = pymunk.Space()
space.gravity = (0,-1000)

def balle_dynamique(masse,rayon,elasticity=0.0,friction=0.0):
    moment = pymunk.moment_for_circle(masse,0,rayon)
    body = pymunk.Body(masse,moment)
    shape = pymunk.Circle(body,rayon)
    shape.elasticity = elasticity
    shape.friction = friction
    space.add(body,shape)
    return body

def segment_statique(pt1,pt2,epaisseur,elasticity=0.0,friction=0.0):
    shape = pymunk.Segment(space.static_body,pt1,pt2,epaisseur)
    shape.elasticity = elasticity
    shape.friction = friction
    space.add(shape)
    return shape

# Le module pyglet s'occupera du rendu de l'animation.

import pyglet
from pymunk.pyglet_util import DrawOptions   # pour dessiner les formes facilement

class My_pymunk_app(pyglet.window.Window):
    def __init__(self,*args,**kwargs):       # on peut utiliser les options d'init de Window
        pyglet.window.Window.__init__(self,*args,**kwargs)
        pyglet.clock.schedule_interval(self.update,1/60)    # horloge à 60 Hz
        self.draw_options = pymunk.pyglet_util.DrawOptions()
        self.draw_options.flags = self.draw_options.DRAW_SHAPES
        # un corps dynamique de masse 1 (disque de rayon R) est ajouté à l'espace
        self.body = balle_dynamique(1,20,0.9)
        self.body.position = (300,580)
        # on ajoute un segment horizontal statique (ni masse ni moment)
        segment_statique((200,-1),(400,-1),1,0.9) # et si l'élasticité vaut 0 ?
        #pyglet.gl.glClearColor(0.2,0.2,0.1,1)          # couleur du fond avec OpenGL
        print('position =',self.body.position)

    def on_draw(self):  # redéfinition de la méthode on_draw de la fenêtre courante
        self.clear()
        space.debug_draw(self.draw_options)

    def update(self,dt):  # callback de l'horloge
        # dt représente le temps en secondes écoulé depuis le dernier appel
        if (self.body.position.y < 21) and (abs(self.body.velocity.y) < 1):  # quasi au repos ?
            print('STOP, y=',self.body.position.y, 'vitesse =',self.body.velocity.y)
            pyglet.clock.unschedule(self.update)    # on coupe cette horloge
        else:
            small_dt = 1/250
            x = 0
            while x < dt:
                x += small_dt
                space.step(small_dt)

app = My_pymunk_app(width=600,height=600,caption="exo_pymunk_v1")  # 640 x 480 par défaut
pyglet.app.run()          # bloquant, comme mainloop...
print('Fini')
pyglet.app.exit()

# Dans une version de Spyder, j'ai eu un problème (la balle reste au sol lors d'une
# seconde exécution. Un %reset à la console n'y changeant rien, j'ai du tuer la console
# courante... Hum.

# COMPLEMENTS EN LIGNE
from webbrowser import open as browse
#browse('https://pyglet.readthedocs.io/en/latest/')
#browse('http://www.poketcode.com/en/pyglet/index.html')    # code snippets, Python 2
#browse('https://media.readthedocs.org/pdf/pymunk/latest/pymunk.pdf')
#browse('http://www.pymunk.org/en/latest/index.html')       # API doc, tutorials...
#browse('https://www.youtube.com/watch?v=z_Sx9N39KHk')      # démos :-)


