#!/usr/bin/env python3
# -*- coding: utf-8 -*-

######################################################################
### PYTHON. APPRENTISSAGE ACTIF (JP. ROY, EDITIONS ELLIPSES, 2020) ###
######################################################################

"""
exo_pymunk_rebonds.py (11.130)
SOLUTION de la QUESTION 2 de l'exercice n° 130 du livre.
Corps dynamiques et statiques en Chipmunk.
L'intégration de Pyglet avec la console de Spyder n'est pas complète
à ce jour (Spyder 3.3.6). En cas de problèmes à l'exécution, utilisez
IDLE pour programmer avec pymunk (commande 'idle3' au Terminal),
ou bien exécutez directement au Terminal, ou bien tuez la console courante
de Spyder pour exécuter à nouveau...
Toujours pas de '%gui pyglet' à la console IPython. Ceci peut évoluer...
"""

import pymunk

space = pymunk.Space()
space.gravity = (0,-1000)

def balle_dynamique(masse,rayon,elasticity=0.0,friction=0.0):
    moment = pymunk.moment_for_circle(masse,0,rayon)
    body = pymunk.Body(masse,moment)
    shape = pymunk.Circle(body,rayon)
    shape.elasticity = elasticity
    shape.friction = friction
    space.add(body,shape)
    return body

def boite_dynamique(masse,cote,elasticity=0.0,friction=0.0):
    shape = pymunk.Poly.create_box(None,size=(cote,cote))       # None sera modifié plus bas...
    shape.elasticity = elasticity
    shape.friction = friction
    moment = pymunk.moment_for_poly(masse,shape.get_vertices()) # vertices : sommets, cf la doc
    body = pymunk.Body(masse,moment)
    shape.body = body
    space.add(body,shape)
    return body

def segment_statique(pt1,pt2,epaisseur,elasticity=0.0,friction=0.0):
    shape = pymunk.Segment(space.static_body,pt1,pt2,epaisseur)
    shape.elasticity = elasticity
    shape.friction = friction
    space.add(shape)
    return shape

# Le module pyglet s'occupera du rendu de l'animation.

import pyglet
from pymunk.pyglet_util import DrawOptions   # pour dessiner les formes facilement

class My_pymunk_app(pyglet.window.Window):
    def __init__(self,*args,**kwargs):       # on peut utiliser les options d'init de Window
        pyglet.window.Window.__init__(self,*args,**kwargs)
        pyglet.clock.schedule_interval(self.update,1/60)    # horloge à 60 Hz
        self.draw_options = pymunk.pyglet_util.DrawOptions()
        self.draw_options.flags = self.draw_options.DRAW_SHAPES
        # un corps dynamique de masse 1 (disque de rayon R) est ajouté à l'espace
        self.body1 = balle_dynamique(1,20,0.8,0.3)
        self.body1.position = (115,570)
        # un second corps dynamique, une "boîte" (box) de masse 1 et de côté 40
        self.body2 = boite_dynamique(1,40,0.4,0.9)      # essayez friction = 0.1
        self.body2.position = (400,500)
        # on ajoute 5 segments statiques (ni masse ni moment)
        segments =[segment_statique((80,400),(320,320),3),      # segment oblique
                   segment_statique((0,140),(270,45),3),        # segment oblique
                   segment_statique((600,320),(280,190),3),     # segment oblique
                   segment_statique((0,-1),(600,-1),1,),        # le sol statique
                   segment_statique((480,0),(480,40),30)]       # un bloc statique
        for s in segments :
            s.friction = 0.9 ; s.elasticity = 0.2
        segments[-1].elasticity = 9                             # mais bloc à ressort :-)
        # réglage de la couleur du fond, histoire d'utiliser OpenGL
        pyglet.gl.glClearColor(0.4,0.2,0,1)                     # RGBA

    def on_draw(self):  # redéfinition de la méthode on_draw de la fenêtre courante
        self.clear()
        space.debug_draw(self.draw_options)

    def fini(self):
        if (self.body1.position.y < 21) and (abs(self.body1.velocity.x) < 1):   # balle quasi au repos ?
            return True
        return self.body1.position.y < 0                                        # ou balle dehors

    def update(self,dt):  # callback de l'horloge
        # dt représente le temps en secondes écoulé depuis le dernier appel
        if self.fini():
            print('STOP, y=',self.body1.position.y, 'vitesse =',self.body1.velocity.y)  # la balle
            pyglet.clock.unschedule(self.update)    # on coupe cette horloge
        else:
            small_dt = 1/250
            x = 0
            while x < dt:
                x += small_dt
                space.step(small_dt)

if __name__ == '__main__':
    app = My_pymunk_app(width=600,height=600,caption="exo_pymunk_v1")  # 640 x 480 par défaut
    pyglet.app.run()      # bloquant comme mainloop
    print('Fini')
    pyglet.app.exit()
    
    # COMPLEMENTS EN LIGNE
    from webbrowser import open as browse
    #browse('https://pyglet.readthedocs.io/en/latest/')         # doc pyglet
    #browse('http://www.poketcode.com/en/pyglet/index.html')    # code snippets, mais en Python 2
    #browse('https://media.readthedocs.org/pdf/pymunk/latest/pymunk.pdf')   # doc pymunk
    #browse('http://www.pymunk.org/en/latest/index.html')       # API doc, tutorials...
    #browse('https://www.youtube.com/watch?v=z_Sx9N39KHk')      # démos :-)
    #browse('https://fr.wikipedia.org/wiki/Flipper')            # pour les "bumpers"
    #browse('https://www.youtube.com/watch?v=sTHoWaTNJiA')      # une machine de Goldberg


