#!/usr/bin/env python3
# -*- coding: utf-8 -*-

######################################################################
### PYTHON. APPRENTISSAGE ACTIF (JP. ROY, EDITIONS ELLIPSES, 2020) ###
######################################################################

"""
exo_pymunk_pendule.py (11.131)
Le pendule de Newton.
L'intégration de Pyglet avec la console de Spyder n'est pas complète
à ce jour (Spyder 3.3.6). En cas de problèmes à l'exécution, utilisez
IDLE pour programmer avec pymunk (commande 'idle3' au Terminal),
ou bien exécutez directement au Terminal, ou bien tuez la console courante
de Spyder pour exécuter à nouveau...
Toujours pas de '%gui pyglet' à la console IPython. Ceci peut évoluer...
"""

import pymunk

SPACE = pymunk.Space()
SPACE.gravity = (0,-12800)

LARG = 640  # largeur de la fenêtre

MASSE = 20   # masse d'une boule
RAYON = 25   # rayon d'une boule
MOMENT = pymunk.moment_for_circle(MASSE,0,RAYON)
(X0, Y0) = (LARG/2 - 4*RAYON, 200)    # centre de la première balle

# Construction d'une boule dynamique de centre (x,y) épinglée au corps statique

def boule(x,y):
    body = pymunk.Body(MASSE, MOMENT)
    body.position = (x,y)
    shape = pymunk.Circle(body,RAYON)
    shape.elasticity = 0.9999
    SPACE.add(body,shape)
    # à distance fixe d'une épingle donnée sur le corps statique
    pinjoint = pymunk.constraint.PinJoint(a=SPACE.static_body,b=body,anchor_a=(x,300),anchor_b=(0,0))
    SPACE.add(pinjoint)

# construction des 5 boules
x = X0
for i in range(5):
    boule(x,Y0)
    x += 2*RAYON

# Animation avec pyglet

import pyglet
from pymunk.pyglet_util import DrawOptions     # pour dessiner formes et joints facilement
from pyglet.window import key                  # gestion de la barre espace

class Pendule(pyglet.window.Window):
    def __init__(self,i=1):
        '''On envoie une impulsion à gauche sur les i premières billes parmi les 5
        qui sont toutes de même masse et rayon.'''
        pyglet.window.Window.__init__(self)                         # 640 x 480 par défaut
        self.set_caption('Pendule de Newton')
        pyglet.clock.schedule_interval(self.update,1/60)            # horloge à 60 Hz
        self.draw_options = pymunk.pyglet_util.DrawOptions()
        self.draw_options.flags = self.draw_options.DRAW_SHAPES     # pour les formes
        self.draw_options.flags |= self.draw_options.DRAW_CONSTRAINTS  # pour les joints
        self.i = i                                  # impulsion sur les i premières boules
        self.label = pyglet.text.Label('Tapez <espace> une seule fois',font_size=24,x=10, y=20)

    def on_draw(self):
        self.clear()
        SPACE.debug_draw(self.draw_options)         # pour les formes et les joints
        self.label.draw()                           # un morceau de texte

    def update(self,dt):
        for i in range(13):
            SPACE.step(1/5000)                      # le pas du moteur physique

    def on_key_press(self, symbol, modifiers):
        if symbol == key.SPACE:
            for s in SPACE.shapes[:self.i] :
                s.body.apply_impulse_at_local_point((-15000,0))     # impulsion !
        elif symbol == key.S:     # stopper l'horloge définitivement
            pyglet.clock.unschedule(self.update)

# Avec Spyder, mieux vaut tuer la console courante entre deux exécutions.
# C'est d'ailleurs ce que fait IDLE lorsque l'on demande "Run module", 
# il fait un "Restart shell"...

if __name__ == '__main__':
    window = Pendule(i=4)
    pyglet.app.run()          # bloquant, comme mainloop...
    print('Fini')
    pyglet.app.exit()

    # COMPLEMENTS EN LIGNE
    from webbrowser import open as browse
    #browse('https://www.youtube.com/watch?v=0MEVu_Elvwc')
    #browse('https://fr.wikipedia.org/wiki/Pendule_de_Newton')


