#!/usr/bin/env python3
# -*- coding: utf-8 -*-

######################################################################
### PYTHON. APPRENTISSAGE ACTIF (JP. ROY, EDITIONS ELLIPSES, 2020) ###
######################################################################

"""
exo_racspol.py (11.92)
Recherche des racines simples approchées d'un polynôme sur [a,b],
"""

from exo_polycreux import *         # algèbre des polynômes creux

# Question a)

from exo_bisolve import bisolve

def dichotomie(p,a,b,h):
    '''p est un polynôme creux dont la fonction associée fp est STRICTEMENT
    MONOTONE sur [a,b]. On retourne une approximation à h près de la racine
    x telle que fp(x) = 0, ou bien None s'il n'y en a pas.'''
    fp = poly_fonc(p)
    return bisolve(fp,a,b,h)            # et zou

# Question b)

def derive(p):     # polynôme creux dérivé
    res = poly0()
    for (c,e) in p:
        if e > 0: res.append([e*c,e-1])
    return res
    
# Question c)
# ATTENTION : j'introduis un argument optionnel 'niveau' qui indique le
# niveau de récursion pour me permettre d'afficher un message au lancement
# de la récurrence mais pas à tous les niveaux en profondeur...

def racines(p,a,b,h,niveau=0):                  # p de degre >= 1
    if degre(p) == 0: return []
    # on découpe [a,b] avec les points où la dérivée p' s'annulle
    L = [a] + racines(derive(p),a,b,h,niveau+1) + [b]
    if niveau == 0: 
        print("Découpage aux abscisses :\n",L) # juste pour voir
    res = []
    for i in range(len(L)-1):
        essai = dichotomie(p,L[i],L[i+1],h)    # entre deux lignes en pointillé
        if essai: res.append(essai)
    return res

if __name__ == '__main__':
    # on suppose que a et b ne sont pas déjà des racines et que toutes
    # les racines sont "simples" !
    print("# Recherche par dichotomie d'une racine du polynôme")
    print('# X\u00b2-2 dans [-3,0] où le polynôme est décroissant :')
    print('>>> dichotomie([[1,2],[-2,0]],-3,0,0.001)\n-->',
          dichotomie([[1,2],[-2,0]],-3,0,0.001))
    print('# Et sur [-1,0] ?')
    print('>>> dichotomie([[1,2],[-2,0]],-1,0,0.001)\n-->',
          dichotomie([[1,2],[-2,0]],-1,1,0.001))
    print('# Et sur [0,6] ?')
    print('>>> dichotomie([[1,2],[-2,0]],0,6,0.001)\n-->',
          dichotomie([[1,2],[-2,0]],0,6,0.001))
    
    print('---------------------------------------------')
    
    print('# Soit p le polynôme considéré dans le livre au §11.92,')
    print('# avec 4 racines simples :')
    p = poly_mul([[1,2],[-1,0]],poly_mul([[1,1],[-2,0]],[[1,1],[-3.5,0]]))
    print('>>> p\n-->',p)
    print("# NB. Il s'agit de (X+1)(X-1)(X-2)(X-3.5)  <---- chut !")
    print()

    print('# Racines approchées de p dans [-2,4] avec h=0.001')
    print('>>> racines(p,-2,4,0.001)')
    sols = racines(p,-2,4,0.001)
    print('-->',sols)
    print()
    print('# Racines approchées dans [0,4]')
    print('>>> racines(p,0,4,0.001)')
    sols = racines(p,0,4,0.001)
    print('-->',sols)
    print()
    print('# Racines approchées dans [3,8]')
    print('>>> racines(p,3,8,0.001)')
    sols = racines(p,3,8,0.001)
    print('-->',sols)
    print()
    print('# Racines approchées dans [4,6]')
    print('>>> racines(p,4,6,0.001)')
    sols = racines(p,4,6,0.001)
    print('-->',sols)
    print()
    fp = poly_fonc(p)
    poly_plot(p,-2,4,
              Lpts=[(-2,fp(-2)),
                     (-1,0),
                    (-0.313,fp(-0.313)),
                    (1.49,fp(1.49)),
                    (2.95,fp(2.95)),
                    (3.5,fp(3.5)),
                    (4,fp(4))])
    





    
    
