#!/usr/bin/env python3
# -*- coding: utf-8 -*-

######################################################################
### PYTHON. APPRENTISSAGE ACTIF (JP. ROY, EDITIONS ELLIPSES, 2020) ###
######################################################################

"""
exo_rechdicho.py (11.66)
Recherche dichotomique dans une liste triée.
"""

def rech_dicho_rec(x,LT):       # version 1 : récursive avec tranches de listes
    '''Retourne un indice i tel que L[i] == x (pas nécessairement
    le plus petit s'il ya des répétitions de x. Retourne False si
    x n'est pas dans L.'''
    if LT == []: return False
    m = len(LT) // 2
    if x == LT[m]: return m
    if x < LT[m]: return rech_dicho_rec(x,LT[:m])
    rechdroite = rech_dicho_rec(x,LT[m+1:])
    if not rechdroite: return False        # rappel : None est "faux"...
    return m + 1 + rechdroite   # <— attention !

def rech_dicho(x,LT):           # version 2 : itérative sans tranches
    '''Retourne un indice i tel que L[i] == x (pas nécessairement
    le plus petit s'il ya des répétitions de x. Retourne False si
    x n'est pas dans L.'''
    (a, b) = (0, len(LT) - 1)   # les indices des extrémités
    cpt = 0                     # cpt pour savoir en combien de coups je le trouve !
    while a <= b:
        cpt = cpt + 1
        m = (a + b) // 2
        if LT[m] == x: return (m,cpt)
        if LT[m] > x: b = m - 1
        else: a = m + 1
    return (False,cpt)

# Je vous laisse programmer la version récursive sans tranches...
    
# COMPLEXITE DE LA RECHERCHE DICHOTOMIQUE rech_dicho.
# Soit LT une liste triée en croissant de longueur n. On veut mesurer le nombre
# de comparaisons effectuées. Puisque l'on abandonne chaque fois la moitié
# de la liste en effectuant une comparaison, le coût sera de l'ordre du nombre
# de fois que l'on peut diviser n par 2, soit log(n). L'algorithme de recherche 
# dichotomique est donc de coût LOGARITHMIQUE, en O(log n), à condition que la
# coupure en deux puisse être réalisée en temps constant (ce qui n'est pas le 
# cas de rech_dicho_rec).

    
if __name__ == '__main__':
    L = [0,1,1,2,3,4,5,6,6,8]   # triée !
    print('L =',L)
    print()
    print('Recherche dichotomique par récurrence :')
    print('  Position de 6 dans L :',rech_dicho_rec(6,L))
    print('  Position de 7 dans L :',rech_dicho_rec(7,L))
    print()
    print('Recherche dichotomique itérative :')
    print('  Position de 6 dans L :',rech_dicho(6,L))
    print('  Position de 7 dans L :',rech_dicho(7,L))

    from webbrowser import open as browse
    browse('https://fr.wikipedia.org/wiki/Recherche_dichotomique')

