#!/usr/bin/env python3
# -*- coding: utf-8 -*-

######################################################################
### PYTHON. APPRENTISSAGE ACTIF (JP. ROY, EDITIONS ELLIPSES, 2020) ###
######################################################################

"""
exo_regexp.py (11.110)
Quelques expressions régulières.
"""

import re

# Question a)

def nb_espaces_en_tete(s):
    '''Retourne le nombre d'espaces en tete de la chaine s.'''
    sol = re.search(' *',s)
    return len(sol[0])

# Question b)

def nb_espaces(s):
    '''Retourne le nombre d'espaces en tout dans la chaine s'''
    # et oui, s.count(' ') est clairement plus industriel :-)
    sol = re.findall(' ',s)
    return len(sol) if sol else 0

# Question c)

def premier_mot(s):
    '''Retourne le premier mot (de lettres) de la chaine s'''
    sol = re.search('[a-zA-Z]+',s)
    if sol: return sol[0]

# Question d)

def domaine(url):
    '''Retourne la partie domaine d'une url bien formée'''
    sol = re.search('https?://([^/:]+)',url)
    if sol : return sol[1]

# Question e)

from datetime import date

def inf1000(s):
    '''Retourne True ssi la chaine s est un entier dans [1,999]'''
    # Bug fréquent : [1-999]
    # Séparons en plusieurs tranches de nombres...
    sol = re.search('^([1-9]|[1-9][0-9]|[1-9][0-9][0-9])$',s)
    return bool(sol)        # True ou False


if __name__ == '__main__':
    def test(expr,resultat,numero):
        if expr != resultat:
            print('Bug dans le test {}'.format(numero))
        print('Test',numero,'ok')

    test(nb_espaces_en_tete('une chaîne'),
         0, 1)
    test(nb_espaces_en_tete('     une chaîne'),
         5, 2)
    test(nb_espaces_en_tete(''),
         0, 3)

    test(nb_espaces('les 2 ou 3   bateaux !'),
         7, 4)
    test(nb_espaces('foo'),
         0, 5)

    test(premier_mot('12345Mot!78couic'),
         'Mot', 6)
    test(premier_mot('12345␣!78'),
         None, 7)

    test(all(map(lambda x:inf1000(str(x)),range(1,1000))),
         True, 8)
    test(inf1000('1000'),
         False, 9)
    test(inf1000('0'),
         False, 10)
    test(inf1000('023'),
         False, 11)

    test(domaine('https://foule.coq.vid.uk:5260/play?id=791'),
         'foule.coq.vid.uk', 12)
    test(domaine('http://foule.coq.vid.uk/play?id=791'),
         'foule.coq.vid.uk', 13)

    # COMPLEMENTS EN LIGNE
    from webbrowser import open as browse
    #browse('https://pymotw.com/3/urllib.parse')
    #browse('https://pymotw.com/3/re/')
    #browse('https://docs.python.org/fr/3/library/re.html')


