#!/usr/bin/env python3
# -*- coding: utf-8 -*-

######################################################################
### PYTHON. APPRENTISSAGE ACTIF (JP. ROY, EDITIONS ELLIPSES, 2020) ###
######################################################################

"""
exo_regles.py (11.46)
Mini système de ré-écriture "context-free" (dans le but de
dessiner des fractales avec des L-systèmes, cf exo_tLsyst.py du §11.58).

Une "règle de ré-écriture" s'écrit en Python sous la forme d'un couple (LHS, RHS),
où LHS et RHS sont des chaînes de caractères. Une telle règle exprime que (dans une
chaîne donnée) la sous-chaîne LHS sera remplacée par RHS partout où elle apparaît.
Exemple : ('F', 'F+F-'). L'application de cette règle plusieurs fois de suite en
partant de 'F' produira :
     'F' --> 'F+F-' --> 'F+F-+F+F--' --> 'F+F-+F+F--+F+F-+F+F---' --> etc.
On supposera que LHS est une chaîne à un seul caractère (règle "context-free").
"""

def app_regle(lhs,rhs,s):
    '''Retourne la nouvelle chaine obtenue en appliquant la regle reg partout
    sur la chaine s'''
    return s.replace(lhs,rhs)    # partout !

# Un système de réécriture est une suite de règles (LHS, RHS). La première
# d'entre elles part d'une chaîne traditionnellement notée 'A' (l'axiome).
# Exemple d'une suite de règles, chaque règle est un tuple :
# (('A','F'), ('F','F+F-'), ('+','+-'))

def app_sys(regs,s):
    for reg in regs:
        (lhs, rhs) = reg
        s = app_regle(lhs,rhs,s)
    return s

if __name__ == '__main__':
    print("J'applique deux fois la règle 'F' --> 'F+F-'")
    print("à la chaîne 'aFb' et j'obtiens :",end=' ')
    print(repr(app_regle('F','F+F-',app_regle('F','F+F-','aFb'))))
    print()
    
    SYST = (('A','aFb'),('F','F+F'))
    print("J'applique deux fois le système de réécriture")
    print('   ',SYST)
    print("à la chaîne 'Ab' et j'obtiens :",end=' ')
    print(repr(app_sys(SYST,app_sys(SYST,'Ab'))))
    
    # COMPLEMENTS EN LIGNE
    from webbrowser import open as browse
    #browse('https://fr.wikipedia.org/wiki/Réécriture_(informatique)')
    #browse('https://fr.wikipedia.org/wiki/L-Système')
    