#!/usr/bin/env python3
# -*- coding: utf-8 -*-

######################################################################
### PYTHON. APPRENTISSAGE ACTIF (JP. ROY, EDITIONS ELLIPSES, 2020) ###
######################################################################

"""
exo_repet.py (11.95)
Générateur des éléments distincts d'un objet itérable (dans un ordre non spécifié).
"""

# Rappel : lorsque obj est un objet itérable, alors iter(obj) est un itérateur sur
# les éléments de obj. L'idée ici est d'itérer sur les éléments distincts de obj
# (dans un ordre non spécifié) plutôt que les générer tous avec set(obj).

def sans_doublons(obj):
    '''Retourne un générateur sur les éléments distincts d'un objet itérable obj,
    mais sans construire l'ensemble set(obj), seulement celui des éléments déjà vus.'''
    it = iter(obj)          # un itérateur sur les éléments de obj
    deja_vus = set()        # l'ensemble des éléments déjà vus dans seq
    while True:
        x = next(it)        # lèvera peut-être une exception StopIteration
        if x not in deja_vus:
            deja_vus.add(x)
            yield x

if __name__ == '__main__':
    L = [3,2,4,5,2,2,6,1,4]     # essayez avec 'cocorico' ou {1:2, 3:4, 2:3}
    print('>>> L\n-->',L)
    print('>>> g = sans_doublons(L)')
    g = sans_doublons(L)        # ceci n'est qu'un générateur : une promesse de calcul...
    print('>>> type(g)\n-->',type(g))
    while True:
        try: x = next(g) ; print(x,end=' ')
        except: print('FINI') ; break

    # COMPLEMENTS EN LIGNE
    from webbrowser import open as browse
    #browse('https://docs.python.org/fr/3.7/glossary.html#term-generator')



