#!/usr/bin/env python3
# -*- coding: utf-8 -*-

######################################################################
### PYTHON. APPRENTISSAGE ACTIF (JP. ROY, EDITIONS ELLIPSES, 2020) ###
######################################################################

"""
exo_rsa1.py (11.36)
Introduction à la cryptographie à clé publique
"""

from exo_prime2 import gcd, fermat_premier, randint, puismod
from exo_pgcd import invmod

def rand_premier(k) :
    '''Retourne un nombre Fermat-premier ayant au moins k chiffres'''
    n = randint(10**(k-1),10**k-1)    # un nombre ayant k chiffres
    if n % 2 == 0 : n = n - 1
    while True :        
        if fermat_premier(n) : return n
        n = n + 2

def nouvelles_cles() :
    '''Retourne (e,n,d) avec P=(e,n) publique et d strictement prive.'''
    # on choisit deux grands nombres premiers
    p = rand_premier(randint(20,30))     # (randint(48,50))
    q = rand_premier(randint(20,30))     # randint(48,50))
    # on forme leur produit qui est grand avec deux facteurs premiers
    n = p * q
    # on cherche un petit entier e premier avec phi=(p-1)(q-1)
    phi = (p-1)*(q-1)
    e = 45327681 # 2
    while gcd(e,phi) != 1 : e = e + 1
    # on calcule l'inverse d de e modulo n
    d = invmod(e,phi)
    return (e,n,d) 

def code(msg_clair,e,n) :
    ''' Retourne le codage du message msg (un entier positif) avec la clé publique'''
    return puismod(msg_clair,e,n)

def decode(msg_crypt,d,n) :
    '''Retourne le decodage du message msg (un entier positif) avec la clé privée.'''
    return puismod(msg_crypt,d,n)

if __name__ == '__main__' :
    (e, n, d) = nouvelles_cles()   # Alice : e, n publiques, et d privée 
    print("Clés d'Alice (e,n) publiques et d privée :")
    print('  e = {}\n  n = {}\n  d = {}'.format(e,n,d))
    print()
    
    M = 2021
    print('''Bob veut envoyer à Alice le message 2021 (un entier).
Il le code avec les clés publiques d'Alice (e,n) trouvées 
dans un annuaire public (celui de Vladivostok ?).''')
    C = code(M, e, n)
    print("Le message crypté qu'il obtient pour 2021 est le nombre :\n  ",C)
    print('''Il envoie ce message crypté à Alice (il se peut que
le message soit intercepté, pas grave).''')
    print()
    
    print('Ok, Alice a reçu le message cryté de Bob.')
    print('''Elle va le décoder avec sa clé privée d, que même Bob
ne connaît pas !''')
    D = decode(C, d, n)
    print('Et elle peut alors lire :',D)
    # Bob n'a pas besoin de ses clés pour envoyer un message crypté. Il peut
    # mettre son nom dans le message et l'envoyer de manière anonyme...
    # ====> Il n'est pas pratiquement possible de trouver d à partir de e et n.
    print()
    
    '''
    THEOREME : decode est l'inverse de code. Il faut vérifier que :
    Si :     M = message en clair (un entier positif)
             M1 = code(M,e,n)
             M2 = decode(M1,d,n)
    
    ALORS :  M2 = M
    -------------------
    PREUVE. Résumons ce que l'on sait :
            p,q premiers ; n = p*q ; phi = (p-1)*(q-1) ; pgcd(e,phi) = 1
            e*d = 1 mod n ; M1 = M**e mod n
    Il faut prouver que M = M1**d mod n. Puisque p et q sont premiers entre eux, 
    il suffit de prouver que M = M1**d mod p ET M = M1**d mod q, c'est le 
    "théorème chinois" en arithmétique :
       SI  p,q premiers     
       ALORS [k est multiple de n=p*q] <==> [k multiple de p ET k multiple de q]
    Montrons-le pour p par exemple (idem pour q) : montrons que M = M1**d mod p.
    - si M = 0 mod p, alors M1 = 0 mod p, et l'égalité cherchée est vraie. OK.
    - sinon : e*d = 1 mod m ==> e*d = 1 + k*m = 1 + k*(p-1)*(q-1)
      d'où M**(e*d) = M**(1+k*(p-1)*(q-1)) mod p
                    = M * [M**(p-1)]**(k*(q-1)) mod p
                      mais M**(p-1) = 1 mod p par Fermat,
      donc M**(e*d) = M * 1 mod p = M mod p     CQFD
    '''

    print('And the proof of the theorem seems to be :',
          decode(code(M,e,n),d,n) == code(decode(M,d,n),e,n) == M)
    
    # COMPLEMENTS EN LIGNE
    from webbrowser import open as browse
    browse('https://fr.wikipedia.org/wiki/Arithmétique_modulaire')
    browse('https://fr.wikibooks.org/wiki/Approfondissements_de_lycée/Arithmétique_modulaire_approfondie')
    browse('https://fr.wikipedia.org/wiki/Chiffrement_RSA')
    
    
    






    
    
