#!/usr/bin/env python3
# -*- coding: utf-8 -*-

######################################################################
### PYTHON. APPRENTISSAGE ACTIF (JP. ROY, EDITIONS ELLIPSES, 2020) ###
######################################################################

"""
exo_rsa2.py (11.81)
Introduction à la cryptographie à clé publique.
On code maintenant des chaînes de caractères ASCII.
"""
__all__ = ['rsa_code', 'rsa_decode']     # pour un "from exo_rsa2 import *"

from exo_rsa1 import nouvelles_cles, code, decode
# nouvelles_cles()   --> (e,n,d)
# code(M,e,n)            e,n publiques
# decode(C,d,n)          d privée

# Transformation d'une string ASCII en un entier unique.
# UTF-8 : un caractère ASCII est encodé sur un seul octet (8 bits).

def int256(L):                    # L non vide
    '''Prend une liste L de codes ASCII entiers [a,b,c,...] et retourne
    l'entier n = a + b*256 + c*256**2 + ...'''
    res = 0
    puis = 1
    for k in L:
        res = res + k * puis
        puis = puis * 256         # pour éviter de calculer les puissances
    return res

def str2int256(s):
    '''Prend une chaîne ASCII s et retourne un entier en base 256
    contenant la même information que s'''
    L = list(map(ord,s))    # L est une liste de codes ASCII [a,b,c,...]
    # on convertit la liste d'entiers [a,b,c,...] en un entier en base 256 :
    # a + b*256 + c*256**2 + ... qui contient la même information que L !
    return int256(L)
    
# Codage RSA d'une chaîne ASCII (les caractères Unicode prennent
# un nombre variable d'octets en UTF-8, c'est plus délicat...)

def rsa_code(M,e,n):
    '''Retourne un entier codant la chaîne s avec des clés publiques (e,n)'''
    return code(str2int256(M),e,n)

# Décodage RSA d'un entier et reconstitution d'une chaîne ASCII.

def int2str256(n):
    '''Prend un entier n = n = a + b*256 + c*256**2 + ... et retourne
    la liste [a,b,c,...]. C'est donc l'inverse de str2int256.'''
    res = []
    while n > 0:
        res.append(n % 256)
        n = n // 256
    return ''.join(map(chr,res))

def rsa_decode(C,d,n):     # avec des clés publiques (e,n)
    return int2str256(decode(C,d,n))

if __name__ == '__main__':
    if int256([65,97,33]) != 65 + 97*256 + 33*256**2 :   # == 2187585
        raise ValueError('Bug dans int256')
    if str2int256('Aa!') != 2187585: 
        raise ValueError('Bug dans str2int')
        
    M = 'Attention au Bitcoin !'
    # les clés publiques RSA d'Alice sont visibles par tous !
    e = 5
    n = 3184589369717986715680750677394609430667337706683772242691278326427757695975718154088185780818590997
    print('Bob envoie à Alice le message :\n   ',repr(M))
    print("Il a utilisé pour cela les clés publiques e et n d'Alice.")
    print()
    
    print('Alice reçoit le message crypté :')
    C = rsa_code(M,e,n)
    print(C)
    print('Elle sort sa clé privée d (que Bob ne connaît pas)')
    print('et décode ce message :')
    d = 2547671495774389372544600541915687544533870165346917503445872007755305339255246304937720983397760061
    D = rsa_decode(C,d,n)
    print('   ',repr(D))
    print()
    
    print('Prise de panique, elle vend tous ses BTC.')
    
    # NB. Les clés publiques d'Alice sont visibles par tous, donc n'importe qui
    # peut envoyer un message à Alice en se faisant passer pour Bob !? Ceci est
    # un problème voisin : "la signature numérique" (digital key signature)...
    # Il est résolu. Voir le livre [Cor].
    
    # Voir le fichier du problème d'Agregation 2006 : "rsa_agreg_2006.pdf"
    
    # COMPLEMENTS EN LIGNE
    from webbrowser import open as browse
    #browse('https://tel.archives-ouvertes.fr/tel-00112121/document')
    #browse('https://fr.wikipedia.org/wiki/Arithmétique_modulaire')
    #browse('https://fr.wikipedia.org/wiki/Chiffrement_RSA')
    


