#!/usr/bin/env python3
# -*- coding: utf-8 -*-

######################################################################
### PYTHON. APPRENTISSAGE ACTIF (JP. ROY, EDITIONS ELLIPSES, 2020) ###
######################################################################

"""
exo_rss.py (11.125)
Utilisation de Beautiful Soup (bs4) pour analyser une page Web,
ici un flux RSS récupéré avec Requests.
"""

import requests
import bs4

# la dernière url est prise en compte
urll = 'http://rss.allocine.fr/ac/bandesannonces/anepasmanquer'
urll = 'http://feeds.arstechnica.com/arstechnica/cars'
urll = 'http://rss.cnn.com/rss/cnn_topstories.rss'
urll = 'https://www.canal-u.tv/producteurs/format.rss/producteur.59/podcast?xts=248546&xtor=RSS-1&Rdt=On'
urll = 'http://www.mediapart.fr/journal/podcast/chronique/rss'
url = 'http://news.mit.edu/rss/topic/physics'

reponse = requests.get(url)     # cf § 7.4 du livre...
doc_html = reponse.text
soupe = bs4.BeautifulSoup(doc_html,'html.parser')    # ou 'lxml'...
#soupe = bs4.BeautifulStoneSoup(doc_html,selfClosingTags=['link'])   # voir NB à la fin du fichier

# Bon, j'ai une soupe de tags. Il faut en trouver la structure pour un rendu
# sympathique. Je commence par afficher la soupe pour voir quel est le tag
# qui caractérise chaque news. Ceci fait je remets la ligne 33 en commentaire.

def mettre_le_nez_dans_la_soupe():
    with open('soupe.txt','w',encoding='utf-8') as f_out:
        f_out.write(str(soupe))     # fichier 'soupe.txt' sur disque
    raise Exception('STOP')
#mettre_le_nez_dans_la_soupe()

# Je vois en première ligne que le fichier est en XML (donc structuré en tags), et
# encodé en UTF-8. Tout est dans un tag <rss> contenant un canal <channel> de
# titre <title>, ici "MIT News - Physics".
# Plus bas on trouve un <item>...</item> qui contient le premier article, avec
# un <title> (titre), un <link> (lien), un <description> (résumé), et d'autres
# tags optionnels... Il faut donc se concentrer sur chaque <item>...</item>.

items = soupe.findAll('item')

item_test = soupe.item    # le premier <item>...</item>, regardez sa structure !

# J'obtiens un ensemble itérable d'arbres et je vais transformer chaque arbre
# <item>...</item> en un dictionnaire Python dico, et mettre tous ces
# dictionnaires dico dans une liste L que je pourrai ensuite visualiser (avec
# tkinter ou en générant un fichier HTML comme les lecteurs RSS, etc.)...

# Parfois certains sources RSS utilisent un tag <link/> suivi d'une URL au
# lieu d'un classique <link>...</link>, ce qui est ennuyeux.
# D'où la fonction suivante.
def lien(item):   # au cas où pas de <link>...</link> mais un seul <link/>
    # type(item) == bs4.element.Tag    :    <item>...</item>
    link = item.link
    if link.text == '':     # c'est le cas du <link/> isolé suivi de l'url
        L = item.contents   # on revient à la soupe pour chercher l'url (hum)
        for i,x in enumerate(L):
            if str(x) == '<link/>': return L[i+1]
        return '???'

L = []            # transformation des items en un dictionnaire Python
for item in items:
    # type(item) == <class 'bs4.element.Tag'>
    dico = {
            # obligatoires :
            'title': item.title,
            'description': item.description,
            'link': lien(item),            # cf fonction 'lien' ci-dessus...
            #'link': item.link.text,       # voir NB à la fin du fichier

            # quelques optionnels (None si absents) :
            'pubdate': item.pubdate,
            'image': item.image,
            'enclosure': item.enclosure
    }
    L.append(dico)

# rendu minimaliste du flux RSS en temps réel:

texte = soupe.title.text   # un attribut de la soupe avant les <item> proprement dits.
print(len(texte)*'=')
print(texte)
print(len(texte)*'=','\n')

# Affichage des 6 dernières informations en provenance du flux RSS

for dico in L[:6]:
    #print('item:\n',dico,'\n')   # pour voir un item sous forme de dictionnaire

    # les trois éléments ci-dessous sont en principe obligatoires en RSS :
    print('title:\n',dico['title'].text,'\n')
    print('description:\n',dico['description'].text,'\n')
    print('link:\n',dico['link'],'\n')

    # les éléments suivants sont optionnels :
    if dico['pubdate']: print('pubdate:\n',dico['pubdate'],'\n')
    if dico['image']: print('image:\n',dico['image'],'\n')                     # .jpg ?
    if dico['enclosure']: print('enclosure:\n',dico['enclosure']['url'],'\n')  # MP3, VIDEO ?

    print('---------------\n')

#############################################################################

from webbrowser import open as browse
### un genre de BeautifulSoup spécialisé dans les flux RSS
#browse('https://feedparser.readthedocs.io/en/latest/')        # conda install feedparser

### la doc de BeautifulSoup
#browse('https://beautiful-soup-4.readthedocs.io/en/latest/')  # conda install beautifulsoup4

### la spécification de RSS 2.0
#browse('https://validator.w3.org/feed/docs/rss2.html')

########################## NB. autre solution ###############################

# Pour les flux RSS qui sont en XML mais comportent des tags auto-fermants
# comme <link/>, vous pouvez utiliser la classe BeautifulStoneSoup de bs4
# pour éviter la recherche manuelle ci-dessus avec la fonction 'lien'...
#
# en ligne 22:
# soupe = bs4.BeautifulStoneSoup(doc_html,selfClosingTags=['link'])
#
# en ligne 70:
# 'link': item.link.text,

