#!/usr/bin/env python3
# -*- coding: utf-8 -*-

######################################################################
### PYTHON. APPRENTISSAGE ACTIF (JP. ROY, EDITIONS ELLIPSES, 2020) ###
######################################################################

"""
exo_sdt1.py (11.35)
Régime permanent d'un système dépendant du temps
"""

# NB. Il s'agit d'un exemple simple. Si l'on sait résoudre les équations 
# différentielles du 1er ordre, on trouvera vite la solution exacte 
# f(t) = 1 - exp(-t) qui tend vers 1 lorsque t tend vers +infini... 
# Mais revenons aux principes premiers :

# Tout revient à approximer f(t + dt) connaissant f(t), pour dt petit.
# On revient à la définition d'une dérivée. Pour dt voisin de 0 :
# f'(t) = [f(t + dt) - f(t)] / dt     (en fait la limite lorsque dt -> 0)
# Donc f(t + dt) = f(t) + dt * f'(t) et en portant dans l'équa. diff :
# f(t + dt) = f(t) + dt * (1 - f(t)) = (1 - dt) * f(t) + dt

(t, y) = (0, 0)      # conditions initiales
dt = 0.1             # discrétisation (débit en petits bouts) du temps
for i in range(100):    
    (t, y) = (t + dt, (1 - dt) * y + dt)
    print('y =',y)
# on observe que le système tend à se stabiliser vers la valeur 1

# Il s'agit d'un système analogue à un circuit électrique soumis à une 
# tension si vous avez fait de la physique au lycée. Ou à la décharge d'un 
# condensateur...

# COMPLEMENTS EN LIGNE
# Voir l'avant-dernière figure de la dernière page de :
#   http://www.physagreg.fr/electrocinetique/ec2/EC2-essentiel.pdf
# qui montre la stabilisation de l'intensité du courant.
from webbrowser import open as browse
#browse('http://www.physagreg.fr/electrocinetique/ec2/EC2-essentiel.pdf')

    
    

                
    


    

