#!/usr/bin/env python3
# -*- coding: utf-8 -*-

######################################################################
### PYTHON. APPRENTISSAGE ACTIF (JP. ROY, EDITIONS ELLIPSES, 2020) ###
######################################################################

"""
exo_simplif.py (11.106)
Embryon de simplificateur d'arbres binaires d'expressions
"""

from arbres2 import *

# Les règles sont TRES loin d'être suffisantes... La stratégie pour
# avoir un bon simplificateur est plus compliquée, il faut ordonner
# les expressions, etc.

# petits utilitaires
def est_num(x): return type(x) in (int,float)
def est_noeud(a): return not est_feuille(a)
def est_add(a): return est_noeud(a) and (racine(a) == '+')
def est_sub(a): return est_noeud(a) and (racine(a) == '-')
def est_mul(a): return est_noeud(a) and (racine(a) == '*')
def est_div(a): return est_noeud(a) and (racine(a) == '/')

def simplif(a):                # arbre --> arbre
    '''Retourne l'arbre binaire simplifié de l'arbre
    binaire a (avec certaines règles).'''
    if est_feuille(a) : return a
    r = racine(a)
    if r == '+': return simplif_add(a)
    if r == '-': return simplif_sub(a)
    if r == '*': return simplif_mul(a)
    if r == '/': return simplif_div(a)
    raise ValueError('Opérateur inconnu \{\}'.format(r))

def simplif_add(a):
    '''Retourne l'arbre simplifié d'un noeud de racine +'''
    sg = simplif(fg(a))
    sd = simplif(fd(a))
    if est_num(sg) and est_num(sd): return sg + sd
    if sd == 0: return sg
    if sg == 0: return sd
    if sd == arbre('*',-1,sg): return 0             # a + (-a) == 0
    if sg == arbre('*',-1,sd): return 0             # (-a) + a == 0
    if sg == sd: return arbre('*',2,sg)             # a + a == 2*a
    # et deux règles de factorisation:
    if est_mul(sg) and est_mul(sd) and (fd(sg)==fd(sd)):  # a*c + b*c == (a+b)*c
        return simplif(arbre('*',arbre('+',fg(sg),fg(sd)),fd(sg)))
    if est_mul(sg) and est_mul(sd) and (fg(sg)==fg(sd)):  # a*b + a*c == a*(b+c)
        return simplif(arbre('*',fg(sg),arbre('+',fd(sg),fd(sd))))
    return arbre('+',sg,sd)     # rien d'autre pour l'instant...

def simplif_sub(a):
    '''Retourne l'arbre simplifié d'un noeud de racine -'''
    sg = simplif(fg(a))
    sd = simplif(fd(a))
    if est_num(sg) and est_num(sd) : return sg - sd
    if sd == 0: return sg
    if sg == 0: return simplif(arbre('*',-1,sd))   # comme sympy
    if sg == sd: return 0
    if est_mul(sd) and (fg(sd) == -1): return arbre('+',sg,fd(sd))
    if est_mul(sg) and est_mul(sd) and (fd(sg)==fd(sd)):
        return simplif(arbre('*',arbre('-',fg(sg),fg(sd)),fd(sg)))  # a*c+b*c == (a+b)*c
    if est_mul(sg) and est_mul(sd) and (fd(sg)==fd(sd)):
        return simplif(arbre('*',arbre('-',fg(sg),fg(sd)),fd(sg)))  # a*b+a*c == a*(b+c)
    return arbre('-',sg,sd)     # rien d'autre pour l'instant...

def simplif_mul(a):
    '''Retourne l'arbre simplifié d'un noeud de racine *'''
    sg = simplif(fg(a))
    sd = simplif(fd(a))
    if est_num(sg) and est_num(sd): return sg * sd
    if (sd == 0) or (sg == 0): return 0
    if sg == 1: return sd
    if sd == 1: return sg
    if est_div(sd) and (fd(sd) == sg): return fg(sd)  # a*(b/a) == b
    if est_div(sg) and (fd(sg) == sd): return fg(sg)  # (a/b)*b == a
    if (sg == -1) and est_mul(sd):                    # -(a*b) == (-a)*b
        return simplif(arbre('*',arbre('*',-1,fg(sd)),fd(sd)))
    if (sg == -1) and est_mul(sd) and (fg(sd) == -1): return fd(sd) # -(-a) == a
    if est_mul(sd):                                   # a*(b*c) == (a*b)*c
        return simplif(arbre('*',arbre('*',sg,fg(sd)),fd(sd)))
    return arbre('*',sg,sd)     # rien d'autre pour l'instant...

def simplif_div(a):
    '''Retourne l'arbre simplifié d'un noeud de racine /'''
    sg = simplif(fg(a))
    sd = simplif(fd(a))
    if est_num(sg) and est_num(sd): return sg / sd
    if sg == 0 : return 0
    if sd == 0 : return sd
    if sd == -1: return arbre('*',-1,sg)
    if sg == sd: return 1
    if est_div(sd) and (fg(sd) == sg) : return fd(sd)  # a/(a/b) == b
    if est_div(sg) and (fg(sg) == sd) : return simplif(arbre('/',1,fd(sg)))  # (a/b)/a == 1/b
    if est_mul(sg) and est_mul(sd) and (fg(sg) == fg(sd)): # (a*b)/(a*c) == b/c
        return simplif(arbre('/',fd(sg),fd(sd)))
    if est_mul(sg) and est_mul(sd) and (fd(sg) == fd(sd)): # (a*b)/(c*b) == a/c
        return simplif(arbre('/',fg(sg),fg(sd)))
    if est_mul(sd) and (sg == fg(sd)):                   # x/(x*y) == 1/y
        return simplif(arbre('/',1,fg(sd)))
    return arbre('/',sg,sd)     # rien d'autre pour l'instant...

# RAJOUTEZ TOUTES LES REGLES QUE VOUS TROUVEREZ...

if __name__ == '__main__':
    print('La simplification symbolique.')
    print()
    print('# Test avec + -')
    a1 = ['-',['+','x',['-',['+',1,2],3]],'x']
    print('>>> a1\n-->',a1)
    print('>>> simplif(a1)\n-->',simplif(a1))
    print()
    print('# Tests avec + - * /')
    a2 = ['*',['/',2,'x'],['+',['+',1,-1],'x']]
    print('>>> a2\n-->',a2)
    print('>>> simplif(a2)\n-->',simplif(a2))
    print()
    a3 = ['-',['*',3,'x'],['*',['+',1,1],'x']]
    print('>>> a3\n-->',a3)
    print('>>> simplif(a3)\n-->',simplif(a3))
    print()
    a4 = ['/',['+','x',1],['+',0,['/',['+','x',1],['-','y',2]]]]
    print('>>> a4\n-->',a4)
    print('>>> simplif(a4)\n-->',simplif(a4))
    # dans l'exemple ci-dessous, le fait de remplacer le fg x+1 par 1+x
    # montre l'intérêt d'ordonner les expressions !...
    a4bis = ['/',['+',1,'x'],['+',0,['/',['+','x',1],['-','y',2]]]]
    print('>>> a4bis\n-->',a4bis)
    print('>>> simplif(a4bis)\n-->',simplif(a4bis))

    # COMPLEMENTS EN LIGNE
    from webbrowser import open as browse
    # Au début était est la naissance des expressions symboliques avec le
    # langage LISP vers 1958 et l'article fondateur (assez facile à lire)
    # de John McCarthy :
    #browse('http://www-formal.stanford.edu/jmc/recursive.ps')
    
    # Une référence historique (plus spécialisée) est l'un des premiers articles
    # sur le problème de la simplification symbolique :
    #browse('https://groups.csail.mit.edu/mac/users/gjs/6.945/readings/simplification/moses-simp.pdf')
    
    # Ouvrez le site web de l'imposant SAGE qui contient SYMPY
    # pour travailler directement avec Sage-Web :
    #browse('https://www.sagemath.org')
    
    # Et à destination des collèges et lycées :
    #browse('http://www.tangentex.com/CalculSymbolique.htm')
    
    # Enfin, la dernière est une généralité sur la notion de "CAS" :
    #browse('https://fr.wikipedia.org/wiki/Système_de_calcul_formel')
    # mais la version Wikipedia anglaise "Computer algebra" est meilleure...

    # Le théorème de Richardson (1968) a montré qu'il ne pouvait exister
    # un algorithme décidant de l'égalité de deux formules mathématiques.
    # Les systèmes de calcul formel sont donc amenés à faire des choix
    # ou à se faire guider par l'utilisateur...
    
    # Les arbres d'expression sont à ce point importants comme représentation
    # interne que des chercheurs français de la division IA de Facebook 
    # viennent de les utiliser dans le cadre des réseaux de neurones (pages 
    # 295-303) pour faire effectuer au réseau des calculs symboliques, ex:
    # calcul d'intégrales indéfinies.
    # Cela pourrait être une belle avancée dans le deep learning :
    #browse('https://www.quantamagazine.org/symbolic-mathematics-finally-yields-to-neural-networks-20200520/')
    #browse('https://arxiv.org/pdf/1912.01412.pdf')

