#!/usr/bin/env python3
# -*- coding: utf-8 -*-

######################################################################
### PYTHON. APPRENTISSAGE ACTIF (JP. ROY, EDITIONS ELLIPSES, 2020) ###
######################################################################

"""
exo_sinus.py (11.27)
Approximation de sin(x) par sa série de Taylor
"""

from math import factorial, sin, pi

def sinus_v1(x,n):         # x - x**3/3! + x**5/5! + ... jusqu'à  x**(2n+1)
    res = 0
    for k in range(n+1):
        res = res + (-1)**k * x**(2*k+1)/factorial(2*k+1)
    return res

n = 4                      # calcul des n premiers termes de la somme
print('Test de sinus(x,4) avec',n,'termes dans la somme.\n')
print('Test de sinus_v1(pi/3,{}) : {}'.format(n,sinus_v1(pi/3,n)))   
print('  alors que sin(pi/3) vaut',sin(pi/3))
print()

def sinus_v2(x,n) :        # idem avec une compréhension de somme
    return sum((-1)**k * x**(2*k+1)/factorial(2*k+1) for k in range(n+1))

print('Test de sinus_v2(pi/3,{}) : {}'.format(n,sinus_v2(pi/3,n))) 
print('  alors que sin(pi/3) vaut',sin(pi/3))
print()

# Accélération du calcul de la somme.
# on évite les recalculs de ** et factorial.

def sinus(x,n) :          # jusqu'à la puissance 2n+1 incluse
    res = 0               # le résultat
    signe = 1             # éviter le ** dans (-1)**k
    puis = x              # éviter le ** dans x**(2*k+1)
    fact = 1              # éviter factorial qui recalcule tout chaque fois
    x2 = x * x            # éviter ce petit recalcul, un sou est un sou
    for k in range(n+1) :
        res = res + signe * puis / fact
        puis = puis * x2
        fact = fact * (2*k+2) * (2*k+3)
        signe = -signe
    return res
        
print('Version accélérée')     # O(n) multiplications !
print('  de sinus(pi/3,{})       : {}'.format(n,sinus(pi/3,n)))

# Ceci dit, on ne calcule jamais beaucoup de termes dans la somme.
# Mais bon, pour le principe...

# COMPLEMENTS EN LIGNE
from webbrowser import open as browse
#browse('https://fr.wikipedia.org/wiki/Sinus_(mathématiques)')