#!/usr/bin/env python3
# -*- coding: utf-8 -*-

######################################################################
### PYTHON. APPRENTISSAGE ACTIF (JP. ROY, EDITIONS ELLIPSES, 2020) ###
######################################################################

"""
exo_steg.py (11.85)
Stéganographie : cacher un message dans une image.
Ce fichier est le code du §4.3 du livre et doit être
complété pour résoudre l'exercice du §11.85 du livre.
La solution est dans le fichier 'exo_steg_sol.py'...
"""

from PIL import Image, ImageDraw, ImageFont
FONT = '/Library/Fonts/Arial.ttf'   # sur MacOS, adaptez à votre machine...

def encode_image(msg, f_source, f_but):
    '''Le message va être caché dans une copie de l'image contenue dans le
    fichier source. Le résultat sera placé dans le fichier but.'''
    im = Image.open(f_source)
    (largeur,hauteur) = im.size
    print('Image à encoder :',len(im.split()),'couches, dimensions :',largeur,hauteur)
    im_msg = graver(msg, im.size)  # message noir sur fond blanc
    im_msg.show()                 # pour voir le message gravé sur fond noir
    (im_r,im_g,im_b) = im.split()  # les 3 couches RGB
    res = Image.new('RGB',im.size) # image résultat à placer dans f_but
    pixels = res.load()            # une zone de pixels en mémoire (pixmap)
    for i in range(largeur):       # parcours de la matrice de pixels
        for j in range(hauteur):
            # on force le LSB de chaque pixel DU PLAN ROUGE à être le même que le
            # LSB de l'image-texte (0 ou 1) :
            pixel = im_r.getpixel((i,j))            # l'entier 0b1111010
            pixel = bin(pixel)                      # '0b1111010' dont le LSB est 0
            LSB = bin(im_msg.getpixel((i,j)))[-1]   # le bit le plus à droite
            if LSB == '1': pixel = pixel[:-1] + '1' # remplacement du LSB
            else : pixel = pixel[:-1] + '0'
            pixels[i,j] = (int(pixel,2),im_g.getpixel((i,j)),im_b.getpixel((i,j)))
    res.save(f_but)

def graver(msg,dimensions):
    '''Retourne une image noire avec le texte gravé en blanc, au format RVB.'''
    print('Gravure du message',repr(msg),'dans une image',dimensions)
    fnt = ImageFont.truetype(FONT, 24)       # Arial 24
    im = Image.new('RGB',dimensions)
    graveur = ImageDraw.Draw(im)             # outil pour graver du texte dans im
    graveur.text((10,10),text=msg,font=fnt)  # gravure à partir de (10,10) sur fond blanc
    return im.convert(mode='1')              # noir/blanc, 1 bit/pixel

def decode_image(source,but):
    '''Décode dans l'image but le message caché dans l'image source.
    Ceci est un exercice du livre. A compléter !'''
    raise ValueError('IL VOUS FAUT PROGRAMMER LA FONCTION decode_image !\n')

if __name__ == '__main__':
    Image.open('lena.png').show()           # la photo originale de Lena
    encode_image('FOO & BAR','lena.png','lena_steg.png')   # construction de la photo cryptée lena_steg
    Image.open('lena_steg.png').show()      # et voici la photo cryptée. Aucune différence ?...
    decode_image('lena_steg.png','lena_msg.png')           # extraction du message caché dans lena_steg
    Image.open('lena_msg.png').show()       # et voici le message caché !

    from webbrowser import open as browse
    #browse('https://python-pillow.org')
    #browse('https://pillow.readthedocs.io/en/latest/handbook/index.html')





