#!/usr/bin/env python3
# -*- coding: utf-8 -*-

######################################################################
### PYTHON. APPRENTISSAGE ACTIF (JP. ROY, EDITIONS ELLIPSES, 2020) ###
######################################################################

"""
exo_steg_sol.py (11.85)
Stéganographie : cacher un message dans une image. SOLUTION.
"""

from PIL import Image, ImageDraw, ImageFont
ARIAL = '/Library/Fonts/Arial.ttf'   # sur MacOS, adaptez à votre machine...
ARIAL24 = ImageFont.truetype(ARIAL, 24)     # Arial en taille 24

def graver(msg,dimensions):
    '''Retourne une image noire avec le texte en blanc, au format RVB.'''
    print('Gravure du message',repr(msg),'en',dimensions)
    im = Image.new('RGB',dimensions)
    graveur = ImageDraw.Draw(im)            # outil pour graver du texte dans im
    graveur.text((10,110),text=msg,font=ARIAL24) # gravure à partir de (10,110)
    return im.convert(mode='1')             # noir/blanc, 1 bit/pixel

def encode_image(msg, f_source, f_but):
    '''Le message va être caché dans une copie de l'image contenue dans le fichier source.
       Le résultat sera placé dans le fichier but.'''
    im = Image.open(f_source)
    (largeur,hauteur) = im.size
    print('Image à encoder :',len(im.split()),
          'couches, dimensions',(largeur,hauteur))
    im_msg = graver(msg, im.size)  # message noir sur fond blanc
    im_msg.show()                  # pour voir le message gravé sur fond noir
    (im_r,im_g,im_b) = im.split()  # les trois couches RGB
    res = Image.new('RGB',im.size) # image résultat à placer dans f_but
    pixels = res.load()            # pixels est de classe PixelAccess
    # Un objet de type PixelAccess est une zone de pixels en mémoire
    # accessible par pixels[i,j] au lieu de pixels[i][j] comme dans les listes.
    for i in range(largeur):       # parcours de la matrice de pixels
        for j in range(hauteur):
            # on force le LSB de chaque pixel DU PLAN ROUGE à être le même que le
            # LSB de l'image-texte (0 ou 1) :
            pixel = im_r.getpixel((i,j))            # l'entier 0b1111010
            pixel = bin(pixel)                      # '0b1111010' dont le LSB est 0
            LSB = bin(im_msg.getpixel((i,j)))[-1]   # le bit le plus à droite
            if LSB == '1': pixel = pixel[:-1] + '1' # remplacement du LSB
            else : pixel = pixel[:-1] + '0'
            pixels[i,j] = (int(pixel,2),im_g.getpixel((i,j)),im_b.getpixel((i,j)))
    res.save(f_but)

def decode_image(source,but):      # CORRECTION...
    '''Décode dans l'image but le message caché dans l'image source'''
    print('Décodage du message caché dans',source)
    encoded_image = Image.open(source)
    red_channel = encoded_image.split()[0]
    (largeur,hauteur) = encoded_image.size
    decoded_image = Image.new("RGB", encoded_image.size)
    pixels = decoded_image.load()
    for i in range(largeur) :
        for j in range(hauteur) :
            if bin(red_channel.getpixel((i, j)))[-1] == '0':
                pixels[i, j] = (255, 255, 255)
            else:
                pixels[i, j] = (0,0,0)
    decoded_image.save(but)

if __name__ == '__main__':
    encode_image('FOO & BAR','lena.png','lena_steg.png')
    Image.open('lena.png').show()
    Image.open('lena_steg.png').show()
    decode_image('lena_steg.png','lena_msg.png')
    Image.open('lena_msg.png').show()

    # Toujours la doc sous les yeux !
    from webbrowser import open as browse
    #browse('https://pillow.readthedocs.io/en/latest/handbook/index.html')
    #browse('https://pillow.readthedocs.io/en/latest/reference/PixelAccess.html')
    #browse('https://python-pillow.org')





