#!/usr/bin/env python3
# -*- coding: utf-8 -*-

######################################################################
### PYTHON. APPRENTISSAGE ACTIF (JP. ROY, EDITIONS ELLIPSES, 2020) ###
######################################################################

"""
exo_Lsyst.py (11.58)
Tracé d'une fractale comme "L-système" (système de Lindenmayer, avec des
règles de réécriture).
"""

from turtle_init import *    # fichier d'utilitaires tortue qui ajoute :
# from turtle import *
# init(x=0,y=0,cap=0)
# afficher(msg,x,y)
# sleep(dt)            en secondes flottantes
# deg_rad(a_degrés)    conversion degrés -> radians
# rot(M,K,a_degrés)    K = centre de rotation
# segment(M1,M2)       le tracé d'un segment
# pi, sqrt, sin, cos, randint   pour les calculs fréquents

from exo_regles import *        # app_regle(lhs,rhs,s) et app_sys(regs,s)

def exec_sys(regs,n,u,a):
    '''Dessine la courbe obtenue en appliquant n fois le L-systeme regs a la
    chaine "A", avec un segment de longueur u et un angle a pour "+" vers la
    gauche et pour "-" vers la droite, en degrés. Le résultat est la longueur
    de la chaine contenant les instructions du tracé de la courbe.'''
    # construction du code
    s = 'A'
    for i in range(n) :
        s = app_sys(regs,s)
    # exécution du code engendré
    for c in s:
        if c == 'F': forward(u)
        elif c == '+': left(a)
        elif c == '-': right(a)
        else: raise ValueError('instruction inconnue : ' + c)
    return len(s)

if __name__ == '__main__':
    SYST = (('A','F'),('F','+F--F+'))
    print('Approximant de niveau 5 de la courbe du dragon')
    getcanvas().focus_force()     # getcanvas() --> la fenêtre graphique
    clear()
    init(-110,-100,0)
    afficher('Le dragon vu comme "L-système"',0,220)
    afficher('(A --> F, F --> +F--F+)',0,185)
    ht()
    tracer(False)
    n = exec_sys(SYST,11,5,45)
    tracer(True)
    print(n,'segments ont été tracés !')
    mainloop()
    print('Fini')


# Vous généraliserez sans doute la fonction exec_sys en autorisant d'autres
# symboles que 'F', '+' et '-', par exemple '[' et ']' qui sauve ou restaure
# l'état de la tortue, pour modéliser des effets de branchements (ex: arbres).
# Autre direction : paramétrer les instructions F(10) pour forward(10) par ex.
# On pourrait aussi transformer la chaîne d'instructions en véritable code
# turtle Python, que l'on passe à eval ou exec... Etc.

# COMPLEMENTS EN LIGNE
# Revoir l'exercice "règles de réécriture" exo_regles.py...
from webbrowser import open as browse
browse('http://weitz.de/lindenmayer/')
browse('https://allenpike.com/modeling-plants-with-l-systems/')
browse('https://fr.wikipedia.org/wiki/L-Système')






