#!/usr/bin/env python3
# -*- coding: utf-8 -*-

######################################################################
### PYTHON. APPRENTISSAGE ACTIF (JP. ROY, EDITIONS ELLIPSES, 2020) ###
######################################################################

"""
exo_tcercle.py (11.51)
Dessiner un cercle approché par un polygone régulier.
On donne le centre et le rayon.
"""

# -------------------------------------------------------------

from turtle_init import *    # fichier d'utilitaires tortue qui ajoute :
# from turtle import *
# init(x=0,y=0,cap=0)
# afficher(msg,x,y)
# sleep(dt)            en secondes flottantes
# deg_rad(a_degrés)    conversion degrés -> radians
# rot(M,K,a_degrés)    K = centre de rotation
# segment(M1,M2)       le tracé d'un segment
# pi, sqrt, sin, cos, randint   pour les calculs fréquents

#--------------------------- POLAIRE --------------------------
from cmath import rect    # pour construire un complexe (r,theta)

def cercle_polaire(x,y,r):
    '''Dessine un cercle de centre (x,y) et de rayon r, approché par
    un polygone régulier à 360 côtés. Version polaire : la tortue avance
    et tourne.'''
    M = pos()             # sauvegarde de la position initiale
    cap = heading()       # et du cap initial
    angle = 0.5           # 0.5 degré (le rayon diminue si l'angle augmente)
    angle_rad = deg_rad(angle)
    coté = r * sqrt(2*(1-cos(angle_rad)))
    up() ; goto(x,y-r) ; down() ; setheading(angle)
    for i in range(720):
        forward(coté)
        left(angle)
    up() ; goto(M) ; down() ; setheading(cap)

def cercle_cartésien_v1(a,b,r,n=40):
    '''Dessine un cercle de centre (x,y) et de rayon r, approché par
    un polygone régulier à n côtés. Version cartésienne : on calcule
    les sommets du polygone avec de la trigo.'''
    # on approche le cercle par un polygone régulier à n sommets, donc
    # essentiellement on se promène sur les racines n-èmes de 1. Etat
    # de la tortue invariant.
    M = pos()
    alpha = 2*pi / 60
    up() ; goto(a + r,b) ; down()      # depart au pole Est du cercle
    for k in range(61):               # pour k = 0,...,60
        goto(a + r*cos(k*alpha), b + r*sin(k*alpha))
    up() ; goto(M) ; down()

def cercle_cartésien_v2(x,y,r,n=40):
    '''Dessine un cercle de centre (x,y) et de rayon r, approché par
    un polygone régulier à n côtés. Version cartésienne : on calcule
    les sommets complexes du polygone par des rotations.'''
    cgoto = lambda z: goto(z.real, z.imag)        # goto dans le plan complexe
    M = pos()             # sauvegarde de la position initiale
    u = rect(1,2*pi/n)    # complexe de module 1 pour une rotation
    z0 = complex(x,y)     # centre du cercle
    z = z0 + r            # sommet au pôle Est du cercle
    up() ; cgoto(z) ; down()
    for i in range(n):
        zs = z0 + u * (z - z0)  # expression complexe d'une rotation de centre z0
        cgoto(zs)
        z = zs
    up() ; goto(M) ; down()

# tracé rapide
getcanvas().focus_force()    # getcanvas() --> la fenêtre graphique
tracer(False)
reset()
R = 100
afficher('POLAIRE',-100,270) ; afficher('n=720',-100,240)
cercle_polaire(-100,0,R)
init(-100,0,0) ; forward(R) ; left(90) ; forward(2*R)  # vérif. du rayon
pencolor("red")
afficher('CARTESIEN',130,270) ; afficher('n=40',130,240)
cercle_cartésien_v2(100,0,R)
init(100,0,0) ; forward(R) ; left(90) ; forward(2*R)   # vérif. du rayon
tracer(True)

mainloop()      # pour une exécution au Terminal
print('Fini')

# COMPLEMENTS EN LIGNE
# Ceci dit, c'est un bon entraînement tortue, mais les logiciels graphiques
# professionnels utilisent des algorithmes bien plus sophistiqués comme
# celui de Bresenham par exemple.
from webbrowser import open as browse
browse("https://fr.wikipedia.org/wiki/Algorithme_de_tracé_d'arc_de_cercle_de_Bresenham")
