#!/usr/bin/env python3
# -*- coding: utf-8 -*-

######################################################################
### PYTHON. APPRENTISSAGE ACTIF (JP. ROY, EDITIONS ELLIPSES, 2020) ###
######################################################################

"""
exo_tflaque.py (11.57)
Animation avec un (minuscule) simulacre d'intelligence artificielle
"""

# -------------------------------------------------------------

from turtle_init import *    # fichier d'utilitaires tortue qui ajoute :
# from turtle import *
# init(x=0,y=0,cap=0)
# afficher(msg,x,y)
# sleep(dt)            en secondes flottantes
# deg_rad(a_degrés)    conversion degrés -> radians
# rot(M,K,a_degrés)    K = centre de rotation
# segment(M1,M2)       le tracé d'un segment
# pi, sqrt, sin, cos, randint   pour les calculs fréquents

# Des attracteurs circulaires (flaques de liquide odorant) sont déposés dans
# le canvas graphique. Une tortue aveugle est attirée par l'odeur de chacun
# de ces attracteurs mais se déplace (presque) au hasard à vitesse constante v.

# Il est raisonnable d'avoir le moins possible de variables globales
# dans un programme.

v = 4             # vitesse de la tortue en variable globale

def nouvelle_flaque():       # ajoutez de la couleur sur les flaques !
    p = pos()     # sauvegarde de la position courante
    flaque = (randint(-200,200),randint(-200,200))   # pas trop près des bords
    up() ; ht()
    goto(flaque) ; dot(40) ; goto(p)
    st()
    return flaque

def distance_a_la_flaque(f):
    (x1,y1) = pos()
    (x2,y2) = f
    return abs(x1 - x2) + abs(y1 - y2)
# Il s'agit d'une distance autre que la "distance euclidienne".
# Pour cette distance plus rapide à calculer qui ne fait que des additions et
# soustractions, l'ensemble des points situés à même distance d'un point fixe
# n'est plus un cercle mais un losange...

def faire_un_pas_vers_la_flaque(f):
    if randint(1,10) == 1:            # avec 1 chance sur 10,
        setheading(towards(f))        # on regarde en direction de la flaque
    else: left(randint(-20,20))      # sinon on se dirige au hasard
    forward(v)                        # avec une vitesse v
    if en_dehors(): reviens()         # oups, personne ne sort du canevas !

def en_dehors():     # la tortue sort-elle du canvas ?
    (x,y) = pos()
    return (abs(x) > 280) or (abs(y) > 280)

def reviens():       # on la recolle au bord !
    (x,y) = pos()
    if x < -280 : goto(-280,y)
    elif x > 280: goto(280,y)
    if y < -280 : goto(x,-280)
    elif y > 280: goto(x,280)

def animation():
    while True:      # stoppez l'animation en fermant la fenêtre !
        f = nouvelle_flaque()
        while distance_a_la_flaque(f) > 5:
            faire_un_pas_vers_la_flaque(f)

# Mieux vaudrait utiliser la méthode '.after' du canvas de la tortue, en utilisant
# une "RawTurtle" (§ 9.1.10).

getcanvas().focus_force()
ht() ; up() ; goto(-250,250) ; down() ; setheading(135)
shape('turtle')
bgcolor('green')
color(0.2,0.2,0.2)   # gris sombre (le "colormode" est "1.0" par défaut)
animation()

mainloop()




