#!/usr/bin/env python3
# -*- coding: utf-8 -*-

######################################################################
### PYTHON. APPRENTISSAGE ACTIF (JP. ROY, EDITIONS ELLIPSES, 2020) ###
######################################################################

"""exo_trifus.py (11.77)
Tri par fusion par copie d'une liste de nombres.
(mergesort en anglais). Bien meilleur que tri_sel et tri_ins !
"""

def scission(L):         # je coupe au milieu, pas le plus efficace...
    n = len(L)
    return (L[:n//2], L[n//2:])     # mais c'est trop tentant :-)

def fusion(LT1,LT2):     # LT1 et LT2 triées croissantes, résultat trié croissant
    res = []
    i1 = i2 = 0          # i1 pointe au début de LT1, i2 au début de LT2
    while i1 < len(LT1) and i2 < len(LT2) :   # tant que i1 et i2 sont vivants...
        if LT1[i1] <= LT2[i2]: res.append(LT1[i1]) ; i1 = i1 + 1
        else: res.append(LT2[i2]) ; i2 = i2 + 1
    # je dois vérifier pourquoi je suis sorti de la boucle while !
    if i1 == len(LT1): res.extend(LT2[i2:])
    else : res.extend(LT1[i1:])
    return res
        
def tri_fusion(L):
    n = len(L)
    if n <= 1: return L      # Attention au cas de base, n == 0 ne suffit pas !
    (L1, L2) = scission(L)
    return fusion(tri_fusion(L1),tri_fusion(L2))

if __name__ == '__main__':
    L = [8,5,2,3,1,5,4,9,2]
    print('Test de la scission : ',end='')
    if scission(L) != ([8,5,2,3], [1,5,4,9,2]):
        raise ValueError('Bug dans scission !')
    print('Ok')
    
    LT1 = [2, 5, 6, 6, 8, 12, 20]
    LT2 = [0, 2, 7, 8, 15, 19, 23]
    print('Test de la fusion : ',end='')
    if fusion(LT1,LT2) != [0,2,2,5,6,6,7,8,8,12,15,19,20,23] :
        raise ValueError('Bug dans fusion !')
    print('Ok')
    
    LT = tri_fusion(L)
    print('Test de tri_fusion : ',end='')
    if tri_fusion(L) != [1,2,2,3,4,5,5,8,9]:
        raise ValueError('Bug dans tri_fusion !')
    print('Ok\n')
    
    from exo_chrono import test_chrono
    print('Temps de tri_fusion pour 100_000 et 200_000 éléments :')
    print(test_chrono(tri_fusion,100_000))
    print('Résultats en accord avec un tri en O(n log n) ?...')
    print()
    
    print('Les tris primitifs .sort et sorted de Python restent')
    print('les plus rapides. Pour 100_000 et 200_000 éléments :')
    print(test_chrono(lambda L : L.sort(),1000_000))
    print()

    # Les équations de complexité s'écrivent :
    #     c_0 == c_1 == 0  et  c_n == 2c_{n/2} + O(n)
    # et le "Master Theorem" donne c_n == O(n log n).
    # Mais on peut le retrouver à la main en déroulant ces équations...
    from webbrowser import open as browse
    #browse('https://www.toptal.com/developers/sorting-algorithms')
    #browse('https://fr.wikipedia.org/wiki/Master_theorem')
    #browse('https://fr.wikipedia.org/wiki/Tri_fusion')
    
