#!/usr/bin/env python3
# -*- coding: utf-8 -*-

######################################################################
### PYTHON. APPRENTISSAGE ACTIF (JP. ROY, EDITIONS ELLIPSES, 2020) ###
######################################################################

"""
exo_triins.py (11.76)
Le tri par insertion d'une liste de nombres.
"""

from exo_polycreux import insertion # insertion(x,LT) dans une liste triée

def tri_ins_v1(L):                  # Récurrence sur la longueur de L
    '''Retourne une copie triée en croissant - par insertion - de la liste L.'''
    if len(L) == 0: return L
    return insertion(L[0],tri_ins_v1(L[1:]))   # coût(n) = O(n) + cout(n-1) = O(n**2)

def trier_ins_v2(L,i):              # Récurrence sur un indice de liste
    '''Trie sur place la liste L, par insertion et récursivement jusqu'a
    l'indice i inclus. Aucun résultat. Appel avec i = len(L)-1'''
    if i > 0:
        trier_ins_v2(L,i-1)         # hyp. de réc. : je trie sur place la partie L[:i]
        # il suffit d'insérer L[i] dans L[:i] en le faisant glisser vers la gauche
        while (i > 0) and (L[i-1] > L[i]):
            (L[i-1],L[i]) = (L[i],L[i-1])
            i = i - 1

def trier_ins_v3(L):               # itération, sur place
    '''Trie sur place itérativement - par insertion - la liste L. Aucun résultat'''
    # stratégie identique à la récurrence, mais l'hypthèse de récurrence devient
    # un invariant de la boucle : la partie 0..i-1 est déjà triée. Pour i = n, on a
    # donc le résultat.
    for i in range(1,len(L)):        # pour n il n'y a rien à faire !
        # invariant : la partie 0..i-1 est triée
        j = i                      # faire glisser L[i] vers la gauche à sa place
        while (j > 0) and (L[j-1] > L[j]):
            (L[j-1],L[j]) = (L[j],L[j-1])
            j = j - 1

def trier_ins_v3(L,trace=False):   # itération, sur place
    '''Trie sur place itérativement - par insertion - la liste L. Aucun résultat'''
    # stratégie identique à la récurrence, mais l'hypthèse de récurrence devient
    # un invariant de la boucle : la partie 0..i-1 est déjà triée. Pour i = n, on a
    # donc le résultat.
    if trace: print(L)
    for i in range(1,len(L)):        # pour n il n'y a rien à faire !
        # invariant : la partie 0..i-1 est triée
        j = i                      # faire glisser L[i] vers la gauche à sa place
        while (j > 0) and (L[j-1] > L[j]):
            (L[j-1],L[j]) = (L[j],L[j-1])
            j = j - 1
        if trace: print(L)

if __name__ == '__main__':
    L = [6,3,7,1,2,3,8,5]
    print('>>> L\n-->',L)
    print('>>> tri_ins_v1(L)           # récursif avec copie\n-->',tri_ins_v1(L))
    print('>>> L\n-->',L)
    print()
    res = trier_ins_v2(L,len(L)-1)
    print('>>> trier_ins_v2(L,len(L))  # récursif sur place')
    print('>>> L\n-->',L)
    print()
    L = [6,3,7,1,2,3,8,5]
    print('>>> L = [6, 3, 7, 1, 2, 3, 8, 5]')
    trier_ins_v3(L)    # aucun résultat
    print('>>> trier_ins_v3(L)         # itératif sur place')
    print('>>> L\n-->',L)
    print()
    print('Test au chrono, on veut vérifier que la complexité')
    print('est quadratique (3000 et 6000 éléments) :')
    from exo_chrono import test_chrono
    print(test_chrono(trier_ins_v3,3000))
    print("Ok, ça semble bien quadratique ?...")

    # COMPLEMENTS EN LIGNE
    from webbrowser import open as browse
    #browse('https://www.toptal.com/developers/sorting-algorithms')
    #browse('https://fr.wikipedia.org/wiki/Tri_par_insertion')



