#!/usr/bin/env python3
# -*- coding: utf-8 -*-

######################################################################
### PYTHON. APPRENTISSAGE ACTIF (JP. ROY, EDITIONS ELLIPSES, 2020) ###
######################################################################

"""
exo_tri_rap.py (11.78)
Le "tri rapide" : Quicksort.
"""

import random

def tri_rapide(L):
    '''Tri rapide en fonctionnel récursif par pivotage d'une liste
    de nombres L. Pour la modélisation seulement.'''
    if len(L) <= 1: return L
    pivot = L[0]
    L1 = [L[i] for i in range(1,len(L)) if L[i] < pivot]    # on se paye des...
    L2 = [L[i] for i in range(1,len(L)) if L[i] >= pivot]   # ...tranches de listes !
    return tri_rapide(L1) + [pivot] + tri_rapide(L2)
    
if __name__ == '__main__':
    L = [6,2,8,1,3,3,5,7,4]
    print("Version fonctionnelle du tri rapide, avec copie.")
    print('>>> L\n-->',L)
    print('>>> tri_rapide(L)    # une copie triée\n-->',tri_rapide(L))
    print()
    
    print('Test de rapidité pour 3000 et 6000 nombres.')
    print('On attend une complexité en n*log(n).')
    from exo_chrono import test_chrono
    print('- recopie avec notre tri_rapide :',end='\n  ')
    print(test_chrono(tri_rapide,3000))
    # on est loin de battre sort !
    print('- recopie avec la primitive sorted :',end='\n  ')
    print(test_chrono(sorted,3000))
    print('- sur place avec la primitive .sort :',end='\n  ')
    print(test_chrono(lambda L: L.sort(),3000))

    # Analyse de la complexité. Très rapidement, les équations de notre
    # version fonctionnelle sont :
    #     c_0 = c_1 = 0  et  c_n = 2*c_{n/2} + O(n)
    # qui ont une solution en O(n log n) d'après le "Master Theorem".
    
    # COMPLEMENTS EN LIGNE
    from webbrowser import open as browse
#    browse('https://fr.wikipedia.org/wiki/Master_theorem')    # <--
#    browse('http://alexeyvishnevsky.com/2018/03/reinventing-quick-sort/')
#    browse('https://fr.wikipedia.org/wiki/Quickselect')
#    browse('https://fr.wikipedia.org/wiki/Tri_rapide')        # <--

    
