#!/usr/bin/env python3
# -*- coding: utf-8 -*-

######################################################################
### PYTHON. APPRENTISSAGE ACTIF (JP. ROY, EDITIONS ELLIPSES, 2020) ###
######################################################################

"""
exo_tri_stup.py (11.73)
Un mauvais tri pour le fun, et pour 'shuffle'...
"""

import random

def est_triee(L):
    '''Retourne True si et seulement si la liste numérique L
    est triée en ordre croissant.'''
    i = 0
    for i in range(len(L)-1):
        if L[i] > L[i+1]: return False
    return True

def melanger_v1(L):     # question b)
    '''Melange aléatoire des éléments d'une liste L avec Fisher-Yates.
    Par mutations, aucun résultat.'''
    #random.shuffle(L)  # tout simplement, ou bien à la main :
    n = len(L)
    for i in range(n):
        j = random.randint(i,n-1)
        (L[i],L[j]) = (L[j],L[i])         # échange !

def melanger_v2(L):    # alexeyvishnevsky.com, mars 2014
    # une variante de Fisher-Yates qui respecte les couples dans l'ordre
    n = len(L)
    for i in range(n):
        j = random.randint(i,n-1)
        if L[i] > L[j]:
            (L[i],L[j]) = (L[j],L[i])     # échange, sinon on ne bouge pas

def tri_stupide(L,melanger=melanger_v1,trace=False):      # vraiment lent...
    '''Effectue un tri stupide sur place et retourne le nombre de
    permutations essayées.'''
    cpt = 0
    while not est_triee(L):
        melanger(L)
        cpt += 1
        if trace : print('L =',L)
    return cpt

if __name__ == '__main__':
    L = [6,2,8,1,3,5,7,4]
    print('>>> L\n-->',L)
    print('>>> est_triee(L)\n-->',est_triee(L))
    print('>>> est_triee(L[3:6])\n-->',est_triee(L[3:6]))
    print('# Je lance une permutation aléatoire de L (sur place).')
    print('>>> melanger_v1(L)')
    melanger_v1(L)
    print('>>> L\n-->',L)
    print()
    print('# Tri stupide (sur place) avec melanger_v1 :')
    L = [6,2,8,1,3,5,7,4]
    print('>>> L\n-->',L)
    print('>>> tri_stupide(L,melanger=melanger_v1)')
    cpt = tri_stupide(L,melanger=melanger_v1)
    print('L = {}\n(avec {} permutations)'.format(L,cpt))
    print()
    print('# Tri un peu moins stupide (sur place) avec melanger_v2 :')
    L = [6,2,8,1,3,5,7,4]
    print('>>> L\n-->',L)
    print('>>> tri_stupide(L,melanger=melanger_v2)')
    cpt = tri_stupide(L,melanger=melanger_v2)
    print('L = {}\n(avec {} permutations)'.format(L,cpt))
    print()
    print('Pour le tri "un peu moins stupide" (1000 éléments) :')
    from time import time
    L = [random.randint(1,1000) for i in range(1000)] ; Lorig = L[:]  # une copie...
    start = time() ; tri_stupide(L,melanger=melanger_v2) ; end = time()
    print('Avec melanger_v2 : {:.1f} sec'.format(end-start))
    print('Quasiment hors de portée avec melanger_v1...')
    assert L == sorted(Lorig)

    # COMPLEMENTS EN LIGNE
    from webbrowser import open as browse
    #browse('http://alexeyvishnevsky.com/2018/03/reinventing-quick-sort')
    #browse('https://fr.wikipedia.org/wiki/M%e9lange_de_Fisher-Yates')







