#!/usr/bin/env python3
# -*- coding: utf-8 -*-

######################################################################
### PYTHON. APPRENTISSAGE ACTIF (JP. ROY, EDITIONS ELLIPSES, 2020) ###
######################################################################

"""
exo_vrisc1.py (11.103)
Introduction à la compilation d'expressions arithmétiques sous forme
d'arbre binaire d'expression algébrique (cf exo_arbrebin.py du §11.102)
"""

# Machine VRISC1 : une machine à pile avec les instructions :
# (push <nombre>) et (call <opérateur>)

from arbres2 import *   # les 5 fonctions du type abstrait "arbre binaire"
# arbre(r,Ag,Ad), est_feuille(a), racine(a), fg(a), fd(a)

PUSH = 'push' ; CALL = 'call'   # pour les yeux...

def compil1(a):
    '''Prend un arbre binaire algébrique a sur les quatre opérateurs
    binaires + - * /, et retourne une liste contenant du code VRISC pour
    une machine à pile.'''
    if est_feuille(a): return [[PUSH,a]]
    # on effectue un parcours postfixe de l'arbre en produisant du code
    return compil1(fg(a)) + compil1(fd(a)) + [[CALL,racine(a)]]

# NB. En compilant op(a,b), on empile b au-dessus de a. Il faudra en
# tenir compte au moment d'exécuter le code machine...

from operator import __add__, __sub__, __mul__, __truediv__
OPSA = {'+':__add__, '-':__sub__, '*':__mul__, '/':__truediv__}

def op_fonc(op) :
    '''Retourne la fonction Python correspondant à l'opérateur de nom op.'''
    return OPSA[op]

from exo_pile import *
# pile_vide(), est_pile_vide(p), sommet(p), depiler(p), empiler(x,p)

def valeur_var(v,mem):
    '''Retourne la valeur de la variable v dans le contexte de memoire mem'''
    try:
        return mem[v]
    except:
        raise ValueError('Variable indéfinie {}'.format(v))

def exec1(L,pile,mem,debug=False) :
    '''Exécute le code VRISC1 contenu dans la liste L sur une pile, et
    retourne la pile à la fin de l'exécution. La mémoire mem est fixe.'''
    for [instr,arg] in L:
        if debug: print('Exec {} {} sur la pile {}'.format(instr,arg,pile))
        if instr == PUSH:
            if type(arg) in {int, float}: empiler(arg,pile)
            else: empiler(valeur_var(arg,mem),pile)
        elif instr == CALL:
            x2 = depiler(pile)
            x1 = depiler(pile)
            empiler(op_fonc(arg)(x1,x2),pile)
    return pile

def listing(L) :
    '''Affiche le code machine VRISC1 à la verticale.'''
    for [instr,arg] in L:
        print('   {} {}'.format(instr,arg))

def decompil(L):
    '''Prend une liste L == compil(A) et retourne l'arbre A (arbre 2-3).'''
    pass         # à vous... Facile avec VRISC1, plus délicat avec VRISC2

# ----------------------------- test -------------------------------

if __name__ == '__main__':
    print('La machine VRISC1.')
    print()
    atest1 = ['-',['*',2,'x'],['-','x','y']]     # violation de type abstrait
    atest1 = arbre('-',arbre('*',2,'x'),arbre('-','x','y'))      # propre :-)
    print("On s'intéresse à l'arbre d'expression :")
    print(atest1,'\n')
    code1 = compil1(atest1)
    print('Le code VRISC1 produit par la compilation est :')
    print(code1)
    print()
    print('Ou avec un affichage vertical plus lisible :')
    listing(code1)
    print()
    print('On exécute ce code sur une pile vide avec x=3, y=1.')
    p = exec1(code1,pile_vide(),{'x':3,'y':1})
    print('Pile après :',p,"donc la valeur de l'arbre est",sommet(p))

    # COMPLEMENTS EN LIGNE
    from webbrowser import open as browse
    # la première référence est difficile mais vaut le détour dans une
    # seconde lecture.
#    browse('https://opensource.com/article/18/4/introduction-python-bytecode')
#    browse('https://fr.wikipedia.org/wiki/Processeur_basé_sur_la_pile')
