#!/usr/bin/env python3
# -*- coding: utf-8 -*-

######################################################################
### PYTHON. APPRENTISSAGE ACTIF (JP. ROY, EDITIONS ELLIPSES, 2020) ###
######################################################################

"""
exo_vrisc2.py (11.104)
La machine VRISC2 : une machine à pile avec les instructions :
   push <nombre>, call <op>, jmp <etiq>, brf <etiq> (branchements)
"""

# Machine VRISC2 : une machine à pile avec les instructions :
# (push <nombre>), (call <opérateur>), (jmp <étiquette>), (brf <etiquette>)

# On utilise le type abstrait "arbre 2-3".
from arbres2_3 import *
# arbres 2-3 : arbre(r,ag,ad,add=None) et fdd(a)

PUSH = 'push' ; CALL = 'call' ; JMP = 'jmp' ; BRF = 'brf'  # pour les yeux...

from operator import __add__, __sub__, __mul__, __truediv__
OPSA = {'+':__add__, '-':__sub__, '*':__mul__, '/':__truediv__}   # op. numériques
from operator import __eq__, __lt__, __le__, __gt__, __ge__
OPSB = {'=':__eq__,'<':__lt__,'<=':__le__,'>':__gt__,'>=':__ge__} # op. booléens
OPS = {**OPSA, **OPSB}      # réunion de deux dictionnaires

def op_fonc(op):
    '''Retourne la fonction Python correspondant à l'opérateur de nom op.'''
    return OPS[op]

def genetiq():              # version simple avec une fermeture locale
    '''Retourne un générateur numérique 1,2,3,...'''
    cpt = 0
    def incremente():
        nonlocal cpt
        cpt += 1
        return cpt
    return incremente       # une "fermeture" avec variable privée

etiq = genetiq()            # un générateur d'étiquettes

def genetiq():              # mieux : version pur Python avec un générateur
    cpt = 0
    while True:
        yield cpt
        cpt += 1

etiq = genetiq()            # le générateur d'étiquettes (utilisé avec next)

def compil2(a) :
    '''Prend un arbre binaire algébrique a sur les quatre opérateurs
    binaires + - * /, et retourne une liste contenant du code VRISC pour
    une machine à pile.'''
    if est_feuille(a): return [[PUSH,a]]
    if racine(a) in OPS:   # comme dans compil1
        return compil2(fg(a)) + compil2(fd(a)) + [[CALL,racine(a)]]
    if racine(a) == 'if':  # attention : générer des branchements !
        (e1, e2) = (next(etiq),next(etiq))     # deux nouvelles étiquettes
        code = compil2(fg(a))           # compilation du test
        code.append([BRF,e1])           # génération d'un saut conditionnel
        code.extend(compil2(fd(a)))     # compilation de la partie ALORS
        code.append([JMP,e2])           # génération d'un saut inconditionnel
        code.append(e1)                 # génération d'une étiquette
        code.extend(compil2(fdd(a)))    # compilation de la partie SINON
        code.append(e2)                 # génération d'une étiquette
        return code
    raise ValueError('Noeud inconnu de racine {}'.format(racine(a)))

def listing(code):
    '''Affiche proprement le code VRISC, avec les étiquettes décalées.'''
    for (i,x) in enumerate(code):
        print(i,end='\t')
        try:
            print('  {} {}'.format(x[0],x[1]))
        except:
            print(x)

from exo_pile import *

def valeur_var(v,mem):
    '''Retourne la valeur de la variable v dans le contexte de mémoire mem'''
    try:
        return mem[v]
    except:
        raise ValueError('Variable indéfinie {}'.format(v))

def resoudre(L):
    '''Effectue une mutation sur le code VRISC L de sorte à éliminer les
    étiquettes symboliques 0,1,2,... en les remplaçant par le numéro
    de l'instruction à laquelle se brancher. Sinon il faudrait à chaque
    branchement calculer la position de l'étiquette avec .index !'''
    for x in L:
        if type(x) == int: continue        # on ne touche pas aux étiquettes
        if x[0] in {JMP,BRF}:
            x[1] = L.index(x[1]) + 1       # recherche de l'étiquette


# Le code ci-dessous cherche chaque fois la position d'une étiquette dans
# le code VRISC lors d'un JMP ou d'un BRF. Pour y remédier, on peut faire
# une première passe sur le code pour "résoudre les étiquettes".
def exec2(L,pile,mem={},debug=False):
    '''Exécute le code VRISC2 contenu dans la liste L sur une pile, et
    retourne la pile à la fin de l'exécution. La mémoire mem est fixe.'''
    ip = 0                  # numéro de l'instruction courante
    while ip < len(L):      # stop à la fin du programme
        if type(L[ip]) == int:
            if debug:
                print('Etiquette',L[ip])
            ip += 1
        else :
            [instr,arg] = L[ip]
            if debug : print('Exec {} {} sur la pile {}'.format(instr,arg,pile))
            if instr == PUSH:
                if type(arg) in {int, float, bool}:
                    empiler(arg,pile)
                else:
                    empiler(valeur_var(arg,mem),pile)
                ip += 1
            elif instr == CALL:
                x2 = sommet(pile) ; depiler(pile)
                x1 = sommet(pile) ; depiler(pile)
                empiler(op_fonc(arg)(x1,x2),pile)
                ip += 1
            elif instr == JMP:     # branchement inconditionnel à étiquette
                ip = arg
            elif instr == BRF:     # branchement conditionnel à étiquette
                test = sommet(pile) ; depiler(pile)
                if not test:
                    ip = arg
                else:
                    ip += 1
    return pile
####################### BUG remplacer les tuples par des listes mutables !

if __name__ == '__main__':
    print('La machine VRISC2')
    print()
    atest2 = ['+',['if',['=','x',2],['-','x',1],['+','x','y']],['/','x',2]]
    print('>>> atest2\n-->',atest2)
    print()
    print('# Compilation de atest2 :')
    code2 = compil2(atest2)
    print('>>> code2\n-->',code2)
    print()
    print('# ou avec un listing plus classique :')  # étiquettes décalées à gauche
    print('>>> listing(code2)')
    listing(code2)
    print()
    print('# Résolution des étiquettes symboliques :')
    resoudre(code2)    # mutation  de code2, on laisse les étiquettes mortes...
    print('>>> listing(code2)')
    listing(code2)
    print()
    print('# Exécution de ce code sur une pile vide avec x=3, y=1.')
    p = exec2(code2,pile_vide(),mem={'x':3,'y':1})
    print('# Pile finale (résultat du programme au sommet) :')
    print('>>> p\n-->',p)
    print()
    print('# Exécution avec DEBUG sur une pile vide avec x=3, y=1.')
    p = exec2(code2,pile_vide(),mem={'x':3,'y':1},debug=True)
    print('# Pile finale (résultat du programme au sommet) :')
    print('>>> p\n-->',p)

