#!/usr/bin/env python3
# -*- coding: utf-8 -*-

######################################################################
### PYTHON. APPRENTISSAGE ACTIF (JP. ROY, EDITIONS ELLIPSES, 2020) ###
######################################################################

"""
exo_webtime.py (11.121)
Web scraping :-)
"""

import requests, re
from exo_html import html_unicode   # pour la conversion HTML --> Unicode

# sauvegarde sur disque du code source HTML de la page Web contenant l'information
# (j'ai surfé sur cette page pour être sûr qu'elle contient bien l'information !)
with requests.get('http://www.heure.com') as reponse:
    with open('heure.txt','w',encoding='utf-8') as f_out:     # code source HTML
        f_out.write(reponse.text)                             # fichier 'heure.txt' OK

# maintenant que le fichier 'heure.txt' est sur le disque, je le regarde de près
# pour repérer une expression régulière qui caractérise l'emplacement géographique
# de l'information dans le fichier. J'ai trouvé ceci :
motif = re.compile('(http://heure-[^.]+[.]heure[.]com/heure-[^.]+[.]htm)">([^<]+)</a>')

# Je construis alors le dictionnaire VILLES = {ville:url,...}
VILLES = {}
with open('heure.txt','r',encoding='utf-8') as f_in:
    # je me positionne juste avant la zone intéressante
    for ligne in f_in:
        if 'class="Style1">' in ligne: break
    # je suis sur la première ligne intéressante '<a href=...........<br>\n'
    while True:
        ligne = f_in.readline()
        sol = re.search(motif,ligne)
        if not sol: break               # fin de la zone intéressante
        ville = html_unicode(sol[2])    # conversion html -> unicode
        VILLES[ville] = sol[1]

print('VILLES :',list(VILLES.keys()),'\n')
print('Pour Singapour, connexion à :\n{}\n'.format(VILLES['Singapour']))
# NB. Ce dictionnaire ne devrait être calculé qu'une seule fois et mémorisé
# sur le disque... Il permet de connaître l'URL adaptée à chaque ville.

def heure(ville):
    '''Retourne l'heure courante locale de la ville v.'''
    motif = re.compile('Date[(][^,]+,[^,]+,[^,]+, ?([0-9]+), ?([0-9]+)')
    with requests.get(VILLES[ville]) as reponse:    # heure locale
        contenu = reponse.text
        sol = re.search(motif,contenu)
        if sol: return("{}h {}mn".format(sol[1],sol[2]))

for ville in ('Paris','Singapour','New York','Los Angeles'):
    print('{:12} : {}'.format(ville,heure(ville)))

# COMPLEMENTS EN LIGNE
from webbrowser import open as browse
# un exemple de site
#browse('http://www.heure.com')     # https://www.timeanddate.com

# une fois le web scraping par expressions régulières digéré, on peut passer
# à une niveau plus abstrait (BeautifulSoup est en exercice dans le livre)
#browse('https://realpython.com/python-web-scraping-practical-introduction/')



