#!/usr/bin/env python3
# -*- coding: utf-8 -*-

######################################################################
### PYTHON. APPRENTISSAGE ACTIF (JP. ROY, EDITIONS ELLIPSES, 2020) ###
######################################################################

"""
exo_yacc1_student.py (11.113)
Illustration de SLY, logiciel Lex/Yacc pour Python de D. Beazley.
Analyseur syntaxique pour des expressions arithmétiques parenthésées
sur les opérateurs + - * / ** et avec des nombres entiers et
flottants, et des variables. On gère le - unaire !
Complétez ce fichier.
La SOLUTION est dans le fichier "exo_yacc1.py"
"""

from exo_lex1 import ArithLexer, LexerError   # notre lexer d'expressions arithmétiques
from sly import Parser                 # la classe Parser à sous-classer
import operator

class ParserError(Exception):          # juste pour la lisibilité des exceptions
    pass

class ArithParser(Parser):             # une sous-classe de Parser
    tokens = ArithLexer.tokens         # en premier lieu, les tokens du lexer

    # Nouvelle grammaire des expressions pour cet exercice du §11.112 :
    #    (R1)  expr ::= VAR
    #    (R2)  expr ::= FLOAT
    #    (R3)  expr ::= INT
    #    (R4)  expr ::= expr ADD expr   |
    #    (R4)  expr ::= expr SUB expr   |
    #    (R4)  expr ::= expr MUL expr   |  regroupées dans une même méthode
    #    (R4)  expr ::= expr DIV expr   |
    #    (R4)  expr ::= expr PUIS expr  |
    #    (R5)  expr ::= PARG expr PARD
    #    (R6)  expr ::= SUB expr %prec USUB     # ici SUB est le moins unaire

    # Cette grammaire est ambiguë, il faut rajouter des priorités pour
    # permettre au parser de lever l'ambiguïté. Régles de "précédence" :

    precedence = [
        # + et - ont mêmes priorités, et associatives à gauche
        ('left','ADD','SUB'),       # x+y+z == (x+y)+z, idem avec -
        # idem pour * et / mais qui sont plus prioritaires que + et -
        ('left','MUL','DIV'),       # x*y*z == (x*y)*z, idem avec /
        # complétez les précédences pour - unaire (USUB) et ** (PUIS)
    ]

    # Les règles de grammaire proprement dites (ajoutez R1, R2, R4/** et R6)
    @_('INT')
    def expr(self,p):
        print('Règle R3 :',p[0])
        return p[0]                  # un entier positif

    @_('expr ADD expr',
       'expr SUB expr',
       'expr MUL expr',
       'expr DIV expr')  # même action ! Ajoutez l'opérateur **
    def expr(self,p):
        print('Règle R4/{1} : {0}, {1}, {2}'.format(p[0],p[1],p[2]))
        return [p[1], p[0], p[2]]   # traduction vers du Scheme-like (arbre d'analyse)

    @_('PARG expr PARD')
    def expr(self,p):
        print('Règle R5 :',p[0],p[1],p[2])
        return p[1]                  # on élimine les parenthèses

    def error(self,t):   # gestion des erreurs plus complète dans la doc de SLY...
        if t:
            raise ParserError('sur ' + str(t))
        else:
            raise ParserError('expression incomplète')


if __name__ == '__main__':
    print('Proposez des expressions arithmétiques sur les opérateurs')
    print('+ - * / ** et comportant des entiers et flottants positifs')
    print('et des variables. Entrez .fini pour quitter.')
    lexer = ArithLexer()                # l'analyseur lexical
    parser = ArithParser()              # l'analyseur syntaxique
    while True:                         # .fini pour quitter proprement
        try:
            s = '-x2*(1.5+y**2**3)-z'      # un exemple d'expression...
            s = input('votre expression : ')
            if s.lower() == '.fini' : break
            # le générateur de tokens à la demande
            tok = lexer.tokenize(s)
            # l'arbre résultat de l'analyse syntaxique du flot de tokens
            res = parser.parse(tok)
            print('Conversion de',repr(s),':')
            print(res)
        except LexerError as e:
            print("LexerError " + str(e))
        except ParserError as e:
            print("ParserError " + str(e))

    from webbrowser import open as browse
    #browse('https://www.youtube.com/watch?v=zJ9z6Ge-vXs')       # SLY @ PYCON 2018
    #browse('https://sly.readthedocs.io/en/latest/index.html')   # doc de SLY




