#!/usr/bin/env python3
# -*- coding: utf-8 -*-

######################################################################
### PYTHON. APPRENTISSAGE ACTIF (JP. ROY, EDITIONS ELLIPSES, 2020) ###
######################################################################

"""
exo_yacc2.py (11.114)
La calculette avec fonction. Avec lexer, parser, évaluateur.
Ceci est une SOLUTION du §11.114. Le fichier à compléter pour
faire l'exercice est "exo_yacc2_student.py"
"""

# Exemple de session souhaitée :
#      ? x = -4
#      ? x = x + 1
#      ? .mem
#      {'+': <built-in function add>, ....., 'x': -3}
#      ? x + log(-x-2) + sin(pi/2)
#      = -2.0
#      ? defn f(x,y) = x**2-(x+y)*sin(pi/2)
#      ? f
#      <fonction>
#      ? 5 + f(1,-x-1)
#      = 3.0
#      ? x
#      = -3
#      ? y
#      Variable indéfinie : y
#      ? z == 10
#      CalcfParserError sur Token(type='EGAL', value='=', lineno=1, index=3)
#      ? z
#      Erreur : Variable indéfinie z
#      ? .fini
#      Come back soon !
#      >>> x            # <-- retour au toplevel du bon vieux Python
#      NameError: name 'x' is not defined
#      >>> MEM['x']     # il est dans la mémoire de la calculette
#      -3

# --------------------------------- LEXER --------------------------------

from sly import Lexer

class CalcfLexerError(Exception):    # juste pour la lisibilité des exceptions
    pass

class CalcfLexer(Lexer):
    # j'ajoute les types de token VIRG,DEF,CORPS pour gérer les fonctions utilisateur
    tokens = {DEF, VAR, FLOAT, INT, ADD, SUB, PUIS, MUL, DIV, EGAL, PARG, PARD, VIRG}

    # les différents types de tokens avec leur expression régulière
    DEF   = 'defn'        # doit être reconnu avant VAR !!!
    VAR   = '[a-zA-Z][a-zA-Z0-9]*'
    FLOAT = '[0-9]+[.][0-9]+'
    INT   = '[0-9]+'
    ADD   = '\+'          # ou '[+]'
    SUB   = '-'
    PUIS  = '\*\*'        # ou '[*][*]'     et AVANT MUL (essayez de le placer après)
    MUL   = '\*'          # ou '[*]'
    DIV   = '/'
    EGAL  = '='           # absence d'opérateur == dans cette calculette !
    PARG  = '\('          # ou '[(]'
    PARD  = '\)'          # ou '[)]'
    VIRG  = ','

    # la chaîne contenant les caractères à ignorer : espace et tabulation
    ignore = ' \t'

    def DEF(self,t):              # avant la règle VAR !!
        return t

    def VAR(self,t):
        return t

    def FLOAT(self, t):           # une action à la lecture d'un token t de type FLOAT
        t.value = float(t.value)
        return t

    def INT(self, t):             # une action à la lecture d'un token t de type INT
        t.value = int(t.value)
        return t

    def error(self,t):            # je suis sur un caractère provoquant une erreur
        msg = 'ERREUR sur "{}" en ligne {} colonne {}'.format(t.value[0],t.lineno,t.index)
        raise CalcfLexerError(msg)

if __name__ == '__main__':        # test du lexer

    def test_lexer():
        print('Proposez des expressions arithmétiques sur les')
        print('opérateurs + - * / ** sur entiers et flottants,')
        print('avec variables, affectations et defn.')
        print('.fini pour quitter.')
        print('TEST DU LEXER !')
        lexer = CalcfLexer()
        while True:
            s = input('? ')       # test :        defn foo(x,y,z) = -x+2.56*h(y**2-1)
            if s.lower() == '.fini': break
            tok = lexer.tokenize(s)
            for t in tok: print((t.type,t.value), end=' ')
        raise Exception('STOP')

    #test_lexer()


# --------------------------------- PARSER --------------------------------

from sly import Parser                 # la classe Parser à sous-classer

class CalcfParserError(Exception):      # juste pour la lisibilité des exceptions
    pass

class CalcfParser(Parser):
    tokens = CalcfLexer.tokens         # les noms symboliques des tokens
    # {VAR,FLOAT,INT,ADD,SUB,PUIS,MUL,DIV,EGAL,PARG,PARD,VIRG,DEF,CORPS}

    def __init__(self,trace=False):    # trace pour voir les règles appliquées
        Parser.__init__(self)
        self.trace = trace

    # Grammaire des expressions de la calculette:
    #    (R0)   initial ::= topexpr
    #    (R1)   topexpr ::= VAR EGAL expr
    #    (R2)   topexpr ::= DEF VAR PARG params PARD EGAL expr
    #    (R3)   topexpr ::= expr
    #    (R4)   expr    ::= VAR | FLOAT | INT
    #    (R5)   expr    ::= expr ADD expr  | ... | expr PUIS expr
    #    (R6)   expr    ::= VAR PARG exprs PARD   # appel de fct d'arité n > 0
    #    (R7)   expr    ::= PARG expr PARD
    #    (R8)   expr    ::= SUB expr %prec USUB
    #    (R9)   exprs   ::= expr virgexprs
    #    (R10)  virgexprs ::= vide | virgexprs VIRG expr
    #    (R11)  params  ::= VAR virgparams
    #    (R12)  virgparams ::= vide | virgparams VIRG VAR

    # Cette grammaire est ambiguë, il faut rajouter des priorités pour
    # permettre au parser de lever l'ambiguïté. Régles de "précédence" :

    precedence = [
        ('nonassoc','EGAL'),        # non associatif, ni à gauche ni à droite
        # + et - ont mêmes priorités, et associatives à gauche
        ('left','ADD','SUB'),       # x+y+z == (x+y)+z, idem avec -
        # idem pour * et / mais qui sont plus prioritaires que + et -
        ('left','MUL','DIV'),       # x*y*z == (x*y)*z, idem avec /
        # opérateur virtuel pour l'opérateur "- unaire"
        ('right','USUB'),
        # ** est la plus prioritaire, et associative à droite
        ('right','PUIS')
    ]

    # Les règles de grammaire proprement dites :

    @_('topexpr')
    def initial(self,p):
        if self.trace: print('Règle R0')
        return p[0]

    @_('VAR EGAL expr')
    def topexpr(self,p):          # affectation
        if self.trace: print('Règle R1')
        return ['$affect',p[0],p[2]]

    @_('DEF VAR PARG params PARD EGAL expr')
    def topexpr(self,p):          # définition de fonction
        # 'defn f(x,y,z) = x+2*y-z'  ------>  MEM['f'] = lambda x,y,z: x+2*y-z
        if self.trace: print('Règle R2')
        return ['$defn',p[1],p[3],p[6]]

    @_('expr')
    def topexpr(self,p):
        if self.trace: print('Règle R3')
        return p[0]

    @_('VAR', 'FLOAT', 'INT')
    def expr(self,p):
        if self.trace: print('Règle R4 :',p[0])
        return p[0]

    @_('expr ADD expr',
       'expr SUB expr',
       'expr MUL expr',
       'expr DIV expr',
       'expr PUIS expr')             # une même action !
    def expr(self,p):
        if self.trace: print('Règle R5/{1}'.format(p[0]))
        return ['$binop',p[1],p[0],p[2]]

    @_('VAR PARG exprs PARD')        # un appel de fonction d'arité n > 0
    def expr(self,p):
        if self.trace: print('Règle R6')
        return ['$app', p[0], p[2]]  # application d'une fonction à une liste d'arguments

    @_('PARG expr PARD')
    def expr(self,p):
        if self.trace: print('Règle R7')
        return p[1]                  # élimination des parenthèses


    @_('SUB expr %prec USUB')        # le - unaire
    def expr(self,p):
        if self.trace: print('Règle R8')
        return ['$binop', '*', -1, p[1]]

    @_('expr virgexprs')             # suite d'expression séparées par une virgule
    def exprs(self,p):
        if self.trace: print('Règle R9')
        return [p[0]] + p[1]               # construction d'une liste !

    @_('')
    def virgexprs(self,p):
        if self.trace: print('Règle R10 vide')
        return []                          # construction d'une liste !

    @_('virgexprs VIRG expr')
    def virgexprs(self,p):
        if self.trace: print('Règle R10')
        return p[0] + [p[2]]               # construction d'une liste !

    @_('VAR virgparams')              # suite de paramètres séparés par une virgule
    def params(self,p):
        if self.trace: print('Règle R11')
        return [p[0]] + p[1]               # construction d'une liste !


    @_('virgparams VIRG VAR')
    def virgparams(self,p):
        if self.trace: print('Règle R12')
        return p[0] + [p[2]]               # construction d'une liste !

    @_('')
    def virgparams(self,p):
        if self.trace: print('Règle R12 vide')
        return []                          # construction d'une liste !

    def error(self,t):   # gestion des erreurs plus complète dans la doc de SLY...
        if t: raise CalcfParserError('sur ' + str(t))
        raise CalcfParserError('expression incomplète')


if __name__ == '__main__':       # test de l'analyseur syntaxique

    def test_parser():
        print('Proposez des expressions arithmétiques sur les')
        print('opérateurs + - * / ** sur entiers et flottants,')
        print('avec variables, affectations et defn.')
        print('.fini pour quitter.')
        print('TEST DU PARSER !')
        lexer = CalcfLexer()                # l'analyseur lexical
        parser = CalcfParser(trace=False)   # l'analyseur syntaxique (réglez trace)
        while True:                         # .fini pour quitter proprement
            try:
                s = input('? ')    # test :        defn foo(x,y,z) = -x+2.56*h(y**2-1)
                if s.lower() == '.fini' : break
                res = parser.parse(lexer.tokenize(s))  # tokenize, parse/evaluate, print
                print('=',res)
            except CalcfLexerError as e:
                print("CalcfLexerError " + str(e))
            except CalcfParserError as e:
                print("CalcfParserError " + str(e))
            except Exception as e:
                print('Exception: ' + str(e))
        print('Come back soon !')
        raise Exception('STOP')

    #test_parser()


# --------------------------------- EVALUATEUR -----------------------------

# initialisation de la mémoire de la calculette
from operator import __add__, __sub__, __mul__, __truediv__, __pow__
from math import pi, log, sin, sqrt
MEM = {'+':__add__, '-':__sub__, '*':__mul__, '/':__truediv__, '**':__pow__,
       'pi':pi, 'log':log, 'sin':sin, 'sqrt':sqrt, 'curt':lambda x:x**(1/3)}

class CalcfEvalError(Exception):      # juste pour la lisibilité des exceptions
    pass

# Structure des objets produits par le parser
#      FLOAT | INT | VAR
#      ['$affect', VAR, expr]
#      ['$defn', VAR, [VAR,...], expr]
#      ['$binop',binop,expr1,expr2]            # binop dans {'+','-','*','/','**'}
#      ['$app',VAR,expr1,expr2,...]
#      expr                                    # expression arithmétique

def calc_eval(p):
    '''Prend un objet p résultat de l'analyse syntaxique d'une ligne et en
    calcule la valeur (ou en produit l'effet en mémoire).'''
    if type(p) in (float, int):    # on aurait pu typer les nombres dans le lexer...
        return p
    if type(p) == str:             # variable ?
        try: return MEM[p]
        except KeyError: raise CalcfEvalError('Variable indéfinie : ' + p)
    if (type(p) != list) or (p == []):
        raise CalcfEvalError('Mauvais résultat du parser : ' + p)
    # à partir d'ici p est une liste non vide
    (tete, *L) = p                 # déstructuration
    # on commence par traiter les mots-clé :
    if tete == '$affect':          # affectation ['$affect',var,expr]
        (var, expr) = L
        MEM[var] = calc_eval(expr)
    elif tete == '$defn':           # définition ['$defn',var,Lparams,expr]
        (f, Lparams, expr) = L
        # je construis une "fonction" dans la mémoire MEM sous la forme d'un tuple
        MEM[f] = ('fonction',Lparams,expr)    # on retarde l'évaluation de expr
    # sinon c'est une expression arithmétique :
    elif tete == '$binop':           # opération binaire ['binop','+',expr1,expr2]
        (op,expr1,expr2) = L
        try:
            f = MEM[op]
            return f(calc_eval(expr1),calc_eval(expr2))
        except KeyError:
            raise CalcfEvalError('Opérateur indéfini : ' + op)
    elif tete == '$app':       # application ['app','f',[expr1,expr2,...]]
        (f,Largs) = L
        Lvals = list(map(calc_eval,Largs))   # évaluation récursive des arguments
        # f doit être une fonction dans MEM
        try: f = MEM[f]
        except KeyError: raise CalcfEvalError('Fonction indéfinie : ' + f)
        # mais f peut être une primitive Python ou une fonction construite avec defn
        if callable(f):                 # une primitive Python ou une lambda-expression
            return f(*Lvals)
        elif type(f) == tuple:          # une fonction construite avec defn
            [_, Lparams, expr] = f      # _ == 'fonction', je ne vérifie pas...
            # je dois retourner la valeur du corps expr dans un contexte où les
            # variables de Lparams sont liées aux valeurs de Lvals. Pour cela,
            # sauvegardons le contexte :
            MEM_sauve = {x:MEM[x] for x in Lparams if x in MEM}
            MEM_undef = {x for x in Lparams if x not in MEM}
            # installation du nouveau contexte
            for x,v in zip(Lparams,Lvals): MEM[x] = v
            # évaluation du corps de la fonction dans le nouveau contexte
            res = calc_eval(expr)
            # restauration de l'ancien contexte
            for x in MEM_sauve: MEM[x] = MEM_sauve[x]
            for x in MEM_undef: del MEM[x]
            return res

            # NB. Dans un interprète en grandeur réelle, plutôt que modifier
            # la mémoire comme ci-dessus, on pourrait placer le nouveau contexte
            # en tête et parcourir une suite de contextes. C'est plus général (pour
            # les variables locales, etc) et un (petit) peu plus compliqué.
            # J'espère que vous allez augmenter les capacités de cette calculette...


if __name__ == '__main__':             # test de l'évaluateur

    def calculette():
        print('Proposez des expressions arithmétiques sur les')
        print('opérateurs + - * / ** sur entiers et flottants,')
        print('avec variables, affectations et defn.')
        print('.fini pour quitter et .mem pour la mémoire.')
        print('TEST DE LA CALCULETTE FINALE.')
        lexer = CalcfLexer()                # l'analyseur lexical
        parser = CalcfParser(trace=False)   # l'analyseur syntaxique (réglez trace)
        while True:                        # .fini pour quitter proprement
            s = input('? ')
            if s.lower() == '.fini':
                break
            elif s.lower() == '.mem':
                print('MEM =',MEM)
                continue
            else:
                try:
                    arbre = parser.parse(lexer.tokenize(s))  # traduction en arbre
                    #print('parse =',arbre)
                    res = calc_eval(arbre)                   # evaluation de l'arbre
                    if res:
                        if (type(res) == tuple) and (res[0] == 'fonction'):
                            print('= <fonction>')
                        else:
                            print('=',res)
                except CalcfLexerError as e:
                    print("CalcfLexerError " + str(e))
                except CalcfParserError as e:
                    print("CalcfParserError " + str(e))
                except Exception as e:
                    print(e)
        print('Come back soon !')

    calculette()

    from webbrowser import open as browse
    #browse('https://www.youtube.com/watch?v=zJ9z6Ge-vXs')       # SLY @ PYCON 2018
    #browse('https://sly.readthedocs.io/en/latest/index.html')   # doc de SLY
    #browse('https://www.ibm.com/support/knowledgecenter/en/SSLTBW_2.2.0/com.ibm.zos.v2r2.bpxa600/rrlr.htm')

    # Les arbres d'expression sont à ce point importants comme représentation
    # interne que des chercheurs français de la division IA de Facebook 
    # viennent de les utiliser dans le cadre des réseaux de neurones (pages 
    # 295-303) pour faire effectuer au réseau des calculs symboliques, ex:
    # calcul d'intégrales indéfinies.
    # Cela pourrait être une belle avancée dans le deep learning :
    #browse('https://www.quantamagazine.org/symbolic-mathematics-finally-yields-to-neural-networks-20200520/')
    #browse('https://arxiv.org/pdf/1912.01412.pdf')
