#!/usr/bin/env python3
# -*- coding: utf-8 -*-

######################################################################
### PYTHON. APPRENTISSAGE ACTIF (JP. ROY, EDITIONS ELLIPSES, 2020) ###
######################################################################

"""
exo_yacc3.py (11.115)
Une grammaire Yacc (avec SLY) pour les cpbb.
"""

# --------------------------------- LEXER ------------------------------

from sly import Lexer

class CpbbLexerError(Exception):
    pass


class CpbbLexer(Lexer):
    tokens = {PARG, PARD}

    PARG = '\('
    PARD = '\)'

    ignore = ' \t'

    def error(self,t):           # je suis sur un caractère provoquant une erreur
        msg = 'ERREUR sur "{}" en colonne {}'.format(t.value[0],t.index)
        raise CpbbLexerError(msg)

#if __name__ == '__main__':          # test du lexer
#    lexer = CpbbLexer()
#    s = '()() ((( )(()'
#    print('--- chaîne =',repr(s))
#    tok = lexer.tokenize(s)
#    for t in tok:
#        print((t.type,t.value))
#    print()

# --------------------------------- PARSER ------------------------------

from sly import Parser

class CpbbParserError(Exception):      # juste pour la lisibilité des exceptions
    pass

class CpbbParser(Parser):
    debugfile = 'parser.out'           # pour générer la table d'analyse (difficile)
    tokens = CpbbLexer.tokens

    def __init__(self,trace=False):    # trace pour suivre les règles appliquées
        Parser.__init__(self)
        self.trace = trace

    # Grammaire des cpbb :
    #    (R0)  initial ::= cpbb        # le mot 'start' est réservé !
    #    (R1)  cpbb    ::= cpbb cpbb
    #    (R2)  cpbb    ::= PARG cpbb PARD
    #    (R3)  cpbb    ::= empty       # empty est la production vide

    @_('cpbb')
    def initial(self,p):
        if self.trace: print('Règle R0')
        return True

    @_('cpbb cpbb')
    def cpbb(self,p):
        if self.trace: print('Règle R1')
        return True

    @_('PARG cpbb PARD')
    def cpbb(self,p):
        if self.trace: print('Règle R2')
        return True

    @_('empty')
    def cpbb(self,p):
        if self.trace: print('Règle R3')
        return True

    @_('')
    def empty(self,p):
        if self.trace: print('Production vide')
        pass


#version plus compacte mais sans les numéros de règles :
#    @_('cpbb')
#    def initial(self,p):
#        return True
#
#    @_('cpbb cpbb', 'PARG cpbb PARD', 'empty')
#    def cpbb(self,p):
#        return True
#
#    @_('')
#    def empty(self,p):
#        pass

    def error(self,t):   # gestion des erreurs plus complète dans la doc de SLY...
        if t: raise CpbbParserError('sur ' + str(t))
        raise CpbbParserError('expression incomplète')


if __name__ == '__main__':
    lexer = CpbbLexer()
    parser = CpbbParser(trace=True)

    def is_cpbb(s):
        '''Une chaîne s est-elle une cpbb ?'''
        try:
            res = parser.parse(lexer.tokenize(s))
            return res   # True...
        except:
            return False

    while True:                        # .fini pour quitter proprement
        s = input('? ')
        if s.lower() == '.fini' : break
        print(is_cpbb(s))
    print('Come back soon !')

from webbrowser import open as browse
#browse('http://deptinfo.unice.fr/%7ejulia/AL/')
