#!/usr/bin/env python3
# -*- coding: utf-8 -*-

######################################################################
### PYTHON. APPRENTISSAGE ACTIF (JP. ROY, EDITIONS ELLIPSES, 2020) ###
######################################################################

"""
exo_zip.py (11.94)
La fermeture éclair qui ne s'use que si l'on s'en sert !
"""

def szip(s1,s2):
    '''Retourne la liste des couples (x,y) de même rang de deux séquences
    s1 et s2. On stoppe dès qu'une séquence a été complètement parcourue.'''
    return [(s1[i],s2[i]) for i in range(min(len(s1),len(s2)))]

def mon_zip(obj1,obj2):
    '''Retourne le même résultat que zip(it1,it2). On suppose que it1 et it2
    sont deux objets itérables.'''
    it1 = iter(obj1) ; it2 = iter(obj2)       # itérateurs des objets itérables
    while True:
        try:
            y1 = next(it1) ; y2 = next(it2)
            yield (y1,y2)                   # générateur en suspension
        except StopIteration:
            return

def my_zip(obj1,obj2):  # autre solution sans try, en utilisant une sentinelle
    '''Retourne le même résultat que zip(it1,it2). On suppose que it1 et it2
    sont deux objets itérables.'''
    it1 = iter(obj1) ; it2 = iter(obj2)       # itérateurs des objets itérables
    obj = object()          # uniquement là pour son adresse et son unicité !
    while True:
            y1 = next(it1,obj)   # retourne obj si épuisé au lieu d'une exception
            if y1 is obj: return # is pour comparer les adresses mémoire
            y2 = next(it2,obj)   # voir la doc de 'next' en Python...
            if y2 is obj: return
            yield (y1,y2)

if __name__ == '__main__':
    x1 = [1,2,3] ; x2 = 'azerty'
    print('>>> x1\n-->',x1)
    print('>>> x2\n-->',repr(x2))
    print()
    print('>>> szip(x1,x2)\n-->',szip(x1,x2))
    print("# szip n'est pas épuisable :")
    print('>>> szip(x1,x2)\n-->',szip(x1,x2))
    print()

    print('Et maintenant simulation du vrai zip avec my_zip :')
    it = my_zip(x1,x2)
    print('>>> it = my_zip(x1,x2)')
    print('>>> type(it)\n-->',type(it))
    print('>>> list(it)\n-->',list(it))
    print('>>> list(it)\n-->',list(it),' <--  oups, épuisé !')

    # COMPLEMENTS EN LIGNE
    from webbrowser import open as browse
    #browse('https://docs.python.org/fr/3.8/library/functions.html#next')
    #browse('https://docs.python.org/fr/3.8/library/functions.html#zip')

