#!/usr/bin/env python3
# -*- coding: utf-8 -*-

######################################################################
### PYTHON. APPRENTISSAGE ACTIF (JP. ROY, EDITIONS ELLIPSES, 2020) ###
######################################################################

"""
code_lex.py    -- la version du §8.2 à compléter en exercice.
Illustration de SLY, logiciel Lex/Yacc pour Python de D. Beazley.
(SLY remplace l'ancienne version PLY du même auteur).
Analyseur lexical pour des expressions arithmétiques sur les
Opérateurs + - * / ** avec des nombres entiers et flottants et
des variables. Ne modifiez pas ce fichier, utilisez le même qui se
nomme "exo_lex2_student.py" dans le répertoire "2_Exercices".
03/03/2020
"""

# ------------------------ install ---------------------------
# Non installable pour l'instant avec conda. Je me rabats sur pip
# mais dans un nouvel environnement 'parsing'.
# (base) $ conda create --name parsing
# (base) $ conda activate parsing
# J'installe pip dans ce nouvel environnement, puis Spyder
# (parsing) $ conda install pip
# (parsing) $ conda install spyder
# NB. On peut utiliser IDLE si l'on ne veut pas installer Spyder...
# J'installe SLY avec ce pip (commande 'which pip' si l'on n'est pas sûr)
# (parsing) $ which pip
# /Users/roy/anaconda3/envs/parsing/bin/pip       # OK
# (parsing) $ pip install sly
# ...
# Successfully installed sly-0.3
#
# NB1. SLY n'ayant aucune dépendance, on aurait pu l'installer dans (base)...
# NB2. Dans les Préférences de Spyder (menu Outils), réglez l'interpréteur Python 
#      pour qu'il soit celui dans anaconda3/envs/parsing/bin/
# NB3. Si problèmes en plus : avec conda, j'ai downgradé ipython en 7.2.0.
# IDLE n'a pas ces soucis (au moins sur Mac et Windows)...
# ------------------------------------------------------------

from sly import Lexer

class LexerError(Exception):    # juste pour la lisibilité des exceptions
    pass

class ArithLexer(Lexer):
    # l'ensemble des types de tokens (en premier !!)
    tokens = {INT, ADD, SUB, MUL, DIV, PARG, PARD }
    # les différents types de tokens avec leur expression régulière
    INT = '[0-9]+'
    ADD = '[+]'          # ou '\+'
    SUB = '-'
    MUL = '[*]'          # ou '\*'
    DIV = '/'
    PARG = '[(]'         # ou '\('
    PARD = '[)]'         # ou '\)'
    # la chaîne contenant les caractères à ignorer : espace et tabulation
    ignore = ' \t'

    def INT(self, t):             # une action à la lecture d'un token t de type INT
        t.value = int(t.value)
        return t

    def error(self,t):            # je suis sur un caractère provoquant une erreur
        msg = 'ERREUR sur "{}" en ligne {} colonne {}'.format(t.value[0],t.lineno,t.index)
        raise LexerError(msg)


if __name__ == '__main__':
    s = '-205 + 3*(8-255)-   412'    # essayez avec '-205 + 3*@(8-255)-   412'
    lexer = ArithLexer()
    tok = lexer.tokenize(s)          # tok est un itérateur sur les tokens
    for t in tok:
        print(t)
    # ce qui équivaut à :
    # while True:
    #     try: print(next(tok))
    #     except StopIteration: break

    from webbrowser import open as browse
    #browse('https://www.youtube.com/watch?v=zJ9z6Ge-vXs')       # SLY @ PYCON 2018
    #browse('https://sly.readthedocs.io/en/latest/index.html')   # doc de SLY

