#!/usr/bin/env python3
# -*- coding: utf-8 -*-

######################################################################
### PYTHON. APPRENTISSAGE ACTIF (JP. ROY, EDITIONS ELLIPSES, 2020) ###
######################################################################

"""
section_4_1.py
03/03/2020
"""

# Codes du §4.1
print('Le §4.1 : graphisme de la tortue.')

# La gestion des boucles d'évènements par Spyder évolue régulièrement
# et s'avère donc parfois problématique (Mac, Windows). Ceci peut
# provenir de mauvaises interactions entre Tkinter et Qt qui ont chacun
# leur boucle d'évènements. Joint au fait que le développement (libre)
# de Spyder se fait semble-t-il majoritairement sous Linux faute de
# moyens financiers...
# Si vous avez des problèmes, utilisez IDLE ou le TERMINAL ou JUPYTER 
# (ce fichier est disponible au format .ipynb pour Jupyter).

# -------------------------------------------------------------

from turtle_init import *    # fichier d'utilitaires tortue qui ajoute :
# from turtle import *
# init(x=0,y=0,cap=0)  # cap=0 signifie 'Est' en mode 'normal', et 'Nord' en mode 'logo'
# afficher(msg,x,y)
# sleep(dt)            en secondes flottantes
# deg_rad(a_degrés)    conversion degrés -> radians
# rot(M,K,a_degrés)    K = centre de rotation
# segment(M1,M2)       le tracé d'un segment
# pi, sqrt, sin, cos, randint   pour les calculs fréquents

def poly_cart(n,a,b,r):
    '''Dessine un polygone régulier ayant n côtés inscrit
    dans un cercle de rayon r et de centre (a,b).'''
    up() ; goto(a + r,b) ; down()           # départ au pôle Est
    alpha = 2 * pi / n                      # radians pour la trigo
    for k in range(1,n+1):
        goto(a + r * cos(k * alpha), b + r * sin(k * alpha))

def poly_pol(n,c):
    '''Dessine un polygone régulier ayant n côtés,
    côté ayant une longueur c, à partir de la position
    courante de la tortue.'''
    a = 360 / n
    for i in range(n):
        forward(c)
        left(a)

def von_koch(n,L):     # approximant de niveau n, base de longueur L
    if n == 0 :
        forward(L)                           # le générateur
    else :
        von_koch(n-1,L/3)                    # 4 appels récursifs !
        left(60)
        von_koch(n-1,L/3)
        right(120)
        von_koch(n-1,L/3)
        left(60)
        von_koch(n-1,L/3)

if __name__ == '__main__':
    reset()           # effacement canvas graphique, init. tortue au centre
    tracer(False)     # début tracé immédiat
    afficher('Tracés en cartésien',0,220)
    poly_cart(7,0,0,200)
    poly_cart(4,100,50,50)
    tracer(True)      # fin tracé immédiat
    getcanvas().postscript(file='polygone7_4.ps')   # création d'un fichier disque

    tracer(False)
    poly_cart(40,-200,200,30)    # polygone régulier de 40 côtés : quasi-cercle
    tracer(True)      # fin tracé immédiat
    sleep(4)

    # tracé lent, cette fois en polaire, à l'endroit où est la tortue
    reset()
    afficher('Tracé en polaire (mode lent)',0,-100)
    poly_pol(7,100)
    sleep(2)

    # la fractale de von Koch, niveaux 0..7
    tracer(False)     # on repasse en mode rapide
    for i in range(8):
        reset()
        init(-285,0,0)                     # à gauche, cap Est
        von_koch(i,285*2)
        afficher('Von Koch, niveau {}'.format(i),0,-100)
        sleep(2)
    tracer(True)
    getcanvas().postscript(file='von_koch.ps')  # fichier Postscript sur le disque

    # Lancement de la boucle d'évènements (bloquante) dans la fenêtre tortue
    mainloop()
    print('Fini.')     # une fois la fenêtre fermée...

    from webbrowser import open as browse
    #browse('http://blog.kleinproject.org/?p=479&lang=fr')
    #browse('https://fr.wikipedia.org/wiki/Liste_de_fractales_par_dimension_de_Hausdorff')
    #browse('http://algorithmicbotany.org/papers/abop/abop.pdf')
    #browse('https://docs.python.org/fr/3.8/library/turtle.html')
    #browse(https://cs.nyu.edu/courses/spring16/CSCI-UA.0380-004/slides/02/turtle.html#/1')
    #browse('https://gist.github.com/nataliefreed/8483050')  # bébé tortue en Processing ?





