#!/usr/bin/env python3
# -*- coding: utf-8 -*-

######################################################################
### PYTHON. APPRENTISSAGE ACTIF (JP. ROY, EDITIONS ELLIPSES, 2020) ###
######################################################################

"""
section_7_1_2_3.py
14.01.2019
"""

print('Quelques exemples de codes des §7.1--7.3')
print("(les classes d'objets)")
print()

from math import pi

class Cercle:
    '''Cercle(x=0,y=0,r=10,coul='black') pour construire un cercle.
    Méthodes d'instance, de classe, et statiques.'''
    nb_cercles = 0    # un attribut de classe accessible par Cercle.nb_cercles
    
    def __init__(self, x, y, r=10, coul='red'):
        # initialisation des attributs d'instance
        self.x = x
        self.y = y
        self.r = r
        self.coul = coul
        # modification d'un attribut de classe
        Cercle.nb_cercles += 1      # et non nb_cercles !
        
    def __repr__(self):
        x = self.x ; y = self.y ; r = self.r ; coul = self.coul
        return 'Cercle(x={},y={},r={},coul="{}")'.format(x,y,r,coul)
    
    def __eq__(self,obj):           # self == obj ?
        if type(obj) == self.__class__:               # même classe,
            return self.__dict__ == obj.__dict__      # et même attributs
        return False

    def longueur(self):            # une méthode d'instance
        return 2 * pi * self.r
    
    # Le §7.3 a tort en affirmant de len(c) <==> c.len(). La méthode __len__
    # vérifie que le résultat est un entier (le nombre d'éléments d'une collection).
    # Il faut donc convertir le résultat en entier, au prix d'une perte de précision.
    # Bref, ce n'était pas une bonne idée (ici) !...
    def __len__(self):             # voir §7.3
        return int(self.longueur())  # pour pouvoir écrire len(c) au lieu de c.longueur()
    
    @classmethod                   # décorateur des méthodes de classe
    def affiche_etat(cls):         # affichage de l'état de la classe
        # ici cls remplace self et représente la classe courante
        print('nb_cercles =',Cercle.nb_cercles)
        
    @staticmethod                 # décorateur des méthodes statiques
    def perimetre(r):             # une fonction mathématique par ex.
        return 2 * pi * r
    
    def longueur2(self):          # une méthode d'instance
        return Cercle.perimetre(self.r)   # utilisant une méthode statique
    

if __name__ == '__main__' :
    c = Cercle(10,20,5,'red')
    print('Longueur de c :',c.longueur())
    print('Longueur2 de c :',c.longueur2())
    cj = Cercle(x=10,y=20,r=5,coul="red")      # un frère jumeau
    print('c == cj :',c == cj,'et c != cj :',c != cj)
    c2 = c
    Cercle.affiche_etat()         # envoi d'un message à la classe
    c.affiche_etat()              # ok, l'objet passera le message à la classe...
    print('len(c) -->',len(c),'au lieu de',c.longueur())   # hum :-(

    from webbrowser import open as browse
    #browse('https://docs.python.org/fr/3/tutorial/classes.html')
    #browse('https://openclassrooms.com/fr/courses/235344-apprenez-a-programmer-en-python/232721-premiere-approche-des-classes')
    