#!/usr/bin/env python3
# -*- coding: utf-8 -*-

######################################################################
### PYTHON. APPRENTISSAGE ACTIF (JP. ROY, EDITIONS ELLIPSES, 2020) ###
######################################################################

'''
section_9_1_9.py
03/03/2020

Rebond d'une balle à la verticale avec gravitation et frottement au sol.
ATTENTION : près du sol, on force la balle tangente au sol (d'où une petite erreur
non perçue visuellement).
Le frottement F doit être dans [0,1[ car la balle ne remonte pas exactement à son point
de départ si F == 1 à cause du truquage au sol.
Assez réaliste dans un jeu video AVEC frottements...
'''

print('Quelques exemples de codes du §9.1.9')
print()

from tkinter import *

class Anim_balle(Tk):
    '''Chute et rebond d'une balle à la verticale'''

    def __init__(self,w=80,h=400,r=20,grav=0.5,frot=0.95):       # frot dans [0,1[
        Tk.__init__(self)     # super().__init__()
        (self.w, self.h) = (w, h)        # dimensions du canevas
        (self.x, self.y) = (w/2, h/4)    # position initiale de la balle
        self.dy = 0           # vitesse verticale initiale de la balle
        self.grav = grav      # constante de gravitation (additive à la vitesse) : Terre=0.5, Lune=0.1
        self.frot = frot      # coeff. multiplicatif de frottement
        self.r = r            # rayon de la balle
        self.canvas = Canvas(self, width=w,height=h,bg='yellow')
        self.canvas.pack()
        self.focus_force()              # au premier plan
        self.animation()

    def animation(self):                # qu'est-ce qu'un dessin animé ?...
        if not self.fini():
            self.dessiner()
            self.suivant()
            self.after(10,self.animation)      # auto-rappel dans 10 ms

    def dessiner(self):                 # rendu de la scène : la VUE
        self.canvas.delete(ALL)
        x = self.x ; y = self.y ; r = self.r
        self.canvas.create_oval(x-r,y-r,x+r,y+r,fill='red')

    def suivant(self):                  # mise à jour du monde : le MODELE
        print('y =',self.y,'\tdy =',self.dy)
        ys = self.y + self.dy           # prochaine altitude possible ys de la balle
        if ys <= self.h - self.r:       # si la balle est loin du sol...
            self.y = ys                 # on y va,
            self.dy += self.grav        # et on augmente la vitesse
        else :
            self.y = self.h - self.r    # sinon on la pose tangente au sol (petit truquage, chut)
            self.dy -= self.grav
            self.dy *= -self.frot       # effet du frottement au sol, retournement de la vitesse

    def fini(self):                     # l'animation est-elle dans un état final ?
        return (abs(self.y - (self.h - self.r)) < 1) and (abs(self.dy) < 0.3)

if __name__ == '__main__':
    print("Une animation avec tkinter.")
    app = Anim_balle()
    app.mainloop()                      # mainloop est bloquant

    print('Position de la balle :',(app.x,app.y))  # action à la fermeture !
    print('et sa vitesse verticale :',app.dy)
    print('Fini')

    # http://www.fil.univ-lille1.fr/~marvie/python/chapitre6.html


