#!/usr/bin/env python3
# -*- coding: utf-8 -*-

######################################################################
### PYTHON. APPRENTISSAGE ACTIF (JP. ROY, EDITIONS ELLIPSES, 2019) ###
######################################################################

# Une classe Tortue en Python dans le module tortue

# Attention, les coordonnées (0,0) en Processing sont en haut à gauche...
X0 = 300      # origine du repère tortue (au centre d'une fenêtre graphique 600x600)
Y0 = 300

# quelques utilitaires de calcul

def tortue_posn(x,y):
    return (x + X0, Y0 - y)     # changement de repère !

def distance(t1,t2):            # distance entre deux tortues (objets de classe Tortue)
    return abs(t1.x-t2.x) + abs(t1.y-t2.y)

# la classe des tortues

class Tortue:
    tortues = []          # attribut de classe : la liste de toutes les tortues créées

    # le constructeur
    def __init__(self, x=0, y=0, cap=0, pen=True):  # en coordonnées du repère tortue
        self.x = x
        self.y = y
        self.cap = cap                 # en degrés svp !
        self.pen = pen                 # True <==> baissé, False <==> levé
        Tortue.tortues.append(self)    # enregistrement de la nouvelle tortue
    
    # les accesseurs aux composantes de l'état de la tortue courante
    def pos(self):
        '''Retourne un couple (x,y) représentant la position de la tortue courante.'''
        return (self.x, self.y)
        
    def heading(self):
        '''Retourne le cap de la tortue courante en degrés.'''
        return self.cap
        
    # les modificateurs d'état de la tortue courante
    def up(self):
        '''Lève le crayon de la tortue courante.'''
        self.pen = False
        
    def down(self):
        '''Baisse le crayon de la tortue courante.'''
        self.pen = True
    
    def left(self,a):           # a en degrés svp !
        '''Fait tourner sur place la tortue courante de a degrés sur sa gauche.'''
        self.cap = (self.cap + a) % 360

    def right(self,a):          # a en degrés svp !
        '''Fait tourner sur place la tortue courante de a degrés sur sa droite.'''
        self.cap = (self.cap - a) % 360
    
    def setheading(self,a):     # a en degrés svp !
        '''Fait tourner sur place la tortue courante en direction du cap absolu a degrés.'''
        self.cap = a % 360

    def goto(self,pt):          # t.goto((x,y)). TODO : ajouter t.goto(x,y)
        '''Déplace la tortue courante au point (x,y) et laisse une trace si 
        le crayon est baissé.'''
        (x,y) = pt
        if self.pen:
            (xt_posn,yt_posn) = tortue_posn(self.x,self.y)
            (x_posn,y_posn) = tortue_posn(x,y)
            line(xt_posn,yt_posn,x_posn,y_posn)      # primitive Processing
        self.x = x
        self.y = y
        
    def forward(self,d):
        '''La tortue courante avance de d pixels en laissant une trace si
        le crayon est baissé.'''
        angle = radians(self.cap)    # radians est une primitive Processing
        self.goto((self.x + d * cos(angle), self.y + d * sin(angle)))
        
    def toward(self,pt):
        '''Force la tortue à prendre le cap en direction du point pt.'''
        (x,y) = pt
        (xt,yt) = self.pos()
        (dx, dy) = (xt - x, yt - y)
        if dx == 0:
            angle = 90 if dy < 0 else -90
        elif dy == 0:
            angle = 0 if dx < 0 else 180
        else :
            angle = degrees(atan2(dy,dx)) + 180   # degrees, atan2 : primitives Processing
        self.cap = angle
    
    def init(self,pos,cap):
        '''Déplace la tortue courante, sans tracer, à la position pos et lui fait
        prendre le cap cap. Laisse le crayon baissé.'''
        self.up() ; self.goto(pos) ; self.down() ; self.setheading(cap)
