#!/usr/bin/env python3
# -*- coding: utf-8 -*-

######################################################################
### PYTHON. APPRENTISSAGE ACTIF (JP. ROY, EDITIONS ELLIPSES, 2020) ###
######################################################################

"""
exo_tail_recursion.py (11.132)
d'après https://chrispenner.ca/posts/python-tail-recursion
"""

class Recurse(Exception):
    def __init__(self, *args, **kwargs):
        self.args = args
        self.kwargs = kwargs

def recurse(*args, **kwargs):
    raise Recurse(*args, **kwargs)

# L'appel à recurse provoque une exception qui - au lieu d'empiler
# inutilement les nouveaux arguments - fait mourir l'ancien appel à f
# et installe à la place un nouvel appel à f avec les nouveaux arguments.
# Cela ne vaut pas la vraie gestion "propre" de la récursivité terminale
# comme le langage Scheme l'a théorisée, mais l'idée est là...
# ...et exploitable.

def tail_recursive(f):
    def decorated(*args, **kwargs):
        while True:
            try:
                return f(*args, **kwargs)
            except Recurse as r:
                args = r.args
                kwargs = r.kwargs
                continue
    return decorated            # le résultat est une nouvelle fonction

if __name__ == '__main__' :
    # from tail_recursion import tail_recursive, recurse
    # Pas de profondeur maximale de récursion...

    @tail_recursive             # <==> fac_quasi = tail_recursive(fac_quasi)
    def facquasi(n, acc=1):     # quasi-iterative "<==>" iterative !
        if n == 0: return acc
        recurse(n-1, acc*n)     # provoque une exception Recurse(n-1,acc*n)

    # chrono de la version fac_quasi
    from time import time
    start = time() ; n = facquasi(50_000) ; end = time()
    print('50_000! contient',len(str(n)),'chiffres.')
    print('Recursion time = {:.2f} sec'.format(end-start))

    # chrono de la version vraiment itérative
    start = time()
    res = 1
    for i in range(2,50_000):
       res *= i
    end = time()
    print('Iteration time = {:.2f} sec'.format(end-start))

    # la différence n'est pas énorme...

